#############################################
#####  Visualisation for LAMA output #####
#############################################
###################################################
#### Visualisation and analysis of diary results ####
###################################################

##################### Preparation #######################

#### Set local directories ####
# change these to your system paths

# working directory where the outputs are
setwd("C:/LAMA/output")

# filedirectory to save plots
resdir = "D:/HEIMTSA/MegaCase/results/"

# choose file with diary results (..._diaries.csv)
# WARNING: Do Not Try To Load Files Significantly Larger Than 200 MB!
datad = read.csv(file.choose())
# columns: group, diary_id, ETS, biomass, i_1-...

# add information about the country you selected
# change if necessary
country="Finland"
countryID="FI"
it=1000
#########

# read file with outdoor concentration (concentration.csv)
con = read.csv(file.choose(),sep=";")
# read file with population to identify the EMEP cells that belong to this country
pop = read.csv(file.choose())

# get population and concentration for the specific country
popc = subset(pop,
	CountryID==factor(countryID, levels=levels(pop$CountryID))&
	Subgroup==Subgroup[1]&Year==Year[1]&Rate==Rate[1])
con$Emep50_i_j = paste(con$Emep_i,"_",con$Emep_j,sep="")
conc = merge(con,popc,by="Emep50_i_j")

# calculate means for subsets of smokers and non-smokers
m.all = mean(as.vector(as.matrix(subset(datad,select=5:(it+4)))))
m.bg = mean(as.vector(as.matrix(subset(datad,ETS==0&biomass==0,select=5:(it+4)))))
m.ets = mean(as.vector(as.matrix(subset(datad,ETS==1,select=5:(it+4)))))

# build subgroup files
exp.14f = subset(datad, group=="F15", select=5:(it+4))
exp.14m = subset(datad, group=="M15", select=5:(it+4))
exp.workf = subset(datad, group=="Fwork", select=5:(it+4))
exp.workm = subset(datad, group=="Mwork", select=5:(it+4))
exp.nonworkf = subset(datad, group=="Fnonwork", select=5:(it+4))
exp.nonworkm = subset(datad, group=="Mnonwork", select=5:(it+4))
exp.65f = subset(datad, group=="F65", select=5:(it+4))
exp.65m = subset(datad, group=="M65", select=5:(it+4))

exp.14f.bg = subset(datad, group=="F15"&ETS==0&biomass==0, select=5:(it+4))
exp.14m.bg = subset(datad, group=="M15"&ETS==0&biomass==0, select=5:(it+4))
exp.workf.bg = subset(datad, group=="Fwork"&ETS==0&biomass==0, select=5:(it+4))
exp.workm.bg = subset(datad, group=="Mwork"&ETS==0&biomass==0, select=5:(it+4))
exp.nonworkf.bg = subset(datad, group=="Fnonwork"&ETS==0&biomass==0, select=5:(it+4))
exp.nonworkm.bg = subset(datad, group=="Mnonwork"&ETS==0&biomass==0, select=5:(it+4))
exp.65f.bg = subset(datad, group=="F65"&ETS==0&biomass==0, select=5:(it+4))
exp.65m.bg = subset(datad, group=="M65"&ETS==0&biomass==0, select=5:(it+4))

exp.14f.ets = subset(datad, group=="F15"&ETS==1, select=5:(it+4))
exp.14m.ets = subset(datad, group=="M15"&ETS==1, select=5:(it+4))
exp.workf.ets = subset(datad, group=="Fwork"&ETS==1, select=5:(it+4))
exp.workm.ets = subset(datad, group=="Mwork"&ETS==1, select=5:(it+4))
exp.nonworkf.ets = subset(datad, group=="Fnonwork"&ETS==1, select=5:(it+4))
exp.nonworkm.ets = subset(datad, group=="Mnonwork"&ETS==1, select=5:(it+4))
exp.65f.ets = subset(datad, group=="F65"&ETS==1, select=5:(it+4))
exp.65m.ets = subset(datad, group=="M65"&ETS==1, select=5:(it+4))

exp.14f.bio = subset(datad, group=="F15"&ETS==0&biomass==1, select=5:(it+4))
exp.14m.bio = subset(datad, group=="M15"&ETS==0&biomass==1, select=5:(it+4))
exp.workf.bio = subset(datad, group=="Fwork"&ETS==0&biomass==1, select=5:(it+4))
exp.workm.bio = subset(datad, group=="Mwork"&ETS==0&biomass==1, select=5:(it+4))
exp.nonworkf.bio = subset(datad, group=="Fnonwork"&ETS==0&biomass==1, select=5:(it+4))
exp.nonworkm.bio = subset(datad, group=="Mnonwork"&ETS==0&biomass==1, select=5:(it+4))
exp.65f.bio = subset(datad, group=="F65"&ETS==0&biomass==1, select=5:(it+4))
exp.65m.bio = subset(datad, group=="M65"&ETS==0&biomass==1, select=5:(it+4))


##################### subgroup curves #######################
# plot curves for all subgroups
plot(quantile(as.vector(as.matrix(exp.14f)),seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col="white", xlim=c(0,m.ets), xlab="PM2.5 exposure [g/m]", ylab="cumulative percentiles")
lines(quantile(as.vector(as.matrix(exp.14f)),seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=1, lwd="2")
lines(quantile(as.vector(as.matrix(exp.14m)),seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=2, lwd="2")
lines(quantile(as.vector(as.matrix(exp.workf)),seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=3, lwd="2")
lines(quantile(as.vector(as.matrix(exp.workm)),seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=4, lwd="2")
lines(quantile(as.vector(as.matrix(exp.nonworkf)),seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=5, lwd="2")
lines(quantile(as.vector(as.matrix(exp.nonworkm)),seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=6, lwd="2")
lines(quantile(as.vector(as.matrix(exp.65f)),seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=7, lwd="2")
lines(quantile(as.vector(as.matrix(exp.65m)),seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=8, lwd="2")
lines(c(mean(conc$Pol_Value ),mean(conc$Pol_Value )),c(0,1), col=8, lwd=2, lty=3)
lines(c(m.all,m.all),c(0,1), col=1, lwd=2, lty=3)
title(paste("Exposure of population subgroups to PM2.5 in",country,"\n all individuals, including ETS and biomass"))
legend("bottomright", legend=c("F14","M14","Fwork","Mwork","Fnonwork","Mnonwork","F65","M65","Av Exp","Av Con"), lty=c(rep(1,8),3,3), col=c(1:8,1,8), title="Population groups", bg="white")
savePlot(paste(resdir,countryID,"_all_",it,".png",sep=""),type="png")

# plot curves for all subgroups without ETS and biomass
plot(quantile(as.vector(as.matrix(exp.14f.bg)),seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col="white", xlim=c(0,m.ets), xlab="PM2.5 exposure [g/m]", ylab="cumulative percentiles")
lines(quantile(as.vector(as.matrix(exp.14f.bg)),seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=1, lwd="2")
lines(quantile(as.vector(as.matrix(exp.14m.bg)),seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=2, lwd="2")
lines(quantile(as.vector(as.matrix(exp.workf.bg)),seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=3, lwd="2")
lines(quantile(as.vector(as.matrix(exp.workm.bg)),seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=4, lwd="2")
lines(quantile(as.vector(as.matrix(exp.nonworkf.bg)),seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=5, lwd="2")
lines(quantile(as.vector(as.matrix(exp.nonworkm.bg)),seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=6, lwd="2")
lines(quantile(as.vector(as.matrix(exp.65f.bg)),seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=7, lwd="2")
lines(quantile(as.vector(as.matrix(exp.65m.bg)),seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=8, lwd="2")
lines(c(mean(conc$Pol_Value ),mean(conc$Pol_Value )),c(0,1), col=8, lwd=2, lty=3)
lines(c(m.bg,m.bg),c(0,1), col=1, lwd=2, lty=3)
title(paste("Exposure of population subgroups to PM2.5 in",country,"\n without ETS and biomass exposed individuals"))
legend("bottomright", legend=c("F14","M14","Fwork","Mwork","Fnonwork","Mnonwork","F65","M65","Av Exp","Av Con"), lty=c(rep(1,8),3,3), col=c(1:8,1,8), title="Population groups", bg="white")
savePlot(paste(resdir,countryID,"_bg_",it,".png",sep=""),type="png")

# plot curves for all subgroups only ETS
plot(quantile(as.vector(as.matrix(exp.14f.ets)),seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col="white", xlim=c(0,m.ets), xlab="PM2.5 exposure [g/m]", ylab="cumulative percentiles")
lines(quantile(as.vector(as.matrix(exp.14f.ets)),seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=1, lwd="2")
lines(quantile(as.vector(as.matrix(exp.14m.ets)),seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=2, lwd="2")
lines(quantile(as.vector(as.matrix(exp.workf.ets)),seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=3, lwd="2")
lines(quantile(as.vector(as.matrix(exp.workm.ets)),seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=4, lwd="2")
lines(quantile(as.vector(as.matrix(exp.nonworkf.ets)),seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=5, lwd="2")
lines(quantile(as.vector(as.matrix(exp.nonworkm.ets)),seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=6, lwd="2")
lines(quantile(as.vector(as.matrix(exp.65f.ets)),seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=7, lwd="2")
lines(quantile(as.vector(as.matrix(exp.65m.ets)),seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=8, lwd="2")
lines(c(mean(conc$Pol_Value ),mean(conc$Pol_Value )),c(0,1), col=8, lwd=2, lty=3)
lines(c(m.ets,m.ets),c(0,1), col=1, lwd=2, lty=3)
title(paste("Exposure of population subgroups to PM2.5 in",country,"\n only ETS exposed individuals"))
legend("bottomright", legend=c("F14","M14","Fwork","Mwork","Fnonwork","Mnonwork","F65","M65","Av Exp","Av Con"), lty=c(rep(1,8),3,3), col=c(1:8,1,8), title="Population groups", bg="white")
savePlot(paste(resdir,countryID,"_ets_",it,".png",sep=""),type="png")


########################## subgroup boxplots #######################

# build dataframe with one column for groups and one for all results
# combine results
e = c(as.vector(as.matrix(exp.14f)), as.vector(as.matrix(exp.14m)), 
	as.vector(as.matrix(exp.workf)), as.vector(as.matrix(exp.workm)),
	as.vector(as.matrix(exp.nonworkf)), as.vector(as.matrix(exp.nonworkm)),
	as.vector(as.matrix(exp.65f)), as.vector(as.matrix(exp.65m)))
g = c(rep(1,length(exp.14f[,1])*it),rep(2,length(exp.14m[,1])*it),
	rep(3,length(exp.workf[,1])*it),rep(4,length(exp.workm[,1])*it),
	rep(5,length(exp.nonworkf[,1])*it),rep(6,length(exp.nonworkm[,1])*it),
	rep(7,length(exp.65f[,1])*it),rep(8,length(exp.65m[,1])*it))
gr = c(rep("F14",length(exp.14f[,1])*it),rep("M14",length(exp.14m[,1])*it),
	rep("Fwork",length(exp.workf[,1])*it),rep("Mwork",length(exp.workm[,1])*it),
	rep("Fnonwork",length(exp.nonworkf[,1])*it),rep("Mnonwork",length(exp.nonworkm[,1])*it),
	rep("F65",length(exp.65f[,1])*it),rep("M65",length(exp.65m[,1])*it))
results = data.frame(exp=e,groups=gr,gno=g)

# plot results
# boxplots for all subgroups (THIS CAN TAKE CONSIDERABLE TIME AND RESOURCES!)
boxplot(exp~gno,results,ylim=c(0,m.ets),xlab= "Population groups",ylab= "Exposure in g/m", names=c("F14","M14","Fwork","Mwork","Fnonwork","Mnonwork","F65","M65"))
lines(c(0,10), c(m.all,m.all), col=1, lwd="2", lty=3)
lines(c(0,10), c(mean(conc$Pol_Value),mean(conc$Pol_Value)), col=8,lwd="2", lty=3)
title(paste("Exposure of population subgroups to PM2.5 in",country))
savePlot(paste(resdir,countryID,"_boxplot_all_",it,".png",sep=""),type="png")

################### subgroup average confidence intervals ###################
# plot confidence intervals for means
# calculate condfidence intervals
m = 1:8
s = 1:8
se = 1:8
ciL = 1:8
ciU = 1:8
ciVarL = 1:8
ciVarU = 1:8
no = 1:8

newgroups<- cbind(1:8, c("F14","M14","Fwork","Mwork","Fnonwork","Mnonwork","F65","M65"))
subset(datad, group=="F15", select=5:(it+4))

for(i in 1:8){
	# parameters for demographic groups
	gr <- results$exp[results$gno==i]
	m[i] <- mean(gr)
	s[i] <- sd(gr)
	no[i] <- length(gr)
	se[i] <- s[i]/sqrt(no[i])
	ciL[i] <- m[i]-2*se[i]
	ciU[i] <- m[i]+2*se[i]
}
l=10
u=m.ets

plot(1:8, m, ylim=range(c(l,u)), type='p', xlab="Demographic groups", ylab="Exposure", pch=19,axes=FALSE,ann=FALSE)
axis(1,at=1:8,label=c("F14","M14","Fwork","Mwork","Fnonwork","Mnonwork","F65","M65"))
axis(2, at=seq(min(l),max(u),2))
box()
arrows(1:8, ciL, 1:8, ciU, length = 0.05, angle=90, code=3) 
lines(c(0,10), c(m.all,m.all), col="red",lwd="1")
title("Confidence intervals for average exposure per demographic group")
title(xlab= "Population groups")
title(ylab= "Exposure in g/m")
savePlot(paste(resdir,countryID,"_CI_all_",it,".png",sep=""),type="png")

##################### population realisations #####################

# this is the uncertainty/variability separation plot
# calcuate means for each realisation for background Exposure only
exp.bg = subset(datad,ETS==0&biomass==0)
mean.r.bg <- apply(exp.bg[,5:(it+4)],2,mean)

# plot curves for all subgroups only ETS
plot(quantile(exp.bg[,(order(mean.r.bg)[(it-5)]+4)],seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col="white", xlim=c(0,quantile(mean.r.bg,0.98)), xlab="PM2.5 exposure [g/m]", ylab="cumulative percentiles")
lines(quantile(exp.bg[,(order(mean.r.bg)[5]+4)],seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=1, lwd="2")
lines(quantile(exp.bg[,(order(mean.r.bg)[(it*0.1)]+4)],seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=1, lwd="2")
lines(quantile(exp.bg[,(order(mean.r.bg)[(it*0.25)]+4)],seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=1, lwd="2")
lines(quantile(exp.bg[,(order(mean.r.bg)[(it*0.5)]+4)],seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=1, lwd="2")
lines(quantile(exp.bg[,(order(mean.r.bg)[(it*0.75)]+4)],seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=1, lwd="2")
lines(quantile(exp.bg[,(order(mean.r.bg)[(it*0.9)]+4)],seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=1, lwd="2")
lines(quantile(exp.bg[,(order(mean.r.bg)[(it*0.95)]+4)],seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=1, lwd="2")
lines(quantile(as.vector(as.matrix(exp.bg[,5:(it+4)])),seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=4, lwd="4")
title(paste("Realisations for population distributions of PM2.5 exposure in",country,"\n without ETS and biomass"))
savePlot(paste(resdir,countryID,"_realisations_bg_",it,".png",sep=""),type="png")


#################### percentiles #######################

# calculate 5 %, 50 % and 95 % values for curves for whole population
# get 0.05, 0.5 and 0.95 percentiles for realisations for each individual
q = apply(subset(datad,select=i_1:i_1000),1,quantile,probs=c(0.05,0.5,0.95))
quants = data.frame(q005=q[1,],q050=q[2,],q095=q[3,])

# percentiles for individuals without ETS and biomass
q.bg = apply(subset(datad,ETS==0&biomass==0,select=i_1:i_1000),1,quantile,probs=c(0.05,0.5,0.95))
quants.bg = data.frame(q005=q.bg[1,],q050=q.bg[2,],q095=q.bg[3,])

# percentiles for working females without ETS and biomass
q.bg.fwork = apply(subset(datad,ETS==0&biomass==0&group=="Fwork",select=i_1:i_1000),1,quantile,probs=c(0.05,0.5,0.95))
quants.bg.fwork = data.frame(q005=q.bg.fwork[1,],q050=q.bg.fwork[2,],q095=q.bg.fwork[3,])

# get ECDF to calculate probabilities of exceeding a threshold
#e.bg = ecdf(quants.bg$q050)
#1-e.bg(10)	# example for 10 g/m3

# plot percentile curves
plot(quantile(quants$q095,seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col="white", xlim=c(5,(m.ets*2)), xlab="PM2.5 exposure", ylab="probability")
axis(1, at=seq(0,(m.ets*2),10))
#axis(2, at=seq(0,1,0.1))
lines(quantile(quants$q005,seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=3, lwd="3")
lines(quantile(quants$q050,seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=2, lwd="3")
lines(quantile(quants$q095,seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=4, lwd="3")
lines(c(mean(conc$Pol_Val),mean(conc$Pol_Val)),c(0,1), col=8, lwd=2, lty=2)
title(paste("Percentiles for exposure of the population  to PM2.5 in",country,"\n all individuals, including ETS and biomass"))
legend("bottomright", legend=c("5th percentile","50th percentile","95th percentile","Av Con"), lty=c(1,1,1,2),col=c(3,2,4,8), title="Population groups", bg="white")
savePlot(paste(resdir,countryID,"_percentiles_all_",it,".png",sep=""),type="png")

# plot for background exposure
plot(quantile(quants.bg$q095,seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col="white", xlim=c(0,m.all), xlab="PM2.5 exposure", ylab="probability")
axis(1, at=seq(0,m.all,5))
lines(quantile(quants.bg$q005,seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=3, lwd="3")
lines(quantile(quants.bg$q050,seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=2, lwd="3")
lines(quantile(quants.bg$q095,seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=4, lwd="3")
lines(c(mean(conc$Pol_Val),mean(conc$Pol_Val)),c(0,1), col=8, lwd=2, lty=2)
title(paste("Percentiles for exposure of the population  to PM2.5 in",country,"\n without ETS and biomass exposed individuals"))
legend("bottomright", legend=c("5th percentile","50th percentile","95th percentile","Av Con"), lty=c(1,1,1,2),col=c(3,2,4,8), title="Population groups", bg="white")
savePlot(paste(resdir,countryID,"_percentiles_bg_",it,".png",sep=""),type="png")

# plot for working females only
plot(quantile(quants$q095[datad$group=="Fwork"],seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col="white", xlim=c(5,(m.ets*2)), xlab="PM2.5 exposure", ylab="probability")
axis(1, at=seq(0,(m.ets*2),10))
#axis(2, at=seq(0,1,0.1))
lines(quantile(quants$q005[datad$group=="Fwork"],seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=3, lwd="3")
lines(quantile(quants$q050[datad$group=="Fwork"],seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=2, lwd="3")
lines(quantile(quants$q095[datad$group=="Fwork"],seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=4, lwd="3")
lines(c(mean(conc$Pol_Val),mean(conc$Pol_Val)),c(0,1), col=8, lwd=2, lty=2)
title(paste("Percentiles for exposure of the population  to PM2.5 in",country,"\n working females, including ETS and biomass"))
legend("bottomright", legend=c("5th percentile","50th percentile","95th percentile","Av Con"), lty=c(1,1,1,2),col=c(3,2,4,8), title="Population groups", bg="white")
savePlot(paste(resdir,countryID,"_percentiles_fwork_",it,".png",sep=""),type="png")

# background plot for working females only
plot(quantile(quants.bg.fwork$q095,seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col="white", xlim=c(5,m.all), xlab="PM2.5 exposure", ylab="probability")
axis(1, at=seq(0, m.all,10))
#axis(2, at=seq(0,1,0.1))
lines(quantile(quants.bg.fwork$q005,seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=3, lwd="3")
lines(quantile(quants.bg.fwork$q050,seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=2, lwd="3")
lines(quantile(quants.bg.fwork$q095,seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=4, lwd="3")
lines(c(mean(conc$Pol_Val),mean(conc$Pol_Val)),c(0,1), col=8, lwd=2, lty=2)
title(paste("Percentiles for exposure of the population  to PM2.5 in",country,"\n working females, without ETS and biomass exposed individuals"))
legend("bottomright", legend=c("5th percentile","50th percentile","95th percentile","Av Con"), lty=c(1,1,1,2),col=c(3,2,4,8), title="Population groups", bg="white")
savePlot(paste(resdir,countryID,"_percentiles_fwork_bg_",it,".png",sep=""),type="png")


#################### plot individuals ######################

# calculate average exposures over all simulation for each individuals
expo = apply(exp.workf.bg, 1, mean)

# get min and may individual
min = subset(exp.workf.bg,expo==min(expo))
max = subset(exp.workf.bg,expo==max(expo))

# get two pairs of individuals, one with difference and randomly picked
dif = cbind(c(as.vector(as.matrix(min[1,1:it])),as.vector(as.matrix(max[1,1:it]))),
	c(rep(1,it),rep(2,it)))
ran = cbind(c(as.vector(as.matrix(exp.workf.bg[5,1:it])),as.vector(as.matrix(exp.workf.bg[2,1:it]))),
	c(rep(1,it),rep(2,it)))

# plot individual exposure as boxplots
boxplot(dif[,1]~dif[,2],ylim=c(0,100),xlab= "Individual",ylab= "PM2.5 exposure in g/m", names=c("x","y"))
savePlot(paste(resdir,countryID,"_individualExp_differences.png",sep=""),type="png")
boxplot(ran[,1]~ran[,2],ylim=c(0,40),xlab= "Individual",ylab= "PM2.5 exposure in g/m", names=c("x","y"))
savePlot(paste(resdir,countryID,"_individualExp_similar.png",sep=""),type="png")

