#############################################
#####  Visualisation for LAMA output #####
#############################################
###################################################
#### Visualisation and analysis of subgroup results ####
###################################################


##################### Preparation #######################
#### Set local directories ####
# change these to your system paths
LAMA.folder = "C:/LAMA"
#resdir = "C:/LAMA/output/"

# read scenario and country table
scenarios = read.csv(paste(LAMA.folder,"/input/metadata/scenario.csv",sep=""))
countries = read.csv(paste(LAMA.folder,"/input/metadata/country.csv",sep=""))

# add information about the country and scenario you selected
# change if necessary
c = 10
s = 1
it=10000

# get values
scenario=as.character(scenarios$scenario_type[s])
country=as.character(countries$Country_name[c])
countryID=as.character(countries$ISO2[c])

# show scenario and country values
print(paste("Selected country:",country))
print(paste("Selected scenario:",scenario))

# set working directory
setwd(paste("D:/HEIMTSA/MegaCase/results/",scenario, sep=""))

# get LAMA files for all countries
#for(i in 1:30){
#	countryID=as.character(countries$ISO2[i])
#	print(paste("Parsing now country:",countryID))
#	data.c = read.csv(paste(LAMA.folder,"/output/",
#		scenario,"/country/",countryID,"_",scenario,
#		"_PM2.5_i",it,"_subgroups.csv",sep=""))
#	if(i==1){
#		data = data.c[,round(runif(500,2,(it+1)))]
#	}
#	else{
#		# add sample of 500 to data.frame
#		data = cbind(data,data.c[,round(runif(500,2,(it+1)))])
#	}
#}

# get data
# e.g. AT_2010BAU_PM2.5_i10000_subgroups
data.c = read.csv(paste(LAMA.folder,"/output/",
		scenario,"/country/",countryID,"_",scenario,
		"_PM2.5_i",it,"_subgroups.csv",sep=""))
subgroups = as.character(data.c[,1])

# read file with outdoor concentration (concentration.csv)
con = read.csv(paste(LAMA.folder,"/input/data/concentration.csv",sep=""))
# read file with population to identify the EMEP cells that belong to this country
pop = read.csv(paste(LAMA.folder,"/input/data/population.csv",sep=""))

# get population and concentration for the specific country
popc = subset(pop,
	CountryID==factor(countryID, levels=levels(pop$CountryID))&
	Subgroup==Subgroup[1]&Year==Year[1]&Rate==Rate[1])
conc = merge(con,popc,by.x="Emep_i_j",by.y="Emep50_i_j")

# get population and concentration for EU
#popc = subset(pop, Subgroup==Subgroup[1]&Year==Year[1]&Rate==Rate[1])
#con$Emep50_i_j = paste(con$Emep_i,"_",con$Emep_j,sep="")
#conc = merge(con,popc,by="Emep50_i_j")

# calculate means for subsets of smokers and non-smokers
m.all = mean(as.vector(as.matrix(subset(data.c,select=2:(it+1)))))

# clear unused concentration and population data 
rm(con, pop)
#########

# transpose data
data.trans = data.frame("F15"=data.c[1,2:length(data.c[1,])],
	"M15"=data.c[2,2:length(data.c[1,])],"Fwork"=data.c[3,2:length(data.c[1,])],
	"Mwork"=data.c[4,2:length(data.c[1,])],"Fnonwork"=data.c[5,2:length(data.c[1,])],
	"Mnonwork"=data.c[6,2:length(data.c[1,])],"F65"=data.c[7,2:length(data.c[1,])],
	"M65"=data.c[8,2:length(data.c[1,])])

boxplot(data.trans, ylim=c(0,80),xlab= "Population subgroups",ylab= "PM2.5 Exposure in g/m", names=subgroups)
savePlot(paste(countryID,"_boxplot_",it,".png",sep=""),type="png")

##################### Plots #######################
# plot curves for all subgroups
plot(quantile(as.vector(as.matrix(data.c[1,2:(it+1)])),seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col="white", type="l",xlim=c(0,80),xlab="PM2.5 exposure [g/m]", ylab="cumulative percentiles")
lines(quantile(as.vector(as.matrix(data.c[1,2:(it+1)])),seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=1, lwd="2",type="l")
lines(quantile(as.vector(as.matrix(data.c[2,2:(it+1)])),seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=2, lwd="2",type="l")
lines(quantile(as.vector(as.matrix(data.c[3,2:(it+1)])),seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=3, lwd="2",type="l")
lines(quantile(as.vector(as.matrix(data.c[4,2:(it+1)])),seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=4, lwd="2",type="l")
lines(quantile(as.vector(as.matrix(data.c[5,2:(it+1)])),seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=5, lwd="2",type="l")
lines(quantile(as.vector(as.matrix(data.c[6,2:(it+1)])),seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=6, lwd="2",type="l")
lines(quantile(as.vector(as.matrix(data.c[7,2:(it+1)])),seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=7, lwd="2",type="l")
lines(quantile(as.vector(as.matrix(data.c[8,2:(it+1)])),seq(0.01,0.99,0.01)),seq(0.01,0.99,0.01), col=8, lwd="2",type="l")
lines(c(mean(conc$Pol_Value),mean(conc$Pol_Value)),c(0,1), col=8, lwd=2, lty=3)
lines(c(m.all,m.all),c(0,1), col=1, lwd=2, lty=3)
#title(paste("Exposure of population subgroups to PM2.5 in",country,"\n all individuals, including ETS and biomass"))
legend("bottomright", legend=c("F15","M15","Fwork","Mwork","Fnonwork","Mnonwork","F65","M65","Av Exp","Av Con"), lty=c(rep(1,8),3,3), col=c(1:8,1,8), title="Population groups", bg="white")
savePlot(paste(countryID,"_subgroupcurves_",it,".png",sep=""),type="png")



