--FILE CREATES TABLE STRUCTURE OF DATABASE

-- Dropping Tables
----------------------
DROP TABLE IF EXISTS simulation CASCADE;
DROP TABLE IF EXISTS parameter_realisation CASCADE;
DROP TABLE IF EXISTS exposure_country_realisation CASCADE;
DROP TABLE IF EXISTS exposure_grid_realisation CASCADE;


-----------------------------------------------------------------------------------------------------------------------
-- Creating the tables
-----------------------------------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------------------------
-- tables for storing results
-----------------------------------------------------------------------------------------------------------------------

-- Table: simulation
-- represents LAMA simulations
CREATE TABLE simulation
(
  simulation_id varchar(100) NOT NULL,
  pollutant_id varchar(100),
  country_id varchar(30),
  scenario_id smallint,
  grid_based smallint,
  iterations integer,
  description varchar(300),
  PRIMARY KEY (simulation_id)
);

-- Table: parameter_realisation
-- represents realisations for parameters
CREATE TABLE parameter_realisation
(
  parameter_realisation_id SERIAL,
  simulation_id varchar(100) NOT NULL,
  parameter_id varchar(100) NOT NULL,
  microenvironment_id varchar(100),
  iteration integer,
  value numeric,
  PRIMARY KEY(parameter_realisation_id)
);

-- Table: exposure_country_realisation
-- represents realisations for individual exposure
CREATE TABLE exposure_country_realisation
(
  exposure_country_realisation_id SERIAL,
  simulation_id varchar(100) NOT NULL,
  diary_id varchar(50) NOT NULL,
  group_id varchar(100),
  iteration integer,
  value numeric,
  PRIMARY KEY(exposure_country_realisation_id)
);

-- Table: exposure_grid_realisation
-- represents realisations for subgroup exposure on a grid
CREATE TABLE exposure_grid_realisation
(
  exposure_grid_realisation_id SERIAL,
  simulation_id varchar(100) NOT NULL,
  emep50_id varchar(30),
  group_id varchar(100),
  iteration integer,
  value numeric,
  PRIMARY KEY(exposure_grid_realisation_id)
);

-----------------------------------------------------------------------------------------------------------------------
-- add references and foreign keys
-----------------------------------------------------------------------------------------------------------------------
--foreign keys for simulation table
ALTER TABLE simulation ADD FOREIGN KEY (scenario_id) REFERENCES scenario(scenario_id) ON UPDATE CASCADE;
ALTER TABLE simulation ADD FOREIGN KEY (pollutant_id) REFERENCES pollutant(pollutant_id) ON UPDATE CASCADE;
ALTER TABLE simulation ADD FOREIGN KEY (country_id) REFERENCES country(country_id) ON UPDATE CASCADE;

--foreign keys for parameter_realisation table
ALTER TABLE parameter_realisation  ADD FOREIGN KEY (parameter_id) REFERENCES parameter(parameter_id)  ON UPDATE CASCADE;
ALTER TABLE parameter_realisation  ADD FOREIGN KEY (microenvironment_id) REFERENCES microenvironment(microenvironment_id)  ON UPDATE CASCADE;
ALTER TABLE parameter_realisation  ADD FOREIGN KEY (simulation_id) REFERENCES simulation(simulation_id) ON UPDATE CASCADE;

--foreign keys for exposure_country_realisation table
ALTER TABLE exposure_country_realisation  ADD FOREIGN KEY (diary_id) REFERENCES diary(diary_id) ON UPDATE CASCADE;
ALTER TABLE exposure_country_realisation  ADD FOREIGN KEY (group_id) REFERENCES subgroup(group_id) ON UPDATE CASCADE;
ALTER TABLE exposure_country_realisation  ADD FOREIGN KEY (simulation_id) REFERENCES simulation(simulation_id) ON UPDATE CASCADE;

--foreign keys for exposure_grid_realisation table
ALTER TABLE exposure_grid_realisation ADD FOREIGN KEY (emep50_id) REFERENCES emep50_grid(emep50_id) ON UPDATE CASCADE;
ALTER TABLE exposure_grid_realisation ADD FOREIGN KEY (group_id) REFERENCES subgroup(group_id) ON UPDATE CASCADE;
ALTER TABLE exposure_grid_realisation ADD FOREIGN KEY (simulation_id) REFERENCES simulation(simulation_id) ON UPDATE CASCADE;

