package de.ifgi.exposure.datamodel;

import java.util.ArrayList;

public class EMEPgridCountry{
	
	/**
	 * Class to store concentration or population data from EMEP grid for one country
	 */
	private ArrayList<Float> concentration, urban_increment, urban_population_percentage;
	private ArrayList<Float[]> population;
	private Float[] subgroup_pop;
	private ArrayList<String> cellID;
	
	private String id, meta;

	public EMEPgridCountry(String countryid){ 
		this.id = countryid;
		this.concentration = new ArrayList<Float>();
		this.population = new ArrayList<Float[]>();
		this.urban_increment = new ArrayList<Float>();
		this.urban_population_percentage= new ArrayList<Float>();
		this.cellID = new ArrayList<String>();
		subgroup_pop = new Float[8];
	}

	public void next_cell(){
		// add subgroup population to ArrayList
		population.add(subgroup_pop);
		
		// new subgroup array for next cell
		subgroup_pop = new Float[8];
	}
	
	// **** SETTERS ****	
	// SETTERS for individual data
	public void set_countryID(String id){
		this.id = id;
	}
	
	public void set_metadata (String description){
		this.meta = description;
	}
	
	public void set_cellID(String cellID){
		this.cellID.add(cellID);
	}
	
	public void set_populationValue(Float val, int groupindex){
		subgroup_pop[groupindex] = val;
	}
	
	public void set_populationGroup(Float[] group){
		population.add(group);
	}
	
	public void set_concentrationValue(Float val){
		concentration.add(val);
	}
	
	public void set_urban_incrementValue(Float val){
		urban_increment.add(val);
	}
	
	public void set_urban_populationPercentage(Float val){
		urban_population_percentage.add(val);
	}
	
	// SETTERS for ArrayLists
	public void set_cellIDArray(ArrayList<String> list){
		this.cellID = list;
	}
	
	public void set_populationArray(ArrayList<Float[]> list){
		this.population = list;
	}
	
	public void set_concentrationArray(ArrayList<Float> list){
		this.concentration = list;
	}
	
	// **** GETTERS ****
	// GETTERS for individual data
	public int get_size(){
		return cellID.size();
	}
	
	public String get_countryID(){
		return id;
	}
	
	public String get_metadata(){
		return meta;
	}
	
	public String get_cellID(int index){
		return cellID.get(index);
	}
	
	public Float[] get_populationGroups(int index){
		return population.get(index);
	}
	
	public Float get_population(int index, int groupindex){
		return population.get(index)[groupindex];
	}
	
	public Float get_concentration(int index){
		return concentration.get(index);
	}
	
	public Float get_urban_increment(int index){
		return urban_increment.get(index);
	}
	
	public Float get_urban_population_percentage(int index){
		return urban_population_percentage.get(index);
	}
	
	// GETTERS for ArrayLists
	public ArrayList<String> get_cellIDArray(){
		return cellID;
	}
	
	public ArrayList<Float[]> get_populationArray(){
		return population;
	}
	
	public ArrayList<Float> get_concentrationArray(){
		return concentration;
	}
	
	public ArrayList<Float> get_urban_incrementArray(){
		return urban_increment;
	}
	
	public ArrayList<Float> get_urban_population_percentageArray(){
		return urban_population_percentage;
	}
}
