package de.ifgi.exposure.datamodel;

import java.util.ArrayList;
import de.ifgi.exposure.statistics.Realisations;

/***
 * 
 * Class to store results for one country on a grid basis
 * Data per subgroup + total population
 * Offers statistics
 * @author l_gerh01
 *
 */
public class GridResults {

	private String country_id;
	private String[] subgroups;	
	private ArrayList<ArrayList<float[]>> exp_subgroups;
	private ArrayList<float[]> exp_all;
	private ArrayList<int[]> emep_ids;
	
	public GridResults(String country_id, String[] subgroupList, float[][] results, 
			ArrayList<String[]> ids){
		this.country_id = country_id;
		subgroups = subgroupList;	
		
		// fill ArrayLists
		this.split_ids(ids);
		this.split_results(results, ids);
		this.calculate_average();
	}
	
	public GridResults(String country_id, float[][] results, 
			ArrayList<String[]> ids){
		this.country_id = country_id;
		subgroups = new String[1];
		subgroups[0] = "total_pop";
		// fill ArrayLists
		this.split_ids(ids);
		this.split_results(results, ids);
		this.calculate_average();
	}
	
	private void split_ids(ArrayList<String[]> ids){
		emep_ids = new ArrayList<int[]>();	
		// only take every 8th id
		for(int i=0; i<ids.size(); i+=subgroups.length){
			String[] nids = ids.get(i)[0].split("_");
			int[] id_int = new int[2];
			id_int[0] = Integer.parseInt(nids[0]);
			id_int[1] = Integer.parseInt(nids[1]);
			emep_ids.add(id_int);
		}
		//System.out.println("EMEP id size: "+emep_ids.size());
	}
	
	private void split_results(float[][] results, ArrayList<String[]> ids){
		exp_subgroups = new ArrayList<ArrayList<float[]>>();
		// clear existing ArrayLists
//		for(int i=0; i<exp_subgroups.size(); i++){
//			exp_subgroups.remove(i);
//		}
		
		int subLength = subgroups.length;
		// split results matrix into supgroups and calculate population average
		for(int subgroupID=0; subgroupID<subgroups.length; subgroupID++){
			ArrayList<float[]> list = new ArrayList<float[]>();
			
			// fill subgroup arrays with realisation results
			// loop through cellids
			for (int cell=0; cell<emep_ids.size(); cell++){
				int count = cell*subLength+subgroupID;
				//System.out.println(count);
				float[] realisations = results[count];
				list.add(realisations);			
			}
			
			// add subgroup array to ArrayList
			exp_subgroups.add(list);
		}	
	}
	
	private void calculate_average(){
		exp_all = new ArrayList<float[]>();
		int it = exp_subgroups.get(0).get(0).length;
		
		// loop through cells
		for (int cell=0; cell<emep_ids.size(); cell++){
			float[] cell_av = new float[it];
			
			// loop through subgroups
			for(int subgroup=0; subgroup<subgroups.length; subgroup++){
				float[] res = exp_subgroups.get(subgroup).get(cell);
				
				// add value for each realisation
				for(int realisation=0; realisation<res.length; realisation++){
					cell_av[realisation] += res[realisation]/subgroups.length;
				}
			}
			// add averages to ArrayList
			exp_all.add(cell_av);
				
		}
	}
	

	// GETTERS
	public ArrayList<float[]> get_quantiles_List(float[] levels, int subgroup_id){
		ArrayList<float[]> list;
		if(subgroup_id==9)
			list = exp_all;
		else
			list = exp_subgroups.get(subgroup_id);
		ArrayList<float[]> quantileList = new ArrayList<float[]>();
		// loop through cell results
		for(int cell=0; cell<list.size(); cell++){
			float[] quantiles = new float[levels.length];
			Realisations r = new Realisations(list.get(cell));
			for(int l=0; l<levels.length; l++){
				quantiles[l] = r.get_quantile(levels[l]);
			}
			quantileList.add(quantiles);
		}
		return quantileList;
	}
	
	public ArrayList<Float> get_quantiles(float level, int subgroup_id){
		ArrayList<float[]> list;
		if(subgroup_id==9)
			list = exp_all;
		else
			list = exp_subgroups.get(subgroup_id);
		ArrayList<Float> quantileList = new ArrayList<Float>();
		// loop through cell results
		for(int cell=0; cell<list.size(); cell++){
			Realisations r = new Realisations(list.get(cell));
			quantileList.add(r.get_quantile(level));
		}
		return quantileList;
	}
	
	public ArrayList<Float> get_mean(int subgroup_id){
		ArrayList<float[]> list;
		if(subgroup_id==9)
			list = exp_all;
		else
			list = exp_subgroups.get(subgroup_id);
		ArrayList<Float> meanList = new ArrayList<Float>();
		for(int cell=0; cell<list.size(); cell++){
			Realisations r = new Realisations(list.get(cell));
			meanList.add(r.get_mean());
		}
		return meanList;
	}
	
	public ArrayList<Float> get_standard_deviation(int subgroup_id){
		ArrayList<float[]> list;
		if(subgroup_id==9)
			list = exp_all;
		else
			list = exp_subgroups.get(subgroup_id);
		ArrayList<Float> sdList = new ArrayList<Float>();
		for(int cell=0; cell<list.size(); cell++){
			Realisations r = new Realisations(list.get(cell));
			sdList.add(r.get_standard_deviation());
		}
		return sdList;
	}
	
	public ArrayList<int[]> get_EMEP_ids(){
		return emep_ids;
	}
	
	public String get_country_name(){
		return country_id;
	}
}
