package de.ifgi.exposure.datamodel;

import java.util.ArrayList;
import java.util.Hashtable;

/**
 * Class to store parameters for the indoor model
 * @author Lydia Gerharz
 *
 */

public class IndoorParameters {
	// parameters
	private String country_id, me_id;
	private String[] subgroups;
	private Hashtable<String,float[]> parameters;
	private Hashtable<String,String> dist_parameters;
	private Hashtable<String,String[]> meta_parameters;
	
	private ArrayList<String[]> parameter_realisations;
	
	// constructor for distribution parameters
	public IndoorParameters(String country_id, String microenvironment){
		this.country_id = country_id;
		this.me_id = microenvironment;
		parameters = new Hashtable<String,float[]>();
		dist_parameters = new Hashtable<String,String>();
		meta_parameters = new Hashtable<String,String[]>();
		subgroups = new String[9];
	}
	
	public String get_country_id(){
		return country_id;
	}
	
	public String get_me_id(){
		return me_id;
	}
	
	// **** SETTERS ****
	// SETTERS for parameters
	public void set_parameter(String parameter, float[] values){
		parameters.put(parameter, values);
	}
	
	public void update_parameter(String parameter, float val, int index){
		float[] values= parameters.get(parameter);
		values[index] = val;
		parameters.remove(parameter);
		parameters.put(parameter, values);
	}
	
	public void set_parameter_distribution(String parameter, String distribution){
		dist_parameters.put(parameter, distribution);
	}
	
	public void set_parameter_metadata(String parameter, String[] description){
		meta_parameters.put(parameter, description);	
	}
	
	public void set_subgroup(String subgroup, int index){
		subgroups[index] = subgroup;
	}
	
	public void set_parameter_realisations(ArrayList<String[]> array){
		parameter_realisations = array;
	}
	
	// **** GETTERS ****
	public ArrayList<String[]> get_parameter_realisations(){
		return parameter_realisations;
	}
	
	// GETTERS for parameter Hashtables
	public Hashtable<String,float[]> get_parameterHash(){
		return parameters;
	}
	
	public Hashtable<String,String[]> get_parameter_metadataHash(){
		return meta_parameters;
	}
	
	public Hashtable<String,String> get_parameter_distributionHash(){
		return dist_parameters;
	}
	
	public String[] get_subgroups(){
		return subgroups;
	}
	
	// GETTERS for parameters
	public float[] get_parameter(String name){
		return parameters.get(name);
	}
	
	public String get_parameter_distribution(String name){
		return dist_parameters.get(name);
	}
	
	public String[] get_parameter_metadata(String name){
		return meta_parameters.get(name);
	}
	
	public String get_subgroup(int index){
		return subgroups[index];
	}

	
}
