package de.ifgi.exposure.datamodel;
import java.util.ArrayList;
import java.util.Hashtable;

/**
 * Class to store individual time use data for a MTUS country
 * 5 microenvironments: work, home, travel, outdoor, other
 * 
 * @author Lydia Gerharz
 *
 */
public class MTUScountry {

	private String countryID;
	private ArrayList<Integer[]> timePerME, timeSubgroupPerME;
	private String[] diaryME;
	private Integer[] diaryTime;
	private Hashtable<String,String> me2id;
	private Hashtable<String,String> subgroups;
	
	private float[] smoker_realisations, biomass_realisations;
	
	private ArrayList<Integer> age;
	private ArrayList<String> group, sex, emp;
	
	public MTUScountry(String countryID){
		this.countryID = countryID;
		timePerME = new ArrayList<Integer[]>();
		timeSubgroupPerME = new ArrayList<Integer[]>();
		diaryME = new String[5];
		diaryME[0] = "home"; diaryME[1] = "work"; diaryME[2] = "travel"; 
		diaryME[3] = "outdoor"; diaryME[4] = "other";
		me2id = new Hashtable<String,String>();
		me2id.put("home","0"); me2id.put("work","1"); me2id.put("travel","2");
		me2id.put("outdoor","3"); me2id.put("other","4");
		diaryTime = new Integer[5];
		
		group = new ArrayList<String>();
		age = new ArrayList<Integer>();
		sex = new ArrayList<String>();
		emp = new ArrayList<String>();
		
//		subgroups = new Hashtable<String,String>();
//		subgroups.put("F15","1");
//		subgroups.put("M15","2");
//		subgroups.put("Fwork","3");
//		subgroups.put("Mwork","4");
//		subgroups.put("Fnonwork","5");
//		subgroups.put("Mnonwork","6");
//		subgroups.put("F65","7");
//		subgroups.put("M65","8");
//		subgroups.put("","0");
		
	}
	
	public void close_diary(){
		// add diaryTime to ArrayList
		timePerME.add(diaryTime);
		
		// new time array for next diary
		diaryTime = new Integer[5];
	}
	
	
	// **** SETTERS ****
	// SETTERS for individual time use data
	public void set_time(int time, String me){
		diaryTime[Integer.parseInt(me2id.get(me))] = time;
	}
	
	public void set_age(int a){
		age.add(a);	
	}
	
	public void set_sex(String gender){
		if(gender.startsWith("f")||gender.startsWith("F")||gender.startsWith("w")||gender.startsWith("W"))
			sex.add("female");	
		else if(gender.startsWith("m")||gender.startsWith("M"))
			sex.add("male");
		else
			sex.add("unknown");
	}
	
	public void set_employment(String employmentStatus){
		emp.add(employmentStatus);	
	}
	
	public void set_group(String id){
		group.add(id);	
	}
	
	public void set_subgroups(Hashtable<String,String> subgroupHash){
		subgroups = subgroupHash;
		this.calculate_time_per_subgroup();
	}
	
	public void set_microenvironments(Hashtable<String,String> meHash){
		me2id = meHash;
	}
	
	public void set_smokerRealisations(float[] array){
		this.smoker_realisations = array;
	}
	
	public void set_biomassRealisations(float[] array){
		this.biomass_realisations = array;
	}
	
	//**** GETTERS ****
	// GETTERS for name and count
	public String get_countryID(){
		return countryID;
	}
	
	public int get_size(){
		return group.size();
	}
	
	// method to get diary data for database filling
	public String[] get_diaryData(int index){
		String[] data = new String[7];
		// INSERT INTO diary (diary_id, country_id, group_id, age, sex, emp) VALUES ('DE', 'F15', 12, 'female', 'all');	
		data[0] = countryID + "_" + (index+1);
		data[1] = countryID;
		data[2] = group.get(index);
		data[3] = ""+age.get(index);
		data[4] = sex.get(index);
		data[5] = emp.get(index);		
		//TODO: Change hard coded reference
		data[6] = "";
		
		return data;
	}
	
	// GETTERS for subgroup data
	private void calculate_time_per_subgroup(){
		int[][] sum_time_me = new int[subgroups.size()][me2id.size()];
		int[] count_time = new int[subgroups.size()];
		
		// fill subgroup time arrays
		for(int i=0; i<timePerME.size(); i++){
			// add time for each microenvironment
			for(int j=0; j<me2id.size(); j++){
				int t = timePerME.get(i)[j];
				sum_time_me[Integer.parseInt(subgroups.get(group.get(i)))][j] += t;			
			}
			count_time[Integer.parseInt(subgroups.get(group.get(i)))] += 1;
		}

		// calculate average time arrays
		for(int i=0; i<subgroups.size(); i++){
			Integer[] time_me = new Integer[me2id.size()];
			int sum = 0;
			for(int j=0; j<me2id.size(); j++){
				if(count_time[i]>0)
					time_me[j] = sum_time_me[i][j]/count_time[i];
				else
					time_me[j] = 0;
				sum += time_me[j];
			}
		//	System.out.println("Subgroup "+i+": "+sum);
			timeSubgroupPerME.add(time_me);
			
		}
	}
	
	// method to get array with times for all microenvironments in one subgroup
	public Integer[] get_times_subgroup(int subgroup_id){
		return timeSubgroupPerME.get(subgroup_id);
	}
	
	// method to get array with times for one microenvironment in all subgroups
//	public int[] get_times_me(int me_id){
//		// get id of the respective microenvironment
//		int id = Integer.parseInt(me2id.get(me_id));
//		int[] times_me = new int[timeSubgroupPerME.size()];
//		// loop through subgroups to get the respective me time
//		for(int i=0; i<timeSubgroupPerME.size(); i++){
//			times_me[i] = timeSubgroupPerME.get(i)[id];
//		}
//		return times_me;
//	}
	
	public Integer get_time_subgroup(int subgroup_id, String me){
		return timeSubgroupPerME.get(subgroup_id)[Integer.parseInt(me2id.get(me))];
	}
	
	// GETTERS for individual data
	public Integer[] get_times(int index){
		return timePerME.get(index);
	}
	
	public Integer get_time(int index, String me){
		return timePerME.get(index)[Integer.parseInt(me2id.get(me))];
	}
	
	public String get_diaryID(int index){
		return (countryID+ "_" + (index+1));
	} 
	
	public String get_group(int index){
		return group.get(index);
	}
	
	public int get_groupID(int index){
		return Integer.parseInt(subgroups.get(group.get(index)));
	}
	
	public int get_age(int index){
		return age.get(index);
	}
	
	public String get_sex(int index){
		return sex.get(index);
	}
	
	public String get_emp(int index){
		return emp.get(index);
	}
	
	// GETTERS for ArrayLists
	public ArrayList<Integer[]> get_timesArray(){
		return timePerME;
	}
	
	public ArrayList<String> get_groupArray(){
		return group;
	}
	
	public ArrayList<Integer> get_ageArray(){
		return age;
	}
	
	public ArrayList<String> get_sexArray(){
		return sex;
	}
	
	public ArrayList<String> get_empArray(){
		return emp;
	}
	
	public Hashtable<String,String> get_subgroups(){
		return subgroups;
	}
	
	public Hashtable<String,String> get_microenvironments(){
		return me2id;
	}
	
	public float[] get_smokerRealisations(){
		return smoker_realisations;
	}
	
	public float[] get_biomassRealisation(){
		return biomass_realisations;
	}
}
