package de.ifgi.exposure.datamodel;

import java.util.Enumeration;
import java.util.Hashtable;

public class SimulationParameters {

	// default value for folder
	private String LAMAfolder = "C:\\LAMA";
	
	// folder structure for files
	private String dbfile = "\\db\\datamodel_LAMA_postgresql.sql";
	private String dbresultfile = "\\db\\results_LAMA.sql";
	private String inputdatafolder = "\\input\\data";
	private String metadatafolder = "\\input\\metadata";
	private String pardistfile = "\\input\\data\\parameter_distribution.csv";
	private String parfracfile = "\\input\\data\\parameter_fraction.csv";
	private String popfile = "\\input\\data\\population.csv";	
	private String timeusefile = "\\input\\data\\timeuse.csv";
	private String urbincfile = "\\input\\data\\urban_increment.csv";
	private String confile = "\\input\\data\\concentration.csv";
	private String resultfolder = "\\output\\";
	private String visfolder = "\\output\\vis\\";
	private String emepDownloadfile = "EMEPgrid.txt.zip";
	private String emepfile = "\\input\\EMEPgrid.txt";
	private String logfolder = "\\output\\logs/";
	private String addFile = "";
	
	// Database information
	// for MySQL: com.mysql.jdbc.Driver
	private String jdbcDriver = "org.postgresql.Driver";
	private String dbURL = "jdbc:postgresql://localhost:5432/LAMAdb";
	private String dbUser = "pguser";
	private String dbPassword = "retter45";
	
	// metadata
	private String scenario, pollutant, countryName, country_id, scenario_description;
	private int scenario_id;
	private int year;
	
	// parameters for simulation
	private boolean gridResults, individualResults, subgroupResults, realisationsResults, exETS, exBGR, exBMS; 
	private int iterations;
	private String simID;
	private float s_cig_perc;
	
	// data files
	//private MTUScountry mtus;
	//private EMEPcountryGrid con;
	//private EMEPcountryGrid[] pop;
	//private IndoorParameters par;
	
	// lists with definitions
	private Hashtable<String,String> countryHash, subgroupHash;
	private String[] me = new String[5];	
	
	
	public SimulationParameters(String simID, String countryID, String pollutant, 
			String scenario, int iterations, boolean gridbased, boolean individualResults, 
			boolean subgroupResults, boolean storeRealisations){
		this.simID = simID;
		this.country_id = countryID;
		this.pollutant = pollutant;
		this.scenario = scenario;
		this.scenario_description = scenario;
		this.iterations = iterations;
		this.exETS = false;
		this.exBMS = false;
		this.exBGR = false;
		this.gridResults = gridbased;
		this.realisationsResults = storeRealisations;
		this.individualResults = individualResults;
		this.subgroupResults = subgroupResults;

		this.s_cig_perc = 1;	// default
		fillMEarray();
	}
	
	// general constructor
	public SimulationParameters(){
		
	}
	
	private void fillMEarray(){
		me[0] = "home";
		me[1] = "work";
		me[2] = "travel";
		me[3] = "outdoor";
		me[4] = "other";
	}	
	
	// **** SETTERS ****
	// SETTERS for booleans
	public void set_exBiomass(boolean excludeBiomass){
		exBMS = excludeBiomass;
	}
	
	public void set_exETS(boolean excludeETS){
		exETS = excludeETS;
	}
	
	public void set_exBackground(boolean excludeBackground){
		exBGR = excludeBackground;
	}
	
	// SETTERS for individual data
	public String get_simID(){
		return simID;
	}
	
	public String[] get_microenvironments(){
		return me;
	}
	
	public void set_year(int scenario_year){
		year = scenario_year;
	}
	
	public void set_ratioETS(float ratio){
		this.s_cig_perc = ratio;
	}
	
//	public void set_scenario_description(String description){
//		scenario_description = description;
//	}
	
	public void set_countryHash(Hashtable<String,String> countries){
		this.countryHash = countries;
		countryName = countryHash.get(country_id);
	}
	
	public void set_subgroupHash(Hashtable<String,String> subgroups){
		this.subgroupHash = subgroups;
	}
	
	public void set_LAMAfolder(String folder){
		this.LAMAfolder = folder;
	}
	
	// database setters
	public void set_JDBC_Driver(String driver){
		this.jdbcDriver = driver;
		// for MySQL use a different sql file
		if(driver.equals("com.mysql.jdbc.Driver")){
			dbfile="\\db\\datamodel_LAMA_mysql.sql";
			dbresultfile = "\\db\\results_LAMA_mysql.sql";
		}
	}
	
	public void set_dbURL (String url){
		this.dbURL = url;
	}
	
	public void set_dbUser (String user){
		this.dbUser = user;
	}
	
	public void set_dbPassword (String password){
		this.dbPassword = password;
	}

	// **** GETTERS ****
	// GETTERS for individual data
	public int get_year(){
		return year;
	}
	
	public float get_ratioETS(){
		return s_cig_perc;
	}
	
	public String get_countryName(){
		return countryName;
	}
	
	public String get_country_id(){
		return country_id;
	}
	
	public String get_scenario(){
		return scenario;
	}
	
//	public int get_scenario_id_as_integer(){
//		return scenario_id;
//	}
	
//	public String get_scenario_name(){
//		return scenario_description;
//	}
	
	public String get_pollutant(){
		return pollutant;
	}
	
	public int get_iterations(){
		return iterations;
	}
	
	// boolean variables
	public boolean get_gridbased(){
		return gridResults;
	}
	
	public boolean get_individualResults(){
		return individualResults;
	}
		
	public boolean get_subgroupResults(){
		return subgroupResults;
	}
	
	public boolean get_totalPopResults(){
		if(!individualResults&&!subgroupResults)
			return true;
		else
			return false;
	}
	
	public boolean get_storeRealisationResults(){
		return realisationsResults;
	} 
	
	public boolean get_exBiomass(){
		return exBMS;
	}
	
	public boolean get_exETS(){
		return exETS;
	}
	
	public boolean get_exBackground(){
		return exBGR;
	}
	
//	public int get_year(){
//		if(scenario_id==0)
//			return 2010;
//		else if(scenario_id==1||scenario_id==2)
//			return 2020;
//		else if(scenario_id==3||scenario_id==4)
//			return 2030;
//		else if(scenario_id==5||scenario_id==6)
//			return 2050;
//		else
//			return 2010;
//	}
	
	// database variables
	public String get_JDBC_Driver(){
		return jdbcDriver;
	}
	
	public String get_dbURL(){
		return dbURL;
	}
	
	public String get_dbUser(){
		return dbUser;
	}
	
	public String get_dbPassword(){
		return dbPassword;
	}
	
	private void get_addFile(){
		addFile = "";
		if(exETS)
			addFile += "withoutETS_";
		if(exBMS)
			addFile += "withoutBiomass_";
		if(exBGR)
			addFile += "withoutBackground_";
	}
	
	// GETTERS for files and directories
	public String get_resultFile(){
		this.get_addFile();
		if(gridResults)
			return (LAMAfolder+ resultfolder+scenario_description+"\\grid\\"+country_id+"_"+scenario_description+"_"+pollutant+"_i"+iterations+"_"+addFile);
		else
			return (LAMAfolder+ resultfolder+scenario_description+"\\country\\"+country_id+"_"+scenario_description+"_"+pollutant+"_i"+iterations+"_"+addFile);
	}
	
	public String get_scenarioFolder(){
		if(gridResults)
			return (LAMAfolder+ resultfolder+scenario_description+"\\");
		else
			return (LAMAfolder+ resultfolder+scenario_description+"\\");
	}
	
	public String get_resultFolder(){
		if(gridResults)
			return (LAMAfolder+ resultfolder+scenario_description+"\\grid\\");
		else
			return (LAMAfolder+ resultfolder+scenario_description+"\\country\\");
	}
	
	// visualisation files
	public String get_visualisationFile(){
		this.get_addFile();
		return (LAMAfolder+visfolder+scenario_description+"\\"+country_id+"_"+scenario_description+"_"+pollutant+"_i"+iterations+"_"+addFile);
	}
	
	public String get_EUvisualisationFile(){
		this.get_addFile();
		return (LAMAfolder+visfolder+scenario_description+"\\"+"EU_"+scenario_description+"_"+pollutant+"_i"+iterations+"_"+addFile);
	}
	
	public String get_visBatchFile(){
		this.get_addFile();
		return (LAMAfolder+visfolder+country_id+"_"+scenario_description+"_"+pollutant+"_i"+iterations+"_"+addFile);
	}
	
	public String get_visFolder(){
		return (LAMAfolder+visfolder+scenario_description);
	}
	
	public String get_worldShapefileETRS(){
		return "world_countries";
	}
	
	public String get_ETRSFile(){
		return (LAMAfolder+visfolder+"aaigrid_etrs.txt");
	}
	
	public String get_intersectionFile(){
		return (LAMAfolder+visfolder+"emep_intersection.csv");
	}
	
	// others
	public String get_databaseFile(){
		return (LAMAfolder+dbfile);
	}
	
	public String get_resetResultsDBFile(){
		return (LAMAfolder+dbresultfile);
	}
	
	public String get_populationFile(){
		return (LAMAfolder+popfile);
	}
	
	public String get_concentrationFile(){
		return (LAMAfolder+confile);
	}
	
	public String get_timeuseFile(){
		return (LAMAfolder+timeusefile);
	}
	
	public String get_parameterFile(){
		return (LAMAfolder+metadatafolder+"\\parameter.csv");
	}
	
	
	public String get_parameterDistributionFile(){
		return (LAMAfolder+pardistfile);
	}
	
	public String get_parameterFractionFile(){
		return (LAMAfolder+parfracfile);
	}
	
	public String get_urban_incrementFile(){
		return (LAMAfolder+urbincfile);
	}
	
	public String get_emepgridFile(){
		return (LAMAfolder+emepfile);
	}
	
	public String get_emepDownloadFile(){
		return (emepDownloadfile);
	}
	
	public String get_metadataFolder(){
		return (LAMAfolder+metadatafolder);
	}
	
	public String get_inputdataFolder(){
		return (LAMAfolder+inputdatafolder);
	}
	
	public String get_logFile(){
		//return (LAMAfolder+logfolder+country_id+"_"+scenario_description+"_"+simID+".txt");
		return (LAMAfolder+logfolder+country_id+"_"+scenario_description+"_"+pollutant+"_i"+iterations+"_");
	}
	
	// GETTERS for hashtables
	public String[] get_subgroup_array(){
		String[] subgroupArray = new String[subgroupHash.size()];
		Enumeration<String> keys = subgroupHash.keys();
		while(keys.hasMoreElements()){
			String name = keys.nextElement();
			subgroupArray[Integer.parseInt(subgroupHash.get(name))] = name;
		}
		return subgroupArray;
	}
	
	public Hashtable<String,String> get_subgroupHash(){
		return subgroupHash;
	}
	
	public Hashtable<String,String> get_countryHash(){
		return countryHash;
	}
}
