package de.ifgi.exposure.datamodel;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.logging.Level;

import de.ifgi.exposure.control.LogFile;

public class VisualisationParameters {

	private float[] quantiles;
	private boolean startAguila, writeQuantiles, countryWise;
	private ArrayList<Float> data_mean, data_sd, data_perc;
	private ArrayList<ArrayList<Float>> data_quantiles;
	private ArrayList<int[]> ids;
	private Hashtable<String, Float> ids_intersections;
	private int country_count, quantile_count;
	
	private ArrayList<String[]> summary_ids;
	private ArrayList<float[]> summary_results;
	public VisualisationParameters(boolean vis, boolean startAguila, boolean writeQuantiles){
		this.startAguila = startAguila;
		this.writeQuantiles = writeQuantiles;
		if(writeQuantiles){
			data_quantiles = new ArrayList<ArrayList<Float>>();
		}
		else{
			//data_mean = new ArrayList<Float>();
		}
		
		//ids = new ArrayList<int[]>();
		country_count = 0;
		quantile_count = 0;
	}
	
	// SETTERS
	public void set_countryWise_visualisation(boolean countryVis){
		this.countryWise = countryVis;
	}
	
	public void set_quantile_levels(float[] quantile_levels){
		quantiles = quantile_levels;
	}
	
	public void read_intersectionFile(String filename){
		if(ids_intersections==null||ids_intersections.size()==0){
			try{
				
				// first read ascii etrs example file
				BufferedReader input = new BufferedReader(new FileReader(new File(
						filename)));
				ids_intersections = new Hashtable<String, Float>();
				// read cells
				// header Emep50_i, Emep50_j, CountryID, Area_Incl_Sea, Area_Incl_Land, EmepID, ID_intersection
				String row = input.readLine();
				while ((row = input.readLine()) != null) {			
					String[] s = row.split(",");
					// store emep i_j with country id
					String newRow = s[0].trim()+"_"+s[1].trim()+"_"+s[2].trim();
					ids_intersections.put(newRow, Float.parseFloat(s[4].trim()));
				}
				input.close();
			}
			catch(Exception e){
				System.out.println("Exception during reading file: "+ filename+"\n"+e.getMessage());
				LogFile.logger.log(Level.SEVERE,"Exception during reading file: "+ filename+"\n"
						+e.getMessage()+ " Cause "+ e.getCause());
			}
		}
	}
	
	// fill data for European files
	public void set_statistics_data(String country_id, ArrayList<Float> mean, ArrayList<Float> sd, ArrayList<int[]> emep_ids){
		// if this is the first country to add
		if(country_count==0){
			data_mean = mean;
			data_sd = sd;
			data_perc = new ArrayList<Float>();
			for(int i=0; i<data_mean.size(); i++){
				data_perc.add(1F);
			}
			ids = emep_ids;
			country_count++;
		}
		// if there is more than one country, merge cells
		else{
			// loop through new cells
			for(int i=0; i<emep_ids.size(); i++){
				boolean newCell = true;
				String newID = emep_ids.get(i)[0]+"_"+emep_ids.get(i)[1];

				// loop through existing cells to check if the cell value is already there
				for(int j=0; j<ids.size(); j++){
					String oldID = ids.get(j)[0]+"_"+ids.get(j)[1];
					if(newID.equals(oldID)){
						// add cell value according to intersection land percentage
						String countryID = newID + "_" + country_id;
						//System.out.println(countryID);
						Float perc = ids_intersections.get(countryID);
						float newMean, newSD, newPerc;
						if(perc!=null){
							newMean = data_mean.get(j) + mean.get(i)*perc;					
							newSD = data_sd.get(j)+ sd.get(i)*perc;	
							newPerc = data_perc.get(j)+perc;
							data_mean.set(j, newMean);
							data_sd.set(j, newSD);
							data_perc.set(j, newPerc);
							
						}
						//simply average both values with same weight
						else{
							System.out.println("Missing intersection value for: "+countryID);
//							newMean = data_mean.get(j)*0.5F + mean.get(i)*0.5F;					
//							newSD = data_sd.get(j)*0.5F + sd.get(i)*0.5F;	
//							newPerc = data_perc.get(j) + 0.5F;
						}
						// indicate that cell has already been added
						newCell = false;
						j = ids.size();
					}
						
				}
				if(newCell){
					// add cell value according to intersection land percentage
					String countryID = newID + "_" + country_id;
				//	System.out.println(countryID);
					Float perc = ids_intersections.get(countryID);
					if(perc!=null){
						data_mean.add(mean.get(i)*perc);
						data_sd.add(sd.get(i)*perc);
						data_perc.add(perc);
					}
					else{
						System.out.println("Missing intersection value for: "+countryID);
						data_mean.add(mean.get(i));
						data_sd.add(sd.get(i));
						data_perc.add(1F);
					}
					ids.add(emep_ids.get(i));
				}
			}
			country_count++;
			
			//for the last country check intersection percentages
			if(country_count==30){
				for(int i=0; i<data_perc.size(); i++){
					Float newMean, newSD;
					if(data_perc.get(i)!=0){
						newMean = data_mean.get(i)/data_perc.get(i);
						newSD = data_sd.get(i)/data_perc.get(i);
						if(newMean>1000)
							System.out.println("Percentage "+data_perc.get(i)+" for value "+data_mean.get(i));
						data_mean.set(i, newMean);
						data_sd.set(i, newSD);
					}
					else
						System.out.println("Percentage 0: "+data_mean.get(i));
				}
			}
		}
			
	}
	
	public void set_quantiles_data(String country_id, ArrayList<Float> quantileList, float quantileLevel, ArrayList<int[]> emep_ids){
		// if this is the first country and the first quantile to add
		if(country_count==0&&data_quantiles.size()==0){
			data_quantiles.add(quantileList);
			ids = emep_ids;
			quantile_count++;
			country_count++;
		}
		// add other quantiles for the first country
		else if(country_count==1&&data_quantiles.size()<quantiles.length){
			quantile_count++;
			if(quantile_count==quantiles.length)
				quantile_count = 0;
			data_quantiles.add(quantileList);
		}
		// if there is more than one country, merge cells
		else{
			// find index for current quantile level
			int index = 0;
			for(int i=0; i<quantiles.length; i++){
				if(quantiles[i] == quantileLevel)
					index = i;
			}
			// retrieve existing quantile ArrayList
			ArrayList<Float> existingQuantiles = data_quantiles.get(index);
			// loop through new cells
			for(int i=0; i<emep_ids.size(); i++){
				boolean newCell = true;
				String newID = emep_ids.get(i)[0]+"_"+emep_ids.get(i)[1];
				// loop through existing cells to check if the cell value is already there
				for(int j=0; j<existingQuantiles.size(); j++){
					String oldID = ids.get(j)[0]+"_"+ids.get(j)[1];
					//if(ids.get(j)[0])
					//System.out.println("New ID: ");
					if(newID.equals(oldID)){
						// add cell value according to intersection land percentage
						String countryID = newID + "_" + country_id;
						
						Float perc = ids_intersections.get(countryID);
						float newVal;
						if(perc!=null){
							newVal = existingQuantiles.get(j) + quantileList.get(i)*perc;					
						}
						//simply average both values with same weight
						else{
							System.out.println("Missing intersection value for: "+countryID);
							newVal = existingQuantiles.get(j)*0.5F + quantileList.get(i)*0.5F;
						}				
						existingQuantiles.set(j, newVal);
						
						// indicate that cell has already been added
						newCell = false;
						j = existingQuantiles.size();
					}
						
				}
				if(newCell){
					String countryID = newID + "_" + country_id;
				//	System.out.println(countryID);
					Float perc = ids_intersections.get(countryID);
					if(perc!=null){
						existingQuantiles.add(quantileList.get(i)*perc);
					}
					//simply average both values with same weight
					else{
						System.out.println("Missing intersection value for: "+countryID);
						existingQuantiles.add(quantileList.get(i));
					}
					
					// add ids only if they haven't been added before
					if(ids.size()<existingQuantiles.size())
						ids.add(emep_ids.get(i));
				}
			}
			// substitute existing quantile list with new one
			data_quantiles.set(index, existingQuantiles);
			quantile_count++;
			if(quantile_count==quantiles.length){
				quantile_count = 0;
				country_count++;
			}
		}
	}
	// GETTERS	
	// boolean variables
	public boolean get_country_wise(){
		return countryWise;
	}
	
	public boolean get_startAguila(){
		return startAguila;
	}
	
	public boolean get_write_quantiles(){
		return writeQuantiles;
	}
	
	// EMEP data
	public ArrayList<Float> get_mean_data(){
		return data_mean;
	}
	
	public ArrayList<Float> get_sd_data(){
		return data_sd;
	}
	
	public ArrayList<Float> get_quantile_data(float quantile_level){
		// find index for quantile level
		int index = 0;
		for(int i=0; i<quantiles.length; i++){
			if(quantiles[i] == quantile_level)
				index = i;
		}
		return data_quantiles.get(index);
	}
	
	public ArrayList<int[]> get_ids(){
		return ids;
	}
	
	// variables for quantile levels and current count of countries
	public float[] get_quantile_levels(){
		return quantiles;
	}
	
	public int get_country_count(){
		return country_count;
	}
	
}
