package de.ifgi.exposure.db;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.ResultSet;
import java.util.ArrayList;
import java.util.logging.Level;
import de.ifgi.exposure.control.LogFile;

/**
 * Class to retrieve data from LAMADatabase,
 * PostgreSQL in this case.
 * 
 * @author Lydia Gerharz
 */

// connects to LAMADataBase and creates SQL statements
public class DatabaseAccess {
	
	// DBMS and Database-specific connection parameters
	private  String JDBC_DRIVER = "";
	// You may need to change these values
	private String URL = "";
	private String USERNAME = "";
	private String PASSWORD = "";
	private Connection dbConn = null;
	
	public DatabaseAccess(String jdbcDriver, String url, String user, String password){	
		this.JDBC_DRIVER = jdbcDriver;
		this.URL = url;
		this.USERNAME = user;
		this.PASSWORD = password;
	}

	// establish connection
	public boolean connect() {
		System.out.println("Try to connect...");
		boolean success = false;
		try{
			Class.forName(JDBC_DRIVER);
		}
		catch(ClassNotFoundException e){
			System.out.println("Unable to find databasedriver "+JDBC_DRIVER+"\n"
					+ e.getMessage());
			LogFile.logger.log(Level.SEVERE,"Unable to find databasedriver "+JDBC_DRIVER+"\n"
					+ e.getMessage()+ "\n Cause "+e.getCause());
			success = false;
		}
		
		// Create a connection to the database
		
		try {
			dbConn = DriverManager.getConnection(URL, USERNAME, PASSWORD);
			System.out.println("Connected to: " + URL);
			LogFile.logger.log(Level.INFO,"Connected to: " + URL);
			success= true;
		} catch (SQLException e) {
			System.out.println("Unable to connect to database "+URL+"\n"
					+ e.getMessage());
			LogFile.logger.log(Level.SEVERE,"Unable to connect to database "+URL+"\n"
					+ e.getMessage()+"\n Error Code "+e.getErrorCode()+ "\n Cause "+e.getCause());
			success = false;
			//System.exit(1);
		}
		return success;
	}
	
	// **** SIMULATION PARAMETERS ****
	public ArrayList<String[]> select_metadata(String tablename, int columns){
		String sqlSelect = "SELECT * from "+tablename+";";
		ArrayList<String[]> r = selectRows(sqlSelect, columns);	
		return r;
	}
			
	public ArrayList<String[]> select_country(){
		String sqlSelect = "SELECT country_id, country_name from country;";
		ArrayList<String[]> r = selectRows(sqlSelect, 2);	
		return r;
	}
	
	public ArrayList<String[]> select_pollutant(){
		String sqlSelect = "SELECT pollutant_id from pollutant;";
		ArrayList<String[]> r = selectRows(sqlSelect, 1);	
		return r;
	}
	
	public ArrayList<String[]> select_scenario(){
		String sqlSelect = "SELECT scenario_id, year, scenario_description from scenario;";
		ArrayList<String[]> r = selectRows(sqlSelect, 3);	
		return r;
	}
	
	// **** MTUS DATA ****
	public ArrayList<String[]> select_diary(String country_id){
		String sqlSelect = "SELECT diary_id, group_id from diary WHERE country_id = '"
			+country_id+"';";
		ArrayList<String[]> r = selectRows(sqlSelect, 2);	
		return r;
	}
	
	public ArrayList<String[]> select_diary_time(ArrayList<String[]> diary_ids){
		ArrayList<String[]> r = new ArrayList<String[]>(); 
		// loop through diary ids and add times to ArrayList
		for(int i=0; i<diary_ids.size(); i++){
			String sqlSelect = "SELECT microenvironment_id, value from diary_time WHERE diary_id = '"
				+diary_ids.get(i)[0]+"';";
			ResultSet result;
			try{
				result =  this.selectRow(sqlSelect);
				// loop through rows
				while(result.next()){
					String[] row = {result.getString(1), result.getString(2)};
					r.add(row);
				}
			}
			catch (SQLException e) {
				System.out.println("Error for selecting data from diary_time table.");
				System.out.println(sqlSelect);
				System.out.println(e.getMessage());
				LogFile.logger.log(Level.SEVERE,"Error for selecting data from diary_time table.\n"
						+ e.getMessage()+"\n Error Code "+e.getErrorCode()+ "\n Cause "+e.getCause());
				LogFile.logger.log(Level.INFO, "Line: "+sqlSelect);
			} 
		}
		
		return r;
	}

	// **** EMEP DATA ****
	public ArrayList<String[]> select_population(String country_id, String year){	
		String sqlSelect = "SELECT emep50_id, group_id, value from population WHERE country_id = '"
			+country_id+"' AND year = '"+year+"' AND growthrate = 'm';";
		ArrayList<String[]> r = this.selectRows(sqlSelect, 3);
		return r;
	}
	
	public float select_concentration(String emepID, String pollutant, String scenario){	
		float r = 0F;
		String sqlSelect = "SELECT value from concentration WHERE pollutant_id = '"
			+pollutant+"' AND scenario_id = '"+scenario+"' AND emep50_id = '"+emepID+"';";
		ResultSet result;
		try{
			result =  this.selectRow(sqlSelect);
			// get the only result row
			if(result.first())
				r = result.getFloat(1);
		}
		catch (SQLException e) {
			System.out.println("Error for selecting data from concentration table.");
			System.out.println(sqlSelect);
			System.out.println(e.getMessage());
			LogFile.logger.log(Level.SEVERE,"Error for selecting data from concentration table.\n"
					+ e.getMessage()+"\n Error Code "+e.getErrorCode()+ "\n Cause "+e.getCause());
			LogFile.logger.log(Level.INFO, "Line: "+sqlSelect);
		} 
		return r;
	}
	
	public float[] select_urban_increment(String emepID, String pollutant, String scenario_id){	
		float[] r = new float[2];
		
		String sqlSelect = "SELECT value, urban_pop from urban_increment WHERE pollutant_id = '"
			+pollutant+"' AND emep50_id = '"+emepID+"' AND scenario_id = '"+scenario_id+"';";
		ResultSet result;
		try{
			result =  this.selectRow(sqlSelect);
			// get the only result row
			if(result.first()){
				r[0] = result.getFloat(1);
				r[1] = result.getFloat(2);
			}
		}
		catch (SQLException e) {
			System.out.println("Error for selecting data from urban_increment table.");
			System.out.println(sqlSelect);
			System.out.println(e.getMessage());
			LogFile.logger.log(Level.SEVERE,"Error for selecting data from urban_increment table.\n"
					+ e.getMessage()+"\n Error Code "+e.getErrorCode()+ "\n Cause "+e.getCause());
			LogFile.logger.log(Level.INFO, "Line: "+sqlSelect);
		} 
		return r;
	}
	
	// **** PARAMETER DATA ****
	public ArrayList<String[]> select_parameter(){
		// select parameter id and distribution type
		String sqlSelect = "SELECT parameter_id, distribution_type from parameter;";
		ArrayList<String[]> r = this.selectRows(sqlSelect, 2);
		return r;
	}
	
	public ArrayList<String[]> select_parameter_distribution_array(String country_id, 
			String pollutant, String scenario_id){
		// select parameter id, microenvironment, distribution parameter and value
		String sqlSelect = "SELECT parameter_id, microenvironment_id, distribution_parameter, value from parameter_distribution WHERE scenario_id = '"
			+scenario_id+"' AND pollutant_id ='"+pollutant+"' AND country_id = '"+country_id+"';";
		ArrayList<String[]> r = this.selectRows(sqlSelect, 4);
		return r;
	}
	
	public ArrayList<String[]> select_parameter_fraction_array(String country_id, 
			String pollutant, String scenario_id){
		// select parameter id, microenvironment, distribution parameter and value
		String sqlSelect = "SELECT parameter_id, group_id, value from parameter_fraction WHERE scenario_id = '"
			+scenario_id+"' AND pollutant_id ='"+pollutant+"' AND country_id = '"+country_id+"';";
	//	System.out.println(sqlSelect);
		ArrayList<String[]> r = this.selectRows(sqlSelect, 3);
		return r;
	}
	
	// not necessary? 
	public ArrayList<String[]> select_parameter_distribution(ArrayList<String[]> ids, 
			String country_id, String pollutant, String scenario_id){
		// select microenvironment, distribution parameter and value
		ArrayList<String[]> r = new ArrayList<String[]>();
		
		for(int i=0; i<ids.size(); i++){
			String sqlSelect = "SELECT microenvironment_id, distribution_parameter, value from parameter_distribution WHERE parameter_id = '"
				+ids.get(i)[0]+"' AND scenario_id = '"+scenario_id+"' AND pollutant_id ='"+pollutant+"' AND country_id = '"+country_id+"';";
		//	System.out.println(sqlSelect);
			ResultSet result;
			try{
				result =  this.selectRow(sqlSelect);
				// loop through rows
				while(result.next()){
					String[] row = {result.getString(0), result.getString(1)};
					r.add(row);
				}
			}
			catch (SQLException e) {
				System.out.println("Error for selecting data from parameter_distribution table.");
				LogFile.logger.log(Level.SEVERE,"Error for selecting data from parameter_distribution table.\n"
						+ e.getMessage()+"\n Error Code "+e.getErrorCode()+ "\n Cause "+e.getCause());
				LogFile.logger.log(Level.INFO, "Line: "+sqlSelect);
			} 
		}
		return r;
	}
	
	// **** UTILITIES ****
	public ArrayList<String[]> selectRows(String sqlSelect, int columnCount){
		ArrayList<String[]> r = new ArrayList<String[]>();
		ResultSet result; Statement stmt;
		try{
			stmt = dbConn.createStatement(ResultSet.TYPE_SCROLL_SENSITIVE,
	                ResultSet.CONCUR_UPDATABLE);
			result =  stmt.executeQuery(sqlSelect);
			// loop through rows
			while(result.next()){
				String[] row = new String[columnCount];
				for(int i=0; i<columnCount; i++){
					row[i] = result.getString(i+1);
				}
				r.add(row);
			}
		}
		catch (SQLException e) {
			System.out.println("Error during: ");
			System.out.println(sqlSelect);
			System.out.println(e.getMessage());
			LogFile.logger.log(Level.SEVERE,"Error for selecting data from table.\n"
					+ e.getMessage()+"\n Error Code "+e.getErrorCode()+ "\n Cause "+e.getCause());
			LogFile.logger.log(Level.INFO, "Line: "+sqlSelect);
		} 
		
		return r;
	}
	
	public ResultSet selectRow(String sqlSelect) throws SQLException{
		Statement stmt; ResultSet result = null;
		stmt = dbConn.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE,
                ResultSet.CONCUR_UPDATABLE);
		result = stmt.executeQuery(sqlSelect);
		return result;
	}

	public void close(){
		try{
			this.dbConn.close();
		}
		catch(SQLException e){
			System.out.println("SQLException while closing database: " + e.getMessage());
		}
	}
	
	public Connection getDbConn() {
		return dbConn;
	}

	public void setDbConn(Connection dbConn) {
		this.dbConn = dbConn;
	}

	/**
	 * Method connects to the database, issues a simple query and displays the
	 * results
	 * 
	 * @throws ClassNotFoundException
	 * @throws SQLException 
	 */

} 
