package de.ifgi.exposure.db;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.ResultSet;
import java.util.logging.Level;
import de.ifgi.exposure.control.LogFile;
import java.util.ArrayList;

/**
 * Feeder for the LAMADatabase,
 * PostgreSQL in this case.
 * 
 * @author Lydia Gerharz
 */

public class DatabaseFeeder {

	// DBMS and Database-specific connection parameters
	private  String JDBC_DRIVER = "";
	private  String URL = "";
	private  String USERNAME = "";
	private  String PASSWORD = "";
	private Connection dbConn = null;

	public DatabaseFeeder(String jdbcDriver, String url, String user, String password){	
		this.JDBC_DRIVER = jdbcDriver;
		this.URL = url;
		this.USERNAME = user;
		this.PASSWORD = password;
	}

	// establish connection
	public boolean connect() {
		System.out.println("Try to connect...");
		boolean success = false;
		try{
			Class.forName(JDBC_DRIVER);
		}
		catch(ClassNotFoundException e){
			System.out.println("Unable to find databasedriver "+JDBC_DRIVER+"\n"
					+ e.getMessage());
			LogFile.logger.log(Level.SEVERE,"Unable to find databasedriver "+JDBC_DRIVER+"\n"
					+ e.getMessage()+"\n Cause "+e.getCause());
			success = false;
		}
		
		// Create a connection to the database	
		try {
			dbConn = DriverManager.getConnection(URL, USERNAME, PASSWORD);
			System.out.println("Connected to: " + URL);
			success = true;
			LogFile.logger.log(Level.INFO,"Connected to: " + URL);
		} catch (SQLException e) {
			System.out.println("Unable to connect to database "+URL+"\n"
					+ e.getMessage());
			LogFile.logger.log(Level.INFO,"Unable to connect to database "+URL+"\n"
					+ e.getMessage()+" CODE "+e.getErrorCode()+ "\n Cause "+e.getCause());
			success = false;
			System.exit(1);
		}
		return success;
	}
	
	public boolean initialiseDatabase(String filename){
		boolean success;
		StringBuffer query = new StringBuffer();
		Statement stmt;
		
		try{
			stmt = dbConn.createStatement(); 		
			BufferedReader reader = new BufferedReader(new FileReader(filename));
			String line;
		
			while ((line = reader.readLine()) != null) {
				// only add lines which are not comments
				if(!line.startsWith("-")){
					query.append(line);
					if (line.endsWith(";")){
						//System.out.println("query->"+query);
						stmt.addBatch(query.toString());
						stmt.executeBatch();
						query.setLength(0);
					}
				}
			}
			success = true;
		}
		catch(IOException e){
			System.out.println("IOException during initialisation of database: \n" + e.getMessage());
			LogFile.logger.log(Level.SEVERE,"IOException during initialisation of database: \n" 
					+ e.getMessage()+ "\n Cause "+e.getCause());
					
			success = false;
		}
			
		catch(SQLException e){
			System.out.println("SQLException during initialisation of database: \n" + e.getMessage());
			LogFile.logger.log(Level.SEVERE,"SQLException during initialisation of database: \n"
					+ e.getMessage()+" CODE "+e.getErrorCode()+ "\n Cause "+e.getCause());
			success = false;
		}
		return success;
	}
	
	// **** GENERIC TABLE FILLING METHODS****	
	public boolean insertSimpleRow(String[] data, String tablename){
		boolean success;
		String dataList = "";
		
		// append data items for the INSERT query
		for(int i=0; i<data.length; i++){
			// only for the first item
			if(i==0)
				dataList = "'"+data[i]+"'";
			else
				dataList = dataList + ", '"+data[i] +"'";
		}		
		Statement stmt;	String sqlInsert;
		
		// SQL query
		sqlInsert = "INSERT INTO "+tablename+" VALUES ("+dataList+");";			
		//System.out.println(sqlInsert);
		try{
			stmt = dbConn.createStatement();
			stmt.executeUpdate(sqlInsert); 
			success = true;
		}
		catch (SQLException e) {
			System.out.println("Error for inserting data in " + tablename);
			System.out.println(sqlInsert);
			LogFile.logger.log(Level.SEVERE,"Error for inserting data in " + tablename +"\n"
					+ e.getMessage()+" CODE "+e.getErrorCode()+ "\n Cause "+e.getCause());
			LogFile.logger.log(Level.INFO, "Line: "+sqlInsert);
			success = false;
		} 
		return success;
	}
	
	public boolean insertRow(String[] data, String columnnames, String tablename){
		boolean success;
		String dataList = "";
		
		// append data items for the INSERT query
		for(int i=0; i<data.length; i++){
			// only for the first item
			if(i==0)
				dataList = "'"+data[i]+"'";
			else
				dataList = dataList + ", '"+data[i] +"'";
		}		
		Statement stmt;	String sqlInsert;
		
		// SQL query
		sqlInsert = "INSERT INTO "+tablename+" ("+columnnames+") VALUES ("+dataList+");";			
		//System.out.println(sqlInsert);
		try{
			stmt = dbConn.createStatement();
			stmt.executeUpdate(sqlInsert); 
			success = true;
		}
		catch (SQLException e) {
			System.out.println("Error for inserting data in " + tablename);
			System.out.println(sqlInsert);
			LogFile.logger.log(Level.SEVERE,"Error for inserting data in " + tablename +"\n"
					+ e.getMessage()+" CODE "+e.getErrorCode()+ "\n Cause "+e.getCause());
			LogFile.logger.log(Level.INFO, "Line: "+sqlInsert);
			success = false;
		} 
		return success;
	}
	
	// **** METHODS TO INSERT ARRAYLISTS OF ROWS ****	
	public void insertSimpleRows(ArrayList<String[]> data, String tablename){
		int dataCount = data.size();
		int rowCount= 0;
		for (int j=0; j<dataCount; j++){			
			boolean insert = insertSimpleRow(data.get(j),tablename);
			if(insert)
				rowCount++;
			if(tablename.equals("emep50_grid")||tablename.equals("diary")){
				if (rowCount% 100 == 0) // only log every 100 iterations
					System.out.println("Inserted: " + rowCount+" entries into table "+tablename+".");
			}
			else{
				if (rowCount % 10 == 0) // only log every 10 iterations
					System.out.println("Inserted: " + rowCount+" entries into table "+tablename+".");
			}}
		System.out.println("Inserted successfully all "+rowCount+" entries into table "+tablename+".");
	}
	
	public void insertRows(ArrayList<String[]> data, String columnnames, String tablename){
		int rowCount= 0;
		int dataCount = data.size();
		for (int j=0; j<dataCount; j++){			
			boolean insert = insertRow(data.get(j), columnnames, tablename);
			if(insert)
				rowCount++;
			if(tablename.equals("diary_time")||tablename.equals("population")||tablename.equals("concentration")){
				if (rowCount % 1000 == 0) // only log every 100 iterations
					System.out.println("Inserted: " + rowCount+" entries into table "+tablename+".");
			}
			else{
				if (rowCount % 10 == 0) // only log every 10 iterations
					System.out.println("Inserted: " + rowCount+" entries into table "+tablename+".");
			}
		}
		System.out.println("Inserted successfully all "+rowCount+" entries into table "+tablename+".");
	}
	
	// **** EMEP DATA ****	
	// method for emep_grid_fraction table
	public boolean insertEmepFractionRow(String[] row){
		boolean success = false;
		Statement stmt;	String sqlInsert = ""; ResultSet result;
		
		// get procedure id for foi
		String country = "";
		// first check if country is in country table
		String sqlSelectCountry = "SELECT country_name from country WHERE country_id = '"+row[1]+"';";
		try{
			stmt = dbConn.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE,
                    ResultSet.CONCUR_UPDATABLE);
			result = stmt.executeQuery(sqlSelectCountry);
			if(result.first()){
				country = result.getString(1);
			}
			
			if(!country.equals("")){
			//	System.out.println("Country: "+country);
				//INSERT INTO emep50_grid_fraction (emep50_id, country_id, area_fraction, population_fraction) VALUES (1, 'DE', 1, 1);
				// SQL query
				sqlInsert = "INSERT INTO emep50_grid_fraction (emep50_id, country_id, area_fraction) VALUES ('"+row[0]+"', '"+row[1]+"', '"+row[2]+"');";	
			
				stmt = dbConn.createStatement();
				System.out.println(sqlInsert);
				stmt.executeUpdate(sqlInsert); 
				success = true;
			}
			//else
			//	System.out.println("Country does not exist in the database: "+ row[1]);
		}
		catch (SQLException e) {
			System.out.println("Error for inserting data in EMEP fraction table.");
			LogFile.logger.log(Level.SEVERE,"Error for inserting data in EMEP fraction table \n"
					+ e.getMessage()+" CODE "+e.getErrorCode()+ "\n Cause "+e.getCause());
			LogFile.logger.log(Level.INFO, "Line: "+sqlInsert);
			success = false;
		} 
	
		return success;
	}
	
	// **** WRITE RESULTS TO DATABASE ****	
	public boolean writeResult(String[] data, String columnnames, String tablename){
		boolean success;
		String dataList = "";
		
		// append data items for the INSERT query
		for(int i=0; i<data.length; i++){
			// only for the first item
			if(i==0)
				dataList = "'"+data[i]+"'";
			else
				dataList = dataList + ", '"+data[i] +"'";
		//	System.out.println(i+": "+dataList);
		}
		
		Statement stmt;	String sqlInsert;
		
		// SQL query
		sqlInsert = "INSERT INTO "+tablename+" ("+columnnames+") VALUES ("+dataList+");";			
		try{
			stmt = dbConn.createStatement();
		//	System.out.println(sqlInsert);
			stmt.executeUpdate(sqlInsert); 
			success = true;
		}
		catch (SQLException e) {
			System.out.println("Error for inserting data in " + tablename);
			System.out.println(sqlInsert);
			LogFile.logger.log(Level.SEVERE,"Error for inserting data in " + tablename +"\n"
					+ e.getMessage()+" CODE "+e.getErrorCode()+ "\n Cause "+e.getCause());
			LogFile.logger.log(Level.INFO, "Line: "+sqlInsert);
			success = false;
		} 
		return success;
	}

	public void close(){
		try{
			this.dbConn.close();
		}
		catch(SQLException e){
			System.out.println("SQLException while closing database: " + e.getMessage());
		}
	}

	public Connection getDbConn() {
		return dbConn;
	}

	public void setDbConn(Connection dbConn) {
		this.dbConn = dbConn;
	}

	/**
	 * Method connects to the database, issues a simple query and displays the
	 * results
	 * 
	 * @throws ClassNotFoundException
	 * @throws SQLException 
	 */
}
