package de.ifgi.exposure.io;
import java.io.*;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.logging.Level;

import de.ifgi.exposure.datamodel.IndoorParameters;
import de.ifgi.exposure.datamodel.MTUScountry;
import de.ifgi.exposure.datamodel.PopulationSubgroup;
import de.ifgi.exposure.control.LogFile;

public class CsvHandler {
	
//	private Hashtable<String,String> countries, parameter_distribution_type;
//	private ArrayList<PopulationSubgroup> subgroups;
	
	public CsvHandler(){		
	}

	// **** MTUS PARSING METHOD ****
	// adapt columnnames
	// build subgroups
	// build microenvironments
	public MTUScountry readMTUSfile(String filename, String countryid, String delimiter, 
			 ArrayList<PopulationSubgroup> subgroups, Hashtable<String,String> meHash){
		MTUScountry mtus = new MTUScountry(countryid);
		mtus.set_microenvironments(meHash);
	
		Hashtable<String, Integer> columns = new Hashtable<String, Integer>();		
		// read file
		try{
			BufferedReader input = new BufferedReader(new FileReader(new File(
					filename)));
			String row = input.readLine();
			//System.out.println(header);
			String cols[] = row.split(delimiter);
			if(cols.length<2&&delimiter.equals(";"))
			{
				delimiter = ",";
				cols = row.split(delimiter);
			}
			else if(cols.length<2&&delimiter.equals(","))
			{
				delimiter = ";";
				cols = row.split(delimiter);
			}
			// get indices for columns
			for(int i=0;i<cols.length;i++){
				String temp = cols[i].trim();
				columns.put(temp, i);
			}	
			
			// loop through lines
			while ((row = input.readLine()) != null) {			
				String[] s = row.split(delimiter);

				// only parse country specified
				if(countryid.equals(s[columns.get("countryID")].trim())){
					// get age, sex and employment status to build groups
					int age = Integer.parseInt(s[columns.get("age")].trim());
					String sex = this.genderConverter(s[columns.get("sex")].trim());
					int emp = Integer.parseInt(s[columns.get("emp")].trim());
					String group = this.groupBuilder(sex, age, emp, subgroups);
			
					//String group = this.groupConverter(s[columns.get("group")].trim());
					//System.out.println(s[columns.get("group")].trim()+" TO "+ group);
					
					// ignore individual if no subgroup is available
					if(group!=null){			
						// check if microenvironments are already defined
						//columns.get("bla")!=null&&
						if(columns.get("home")!=null&&columns.get("work")!=null&&
								columns.get("travel")!=null&&columns.get("other")!=null&&columns.get("outdoor")!=null){				
							// set time spent per ME and individual details
							mtus.set_time(Integer.parseInt(s[columns.get("home")].trim()), "home");
							mtus.set_time(Integer.parseInt(s[columns.get("work")].trim()), "work");
							mtus.set_time(Integer.parseInt(s[columns.get("travel")].trim()), "travel");
							mtus.set_time(Integer.parseInt(s[columns.get("other")].trim()), "other");
							mtus.set_time(Integer.parseInt(s[columns.get("outdoor")].trim()), "outdoor");
						
							mtus.set_group(group);
							mtus.set_age(age);
							mtus.set_sex(sex);
							mtus.set_employment(emp+"");
							
							mtus.close_diary();
						}
						// otherwise use activities to add up to microenvironments
						else{
							Hashtable<String,Integer> me = get_ME(s,columns);
					
							// check sum of micronenvironments
							if(me!=null){
								System.out.println("home: "+Integer.parseInt(s[columns.get("home")].trim())
										+" calculation: "+me.get("home"));
								mtus.set_time(me.get("home"), "home");
								mtus.set_time(me.get("work"), "work");
								mtus.set_time(me.get("other"), "other");
								mtus.set_time(me.get("travel"), "travel");
								mtus.set_time(me.get("outdoor"), "outdoor");
								
								mtus.set_group(group);
								mtus.set_age(age);
								mtus.set_sex(sex);
								mtus.set_employment(emp+"");
								
								mtus.close_diary();
							}
						}		
					}
				}				
			}
			System.out.println(mtus.get_countryID()+" has been parsed with "+mtus.get_size()+" entries.");
		}
		catch(Exception e){
			System.out.println("Exception during reading time use csv file: "+filename+", \n :"+e.getMessage());
			LogFile.logger.log(Level.SEVERE, "Exception during reading time use csv file: "+filename+", \n :"+e.getMessage());
		}
//		mtus.set_subgroups(subgroups);
		return mtus;	
	}
	
	// **** PARAMETER PARSING METHOD ****
	// method to read indoor parameter files
	public IndoorParameters[] readParameterFile(String parameterfile, String distfile, String fracfile, String pollutant,
			String scenario, String country_id, Hashtable<String,String> subgroups){
		IndoorParameters[] paras = new IndoorParameters[3];
		
		// get parameter objects from parameter file
		IndoorParameters p_home_raw = this.ParameterFile2Object(country_id, "home", parameterfile);
		IndoorParameters p_work_raw = this.ParameterFile2Object(country_id, "work", parameterfile);
		IndoorParameters p_travel_raw = this.ParameterFile2Object(country_id, "travel", parameterfile);
		
		// add parameter values from distribution file
		IndoorParameters p_home_temp = this.readParameterDistributionFile(distfile, "home", country_id, pollutant, scenario, p_home_raw);
		IndoorParameters p_work = this.readParameterDistributionFile(distfile, "work", country_id, pollutant, scenario, p_work_raw);
		IndoorParameters p_travel = this.readParameterDistributionFile(distfile, "travel", country_id, pollutant, scenario, p_travel_raw);
		
		// add parameter values from fraction file
		IndoorParameters p_home = this.readParameterFractionFile(fracfile, p_home_temp, country_id, pollutant, scenario, subgroups);
		
		// first read distribution file
//		IndoorParameters p_home_temp = this.readParameterDistributionFile(distfile, "home", 
//				country_id, pollutant, scenario, delimiter, parameter_distribution_type);
//		IndoorParameters p_work = this.readParameterDistributionFile(distfile, "work", 
//				country_id, pollutant, scenario, delimiter, parameter_distribution_type);
//		IndoorParameters p_travel = this.readParameterDistributionFile(distfile, "travel",
//				country_id, pollutant, scenario, delimiter, parameter_distribution_type);
//		
//		// then add data from fractions file
//		IndoorParameters p_home = this.readParameterFractionFile(fracfile, p_home_temp, 
//				country_id, pollutant, scenario, delimiter, subgroups);
		
		paras[0] = p_home;
		paras[1] = p_work;	
		paras[2] = p_travel;
		return paras;
	}
	
	// method to create parameter object from parameter file
	private IndoorParameters ParameterFile2Object(String country_id, String microenvironment, String parameterfile){
		IndoorParameters para = new IndoorParameters(country_id, microenvironment);
		String currLine = "";
		try{
			BufferedReader input = new BufferedReader(new FileReader(new File(
					parameterfile)));		
			currLine = input.readLine();
			while((currLine = input.readLine())!= null){
				String[] row = currLine.split(",");
				// only add data for the respective microenvironment
				if(row[3].trim().equals(microenvironment)){
					float[] data;
					//for fraction data
					if(row[4].trim().equals("none")){
						// create empty float array
						data = new float[8];
					}
					else{
						// create empty float array
						data = new float[2];	
						para.set_parameter_distribution(row[0].trim(), row[4].trim());
					}
					para.set_parameter(row[0].trim(), data);
				}
			}
		}
		catch(Exception e){
			System.out.println("Exception during reading parameters csv file: "+parameterfile+", \n "+e.toString());
			System.out.println(currLine);
			LogFile.logger.log(Level.SEVERE,"Exception during reading parameters csv file: "+parameterfile+", \n "+e.toString());
			LogFile.logger.log(Level.INFO,currLine);
		}	
		return para;
	}
	
	// method to read parameter distribution details
	private IndoorParameters readParameterDistributionFile(String filename, String me, String country_id, 
			String pollutant, String scenario, IndoorParameters para){
		//IndoorParameters paras = new IndoorParameters(country_id, me);
		String row = "";
		try{
			BufferedReader input = new BufferedReader(new FileReader(new File(
					filename)));		
			row = input.readLine();
			String parameter = "";
			
			// loop through lines
			while ((row = input.readLine()) != null) 
			{	
				// header: parameter_id,pollutant_id,country_id,microenvironment_id,scenario_id,reference,distribution_parameter,value
				String[] st = row.split(",");
				parameter = st[0].trim();
				// only enter parameters for desired country
				if(st[2].trim().equals(country_id)||st[2].trim().equals("EU")){
					
					// parse next parameter only for desired microenvironment, scenario and pollutant
					if(scenario.equals(st[4].trim())&&pollutant.equals(st[1].trim())&&(me.equals(st[3].trim()))){
						int index = 1;
						if(st[6].trim().contains("mean")||st[6].trim().contains("min")){
							index = 0;
						}
						para.update_parameter(parameter, Float.parseFloat(st[7].trim()), index);
//						float[] data = new float[2];
//						String[] meta = new String[2];
//						// get next row with the second parameter
//						row = input.readLine();
//						String[] st2 = row.split(delimiter);
					
						// parse parameter values for uniform distribution
//						if(distribution.equals("unif")){
//							if(st[6].trim().contains("min")){
//								data[0] = Float.parseFloat(st[7].trim());
//								data[1] = Float.parseFloat(st2[7].trim());
//								meta[0] = st[6];
//								meta[1] = st2[6];
//							}
//							else{
//								data[0] = Float.parseFloat(st2[7].trim());
//								data[1] = Float.parseFloat(st[7].trim());	
//								meta[0] = st2[6];
//								meta[1] = st[6];
//							}						
//						}
						// parse parameter values for normal and lognormal distribution
//						else{
//							if(st[6].trim().contains("mean")){
//								data[0] = Float.parseFloat(st[7].trim());
//								data[1] = Float.parseFloat(st2[7].trim());
//								meta[0] = st[6];
//								meta[1] = st2[6];
//							}
//							else{
//								data[0] = Float.parseFloat(st2[7].trim());
//								data[1] = Float.parseFloat(st[7].trim());
//								meta[0] = st2[6];
//								meta[1] = st[6];
//							}							
//						}
//						paras.set_parameter(para, data); 							
//						paras.set_parameter_metadata(para, meta);
					}
				}
			}
		}
		catch(Exception e){
			System.out.println("Exception during reading parameters csv file: "+filename+", \n "+e.toString());
			System.out.println(row);
			LogFile.logger.log(Level.SEVERE,"Exception during reading parameters csv file: "+filename+", \n "+e.toString());
			LogFile.logger.log(Level.INFO,row);
		}	
		
		return para;
	}
	
	// method to read parameter fraction details
	private IndoorParameters readParameterFractionFile(String filename, IndoorParameters para,
			String country_id, String pollutant, String scenario, Hashtable<String,String> subgroups){
		String row = "";
		try{
			BufferedReader input = new BufferedReader(new FileReader(new File(
					filename)));		
			row = input.readLine();
//			int group = 0;
//			float[] data = new float[8];
			
			// loop through lines
			while ((row = input.readLine()) != null) 
			{		
				// header: parameter_id,pollutant_id,country_id,group_id,scenario_id,reference,value
				String[] st = row.split(",");
//				String[] meta_f_cig = new String[8];
				// only enter parameters for desired country
				if(st[1].trim().equals(country_id)||st[1].trim().equals("EU")){	
				
					// parse next parameter only for desired scenario and pollutant
					if(scenario.equals(st[3].trim())){
						
						// first parameter which only has one value per country
						if(st[0].trim().equals("f_bio")){
							para.update_parameter(st[0].trim(), Float.parseFloat(st[5].trim()), 0);
							para.set_subgroup(st[2].trim(), 8); // add subgroup "all"
//							String[] meta = {st[4]};
//							para.set_parameter_metadata(st[0].trim(), meta);
						}
						
						// second parameter which has 8 values per country
						else if(st[0].trim().equals("f_cig")){
							// first fill float array
							int id = Integer.parseInt(subgroups.get(st[2]));
//							data[id] = Float.parseFloat(st[5].trim());
							para.update_parameter(st[0].trim(), Float.parseFloat(st[5].trim()), id);
							para.set_subgroup(st[2].trim(), id);
//							meta_f_cig[id] = st[4].trim();						
//							if(group<7){							
//								group++;
//							}
							// then add float array to parameters list
//							else if(group==7){
//								para.set_parameter("f_cig", data);
//								para.set_parameter_metadata("f_cig", meta_f_cig);
//								group = 0;
//							}	
						}
					}
				}
			}
		}
		catch(Exception e){
			System.out.println("Exception during reading parameters csv file: "+filename+", \n "+e.toString());
			System.out.println(row);
			LogFile.logger.log(Level.SEVERE,"Exception during reading parameters csv file: "+filename+", \n "+e.toString());
			LogFile.logger.log(Level.INFO,row);
		}	
		return para;
	}
	
	// **** ECOSENSE PARSING METHOD ****
	public ArrayList<String[]> readEcoSense (String filename, String pollutant, String scenario, String delimiter){		
		ArrayList<String[]> data = new ArrayList<String[]>();
		String row = "";
		
		try{
			BufferedReader input = new BufferedReader(new FileReader(new File(
					filename)));
			
			row = input.readLine();
			String[] cols = row.split(delimiter);
			int p = 0;
			
			// get column with the respective 
			for(int i=1;i<cols.length;i++){
				if(cols[i].trim().equals(pollutant))
					p=i;
			}
			
			// loop through lines
			while ((row = input.readLine()) != null) {			
				String[] st = row.split(delimiter);
				// EcoSense columns: i50_j50; i50; j50; Pollutant_1; Pollutant_2; Pollutant_3
				// database columns: pollutant_id, emep50_id, scenario_id, value
				String[] temp = {pollutant, (st[1].trim()+"_"+st[2].trim()), scenario, st[p].trim()};
				// add row as it is to the data array
				data.add(temp);		
			}
		}
		catch(Exception e){
			System.out.println("Exception during reading csv file: "+filename+", \n :"+e.getMessage());
			LogFile.logger.log(Level.SEVERE,"Exception during reading csv file: "+filename+", \n :"+e.getMessage());
			LogFile.logger.log(Level.INFO,row);
		}
		return data;
	}
	
	// **** GENERIC TABLE PARSING METHOD ****
	public ArrayList<String[]> readTable (String filename, String delimiter){
//		Hashtable<String,String> hash = new Hashtable<String,String>();			
		ArrayList<String[]> table = new ArrayList<String[]>();
//		if(filename.endsWith("subgroup.csv"))
//			subgroups = new ArrayList<PopulationSubgroup>();
		String row = "";
		
		try{
			BufferedReader input = new BufferedReader(new FileReader(new File(
					filename)));
			row = input.readLine();
			
			// loop through lines
			while ((row = input.readLine()) != null) {	
				//System.out.println(row);
				String[] st = row.split(delimiter);
				// add row as it is to the data array
				table.add(st);
				
//				// fill hashtables (countries, subgroups, parameter_distribution_types
//				if(filename.endsWith("parameter.csv"))
//					hash.put(st[0].trim()+"_"+st[3].trim(),st[4].trim());
//				else if(filename.endsWith("subgroup.csv")){
//					PopulationSubgroup group = new PopulationSubgroup(st[0].trim(), 
//							Integer.parseInt(st[1].trim()));
//					group.set_gender(st[2].trim());
//					group.set_employment_status(st[3].trim());
//					group.set_min_age(Integer.parseInt(st[4].trim()));
//					group.set_max_age(Integer.parseInt(st[5].trim()));
//					subgroups.add(group);
//				}				
//				else if(filename.endsWith("country.csv"))
//					hash.put(st[0].trim(),st[1].trim());	
			}
		}
		catch(Exception e){
			System.out.println("Exception during reading csv file: "+filename+", \n :"+e.getMessage());
			LogFile.logger.log(Level.SEVERE,"Exception during reading csv file: "+filename+", \n :"+e.getMessage());
			LogFile.logger.log(Level.INFO,row);
		}
		
//		if(filename.endsWith("country.csv"))
//			countries = hash;
//		else if(filename.endsWith("parameter.csv"))
//			parameter_distribution_type = hash;
	//	System.out.println(table.size()+" rows");
		return table;
	}
	
	// method for other concentration or population files without grid cell information
	public ArrayList<Float> readSimpleFile(String filename, String delimiter){
		ArrayList<Float> data = new ArrayList<Float>();
		try{
			BufferedReader input = new BufferedReader(new FileReader(new File(
					filename)));
			
			// go through lines
			String row;
			while ((row = input.readLine()) != null) {		
				String[] s = row.split(delimiter);
				int i=0;
				// add values for this row
				while(i<s.length){
					data.add(Float.parseFloat(s[i].trim()));
					i++;
				}
			}
			System.out.println("Number of observations: "+data.size());
			//System.out.println("First observations: "+data.get(0));
		}
		catch(Exception e){
			System.out.println("Exception during reading PAREST csv file: "+filename+", \n :"+e.getMessage());
			LogFile.logger.log(Level.SEVERE, "Exception during reading PAREST csv file: "+filename+", \n :"+e.getMessage());
			
		}	
		
		return data;
	}

	// **** GETTERS FOR HASHTABLES ****	
//	public ArrayList<PopulationSubgroup> subgroupList(){
//		return subgroups;
//	}
//	
//	public Hashtable<String,String> countriesHash(){
//		return countries;
//	}
//	
//	public Hashtable<String,String> parameterDistributionTypeHash(){
//		return parameter_distribution_type;
//	}
	
	// **** USABILITY METHODS ****
	// method to convert groups into a common format
	public String groupConverter(String parsedGroup){
		String group = "";
		if(parsedGroup.contains("nonwork")||parsedGroup.contains("non")){
			if(parsedGroup.contains("F")||parsedGroup.contains("f"))
				group = "Fnonwork";
			else
				group = "Mnonwork";
		}
		else if(parsedGroup.contains("work")||parsedGroup.contains("Work")){ 
			if(parsedGroup.contains("F")||parsedGroup.contains("f"))
				group = "Fwork";
			else
				group = "Mwork";
		}
		else if(parsedGroup.contains("14")||parsedGroup.contains("15")){
			if(parsedGroup.contains("M")||parsedGroup.contains("m"))
				group = "M15";
			else
				group = "F15";
		}
		else if(parsedGroup.contains("65")||parsedGroup.contains("64")||parsedGroup.contains("PLUS")){
			if(parsedGroup.contains("M")||parsedGroup.contains("m"))
				group = "M65";
			else
				group = "F65";
		}
		else{
		//	System.out.println("No subgroup for: "+parsedGroup);
			group = "all";
		}		
		return group;
	}

	//TODO: rewrite method using the PopulationSubgroup ArrayList
	// method to build groups depending on age, emp and sex
	public String groupBuilder(String sex, int age, int emp, ArrayList<PopulationSubgroup> subgroups){
		String group = "";
		// loop through subgroups
		for(int i=0; i<subgroups.size(); i++){
			PopulationSubgroup sg = subgroups.get(i);
			if(age<=sg.get_max_age()&&age>=sg.get_min_age()&&sex.startsWith(sg.get_gender()))
				if(sg.get_employment_status().equals("all")){
					group = sg.get_name();
					i=subgroups.size();
					return group;
				}
				else if(this.employmentConverter(emp).equals(sg.get_employment_status())){
					group = sg.get_name();
					i=subgroups.size();
					return group;
				}
		}
		if(group.equals(""))
			return null;
		return group;	
//		if(age<15){
//			if(sex.equals("male"))
//				group="M15";
//			else if(sex.equals("female"))
//				group="F15";
//			else // if no sex is available, return null
//				return null;
//		}
//		else if(age>14&&age<65){
//			if(sex.equals("male")&&emp>0)
//				group="Mwork";
//			else if(sex.equals("male")&&emp==0)
//				group="Mnonwork";
//			else if(sex.equals("female")&&emp>0)
//				group="Fwork";
//			else if(sex.equals("female")&&emp==0)
//				group="Fnonwork";
//			else // if no sex or emp is available, return null
//				return null;
//		}
//		else if(age>64){
//			if(sex.equals("male"))
//				group="M65";
//			else if(sex.equals("female"))
//				group="F65";
//			else // if no sex is available, return null
//				return null;
//		}
	//	System.out.println(group);

	}
	
	// method to convert gender into a String
	private String genderConverter(String parsedGender){
		String gender = "";
		if(parsedGender.equals("1"))
			gender = "male";
		else if(parsedGender.equals("2"))
			gender = "female";
		else
			gender = "unknown";
		return gender;
	}
	
	// method to convert employment status into a string
	private String employmentConverter(int parsedEmpStatus){
		String emp = "";
		if(parsedEmpStatus==0)
			emp = "no";
		else if(parsedEmpStatus>0)
			emp = "yes";
		else
			emp = "unknown";
		return emp;
	}
	
	// method to calculate time spent per microenvironment
	private Hashtable<String,Integer> get_ME(String[] row, Hashtable<String, Integer> columns){
		Hashtable<String, Integer> me = new Hashtable<String,Integer>();
		
		// get values of all 41 activities
		int[] av = new int[42];
		av[0] = 0;
 		for (int i=1; i<42; i++){
			int av_i = Integer.parseInt(row[columns.get("av"+i)].trim());
			if(av_i>=0)
				av[i] = av_i;
			else
				av[i] = 0;
		}
		//home: av2+av6+av7+av11+ av13+ av15+ av16+ av30+ av31+ av32+ av33+ av34+ av35+ av36+ av39
		int home = 
			av[2]+av[2]+av[6]+av[7]+av[11]+av[13]+av[15]+av[16]
			+av[30]+av[31]+av[32]+av[33]+av[34]+av[35]+av[36]+av[39];

		//work: av1+av3+av4+av8
		int work = av[1]+av[3]+av[4]+av[8];		

		//other: av10+ av14+ av19+ av20+ av22 +av23+av24+av25+av26+av27+av28+ av29+ av37+ av38+ av40+ av41
		int other = 
			av[10]+av[14]+av[19]+av[20]+av[22]+av[23]+av[24]+av[25]
		    +av[26]+av[27]+av[28]+av[29]+av[37]+av[38]+av[40]+av[41];

		//outdoor: av9+av18+av21
		int outdoor = av[9]+av[18]+av[21];		

		//travel: av5+av12+av17
		int travel = av[5]+av[12]+av[17];
		
		int sum = home+work+travel+other+outdoor;
		// check if sum is correct
		if(sum==1440){
		//	System.out.println("Activity sum is correct.");
			me.put("home", home);
			me.put("work", work);
			me.put("other", other);
			me.put("travel", travel);
			me.put("outdoor", outdoor);
			return me;
		}
		else{
			System.out.println("Activity sum is not correct: "+sum);
			System.out.println(row[columns.get("home")].trim()+" TO "+home);
			System.out.println(row[columns.get("work")].trim()+" TO "+work);
			System.out.println(row[columns.get("other")].trim()+" TO "+other);
			System.out.println(row[columns.get("travel")].trim()+" TO "+travel);
			System.out.println(row[columns.get("outdoor")].trim()+" TO "+outdoor);
			return null;
		}	
	}
}
