package de.ifgi.exposure.io;

import java.io.BufferedWriter;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.logging.Level;

import de.ifgi.exposure.datamodel.MTUScountry;
import de.ifgi.exposure.datamodel.PopulationSubgroup;
import de.ifgi.exposure.control.LogFile;

public class CsvWriter {
	private String newFile;
	private BufferedWriter writer;
	
	public CsvWriter(String filedirectory){
		newFile = filedirectory;
	}
	
	//public CsvWriter(String filedirectory, )
	
	public boolean writeIndividualResults(float[][] results, MTUScountry mtus){ 
		String currentLine = "";
		try{
		BufferedWriter out = new BufferedWriter(new OutputStreamWriter(
				new FileOutputStream(newFile)));
		
		// loop through individuals
		for (int i=-1; i<mtus.get_size(); i++){	
			
			// for the first 4 columns write individual details
			// write header in the first row
			if(i==-1)
				out.write("group, diary_id, ETS, biomass");
			// write data in the following individual rows
			else if(i>-1)
				out.write(mtus.get_group(i)+","+mtus.get_diaryID(i)+","+
					mtus.get_smokerRealisations()[i]+","+mtus.get_biomassRealisation()[i]);
			
			// loop through realisations (columns)
			for(int j=0; j<results[0].length; j++){
				// write header
				if(i==-1)
					out.write(",i_"+(j+1));
				
				// write results
				else
					out.write(","+(float)results[i][j]);
			}
			out.newLine();
		}
		out.close();
		}
		
		catch(Exception e){
			System.out.println("Exception during writing file: "+ newFile+"\n"+e.getMessage());
			LogFile.logger.log(Level.SEVERE,"Exception during writing file: "+ newFile+"\n"
					+e.getMessage()+ " Cause "+ e.getCause());
			LogFile.logger.log(Level.INFO, "Line: "+currentLine);
			return false;
		}
		return true;
	}
	
	public boolean writeSubgroupResults(float[][] results, ArrayList<PopulationSubgroup> subgroups){ 
		String currentLine = "";
		try{
		BufferedWriter out = new BufferedWriter(new OutputStreamWriter(
				new FileOutputStream(newFile)));	
			
		// write header
		out.write("group");
		for(int j=0; j<results[0].length; j++){
			out.write(",i_"+(j+1));
		}
		out.newLine();

		//write results
		for(int i=0; i<subgroups.size(); i++){
			
			// write first column
			out.write(subgroups.get(i).get_name());
			// loop through realisations (columns)
			for(int j=0; j<results[0].length; j++){
				// write results
				out.write(","+(float)results[i][j]);
			}
			out.newLine();
		}
		out.close();
		}
		
		catch(Exception e){
			System.out.println("Exception during writing file: "+ newFile+"\n"+e.getMessage());
			LogFile.logger.log(Level.SEVERE,"Exception during writing file: "+ newFile+"\n"
					+e.getMessage()+ " Cause "+ e.getCause());
			LogFile.logger.log(Level.INFO, "Line: "+currentLine);
			return false;
		}
		return true;
	}
	
	public boolean writeTotalPopResults(float[][] results){ 
		String currentLine = "";
		try{
		BufferedWriter out = new BufferedWriter(new OutputStreamWriter(
				new FileOutputStream(newFile)));	
			
		// write header
		out.write("group");
		for(int j=0; j<results[0].length; j++){
			out.write(",i_"+(j+1));
		}
		out.newLine();

		// write results			
		// write first column
		out.write("Total population");
		// loop through realisations (columns)
		for(int j=0; j<results[0].length; j++){
			// write results
			out.write(","+(float)results[0][j]);
		}
		out.newLine();
		out.close();
		}
		
		catch(Exception e){
			System.out.println("Exception during writing file: "+ newFile+"\n"+e.getMessage());
			LogFile.logger.log(Level.SEVERE,"Exception during writing file: "+ newFile+"\n"
					+e.getMessage()+ " Cause "+ e.getCause());
			LogFile.logger.log(Level.INFO, "Line: "+currentLine);
			return false;
		}
		return true;
	}
	
	public boolean writeGridSubgroupResults(float[][] results, ArrayList<String[]> gridbasedIDs){ 
		String currentLine = "";
		try{
		BufferedWriter out = new BufferedWriter(new OutputStreamWriter(
				new FileOutputStream(newFile)));
		//System.out.println("Length of IDs: "+gridbasedIDs.size());
		
		int iSize = gridbasedIDs.size();///results[0].length;
		// loop through grid cells*subgroups		
		for(int i=-1; i<iSize; i++){
			
			// for the first 2 columns write grid cell id and subgroup id
			// write header in the first row
			if(i==-1)
				out.write("emep_id, group_id");
			// write data in the following individual rows
			else if(i>-1){
				// loop through group ids
				out.write(gridbasedIDs.get(i)[0]+","+gridbasedIDs.get(i)[1]);
			}
			
			// loop through realisations (columns)
			for(int j=0; j<results[0].length; j++){
				// write header
				if(i==-1)
					out.write(",i"+(j+1));
				
				// write results
				else{
					out.write(","+(float)results[i][j]);
					currentLine = "gridcell count "+i+": "+results[i][j]+" for gridcell "+gridbasedIDs.get(i)[0]+" and iteration "+(j+1);
				}
			}
			out.newLine();
		}
		out.close();
		}
		
		catch(Exception e){
			System.out.println("Exception during writing file: "+ newFile+"\n"+e.getMessage());
			LogFile.logger.log(Level.SEVERE,"Exception during writing file: "+ newFile+"\n"
					+e.getMessage()+ " Cause "+ e.getCause());
			LogFile.logger.log(Level.INFO, "Line: "+currentLine);
			System.out.println("Line: "+currentLine);
			return false;
		}
		return true;
	}
	
	public boolean writeGridTotalPopResults(float[][] results, ArrayList<String> gridIDs){ 
		String currentLine = "";
		try{
		BufferedWriter out = new BufferedWriter(new OutputStreamWriter(
				new FileOutputStream(newFile)));
		//System.out.println("Length of IDs: "+gridbasedIDs.size());
		
		int iSize = gridIDs.size();///results[0].length;
		// loop through grid cells*subgroups		
		for(int i=-1; i<iSize; i++){
			
			// for the first 2 columns write grid cell id and subgroup id
			// write header in the first row
			if(i==-1)
				out.write("emep_id");
			// write data in the following individual rows
			else if(i>-1){
				// loop through group ids
				out.write(gridIDs.get(i));
			}
			
			// loop through realisations (columns)
			for(int j=0; j<results[0].length; j++){
				// write header
				if(i==-1)
					out.write(",i"+(j+1));
				
				// write results
				else{
					out.write(","+(float)results[i][j]);
					currentLine = "gridcell count "+i+": "+results[i][j]+" for gridcell "+gridIDs.get(i)+" and iteration "+(j+1);
				}
			}
			out.newLine();
		}
		out.close();
		}
		
		catch(Exception e){
			System.out.println("Exception during writing file: "+ newFile+"\n"+e.getMessage());
			LogFile.logger.log(Level.SEVERE,"Exception during writing file: "+ newFile+"\n"
					+e.getMessage()+ " Cause "+ e.getCause());
			LogFile.logger.log(Level.INFO, "Line: "+currentLine);
			System.out.println("Line: "+currentLine);
			return false;
		}
		return true;
	}
	
	public boolean writeRealisationResults(float[][] results, String columnName, ArrayList<String> ids){ 
		try{
			BufferedWriter out = new BufferedWriter(new OutputStreamWriter(
					new FileOutputStream(newFile)));
		
			// write header in the first row
			out.write(columnName);
			for(int i=0; i<results[0].length; i++){
				out.write(",i"+(i+1));
			}
			out.newLine();
			
			// loop through ArrayList	
			for(int i=0; i<ids.size(); i++){		
				// write data in the following individual rows
				out.write(ids.get(i));
				// loop through realisations (columns)
				for(int j=0; j<results[0].length; j++){
					// write results
					out.write(","+(float)results[i][j]);
				}
				out.newLine();
			}
			out.close();
		}
		
		catch(Exception e){
			System.out.println("Exception during writing file: "+ newFile+"\n"+e.getMessage());
			LogFile.logger.log(Level.SEVERE,"Exception during writing file: "+ newFile+"\n"
					+e.getMessage()+ " Cause "+ e.getCause());
			return false;
		}
		return true;
	}
	
	public boolean writeSummaryResults(ArrayList<float[]> results, String columnName, ArrayList<String> ids){ 
		try{
			BufferedWriter out = new BufferedWriter(new OutputStreamWriter(
					new FileOutputStream(newFile)));
		
			// write header in the first row
			out.write(columnName+",mean,sd");
			out.newLine();
			
			// loop through ArrayList	
			for(int i=0; i<ids.size(); i++){		
				// write data in the following individual rows
				out.write(ids.get(i)+","+results.get(i)[0]+","+results.get(i)[1]);
				out.newLine();
			}
			out.close();
		}
		
		catch(Exception e){
			System.out.println("Exception during writing file: "+ newFile+"\n"+e.getMessage());
			LogFile.logger.log(Level.SEVERE,"Exception during writing file: "+ newFile+"\n"
					+e.getMessage()+ " Cause "+ e.getCause());
			return false;
		}
		return true;
	}
	
	public boolean writeParameters(ArrayList<String[]> paras){
		String currentLine = "";
		try{
			BufferedWriter out = new BufferedWriter(new OutputStreamWriter(
					new FileOutputStream(newFile)));
			
			// write header
			out.write("parameter_id, microenvironment_id, iteration, value");
			out.newLine();
			
			for(int i=0; i<paras.size(); i++){
				String row = "";
				for(int j=0; j<paras.get(i).length; j++){
					if(j==0)
						row = paras.get(i)[j];
					else
						row = row + ", "+paras.get(i)[j];
				}
				out.write(row);
				out.newLine();
			}
			out.close();
		}
		
		catch(Exception e){
			System.out.println("Exception during writing file: "+ newFile+"\n"+e.getMessage());
			LogFile.logger.log(Level.SEVERE,"Exception during writing file: "+ newFile+"\n"
					+e.getMessage()+ " Cause "+ e.getCause());
			LogFile.logger.log(Level.INFO, "Line: "+currentLine);
			return false;
		}
		return true;
	}
	
	public boolean writeLogFile(ArrayList<String> logs){
		String currentLine = "";
		try{
			BufferedWriter out = new BufferedWriter(new OutputStreamWriter(
					new FileOutputStream(newFile)));
			for(int i=0; i<logs.size(); i++){
				out.write(logs.get(i));
				out.newLine();
			}
			out.close();
		}
		catch(IOException e){
			System.out.println("Exception during writing file: "+ newFile+"\n"+e.getMessage());
			LogFile.logger.log(Level.SEVERE,"Exception during writing file: "+ newFile+"\n"
					+e.getMessage()+ " Cause "+ e.getCause());
					LogFile.logger.log(Level.INFO, "Line: "+currentLine);
			return false;
		}
		return true;
	}
	
	public void openOutputStream() throws IOException{
		// new OutputWriter instance
			writer= new BufferedWriter(new OutputStreamWriter(
					new FileOutputStream(newFile)));
	}

	public void writeHeader(String header) throws IOException{
		// write header "parameter, value, iteration"
		writer.write(header);
		writer.newLine();
	}
	
	public void closeOutputStream() throws IOException{
		writer.close();
	}
}
