package de.ifgi.exposure.io;

import java.io.File;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.logging.Level;

import de.ifgi.exposure.datamodel.EMEPgridCountry;
import de.ifgi.exposure.datamodel.IndoorParameters;
import de.ifgi.exposure.datamodel.MTUScountry;
import de.ifgi.exposure.datamodel.PopulationSubgroup;
import de.ifgi.exposure.datamodel.SimulationParameters;
import de.ifgi.exposure.db.DatabaseAccess;
import de.ifgi.exposure.db.DatabaseFeeder;
import de.ifgi.exposure.control.LogFile;
import de.ifgi.exposure.datamodel.Metadata;

/***
 * Class to handle data input and database filling
 * @author Lydia Gerharz
 *
 */
public class DataHandler {
	private CsvHandler ch;
	private DatabaseFeeder df;
	private DatabaseAccess da;
	
	// simulation parameters
	String pollutant, scenario, country_id, errorMessage;
	boolean fillDB;
	String[] pollutantList, scenarioList, scenarioIDList, countryList, countryIDList;

	public DataHandler(SimulationParameters sim, boolean fillDB, boolean accessDB){
		this.pollutant = sim.get_pollutant();
		this.scenario = sim.get_scenario();
		this.country_id = sim.get_country_id();
		this.fillDB = fillDB;
		
		ch = new CsvHandler();
		if(fillDB){
			df = new DatabaseFeeder(sim.get_JDBC_Driver(), sim.get_dbURL(),
					sim.get_dbUser(), sim.get_dbPassword());
			df.connect();	
		}
		else if(accessDB){
			da = new DatabaseAccess(sim.get_JDBC_Driver(), sim.get_dbURL(),
					sim.get_dbUser(), sim.get_dbPassword());
			da.connect();
		}
	}
	
	public DataHandler(SimulationParameters sim){
		this.pollutant = sim.get_pollutant();
		this.scenario = sim.get_scenario();
		this.country_id = sim.get_country_id();
		df = new DatabaseFeeder(sim.get_JDBC_Driver(), sim.get_dbURL(), sim.get_dbUser(), sim.get_dbPassword());
		df.connect();	
	}
	
	// general constructor for checking folders and files
	public DataHandler(){
		
	}
	
	// **** METHODS FOR CHECKS ****
	public boolean checkDBconnection(String jdbcDriver, String dbURL, String dbUser, String dbPassword, boolean readDB){
		da = new DatabaseAccess(jdbcDriver, dbURL, dbUser, dbPassword);
		boolean success =  da.connect();
		// get countries, pollutants and scenarios if data is read from database
		if(success&&readDB){
			ArrayList<String[]> country = da.select_country();
			ArrayList<String[]> pol = da.select_pollutant();
			ArrayList<String[]> scen = da.select_scenario();
			
			// get countries
			// sort countries by name
			System.out.println(country.size()-1+" countries found in database.");
			int j = 0;
			countryList = new String[country.size()-1]; countryIDList = new String[country.size()-1];
			for(int i=0; i<countryList.length; i++){
				//if(!country.get(i)[0].equals("EU")){
					countryList[j] = country.get(i)[1];
					countryIDList[j] = country.get(i)[0];
					j++;
				//}				
			}
			
			// get pollutants
			pollutantList = new String[pol.size()];
			for(int i=0; i<pol.size(); i++){
				pollutantList[i] = pol.get(i)[0];
			}
			
			// get scenarios
			scenarioList = new String[scen.size()]; scenarioIDList = new String[scen.size()];
			for(int i=0; i<scen.size(); i++){
				scenarioList[i] = scen.get(i)[1]+", "+scen.get(i)[2];
				scenarioIDList[i] = scen.get(i)[0];
			}
		}
		return success;
	}
	
	public boolean checkLAMAFolder(String folder){
		boolean success = false;
		int simpleFileCount = 0; int inputFileCount = 0;
		SimulationParameters sim = new SimulationParameters();
		sim.set_LAMAfolder(folder);
		ch = new CsvHandler();
		
		// test folder and get countries, scenarios and pollutants from csv files
		try{
			// files from input folder
			ArrayList<File> inputfiles = this.readCsvFromDirectory(sim.get_inputdataFolder());
			for(int i=0; i<inputfiles.size(); i++){
				String file = inputfiles.get(i).toString();
			//	System.out.println(file+", "+sim.get_timeuseFile());
				// check if found file corresponds to desired input files
				if(file.equals(sim.get_timeuseFile())||file.equals(sim.get_concentrationFile())||
						file.equals(sim.get_parameterDistributionFile())||file.equals(sim.get_parameterFractionFile())||
						file.equals(sim.get_populationFile())||file.equals(sim.get_urban_incrementFile()))
						inputFileCount++;
			}
			// files from simple folder
			ArrayList<File> simplefiles = this.readCsvFromDirectory(sim.get_metadataFolder());
			for(int i=0; i<simplefiles.size(); i++){
				String name = simplefiles.get(i).getName();
				// check if found file corresponds to desired input files
				if(name.startsWith("micro")||name.startsWith("para")||name.startsWith("sub"))
						simpleFileCount++;
				else if(name.startsWith("country")){ // get arrays from countries file
					simpleFileCount++;
					ArrayList<String[]> array = this.readMetadataTable(sim.get_metadataFolder(), name);
					
					// get countries
					countryList = new String[array.size()]; countryIDList = new String[array.size()];
					for(int j=0; j<array.size(); j++){
						//System.out.println(array.get(j)[0]);
					//	if(!array.get(j)[0].equals("EU")){
							countryList[j] = array.get(j)[1];
							countryIDList[j] = array.get(j)[0];
					//	}
					}
				}
				else if(name.startsWith("pollutant")){// get arrays from pollutant file
					simpleFileCount++;
					ArrayList<String[]> array = this.readMetadataTable(sim.get_metadataFolder(), name);
					
					// get pollutants
					pollutantList = new String[array.size()];
					for(int j=0; j<array.size(); j++){
						pollutantList[j] = array.get(j)[0];
					}								
				}
				else if(name.startsWith("scenario")){// get arrays from scenario file
					simpleFileCount++;
					ArrayList<String[]> array = this.readMetadataTable(sim.get_metadataFolder(), name);
					// get scenarios
					scenarioList = new String[array.size()]; scenarioIDList = new String[array.size()];
					for(int j=0; j<array.size(); j++){
						scenarioList[j] = array.get(j)[1]+", "+array.get(j)[3];
						scenarioIDList[j] = array.get(j)[0];
					}
				}
			}
			console("Input file count: "+ inputFileCount);
			console("Simple file count: "+ simpleFileCount);
			if(inputFileCount==6){
				if(simpleFileCount==6)
					success = true;
				else{
					console("Not all input files could be found...");
					errorMessage="Not all files in the \\input\\simple folder could be found. \n " +
							"Please make sure you've uploaded all six files. \n " +
							"Check the LAMA software documentation for more details.";
				}
			}
			else{
				console("Not all input files could be found...");
				errorMessage="Not all files in the \\input folder could be found. \n " +
						"Please make sure you've uploaded all six files. \n " +
						"Check the LAMA software documentation for more details.";
			}
		}
		catch(Exception e){
			System.out.println("Exception occured during the LAMAfolder test: "+e.getMessage());
			success = false;
		}
		
		return success;
	}
	
	public String returnErrorMessage(){
		return errorMessage;
	}
	
	// methods to get country, pollutant and scenario lists
	public ArrayList<String[]> get_Countries(){
		ArrayList<String[]> array = new ArrayList<String[]>();
		array.add(countryList);
		array.add(countryIDList);
		return array;
	}
	
	public String[] get_Pollutants(){
		return pollutantList;
	}
	
	public ArrayList<String[]> get_Scenarios(){
		ArrayList<String[]> array = new ArrayList<String[]>();
		array.add(scenarioList);
		array.add(scenarioIDList);
		return array;
	}
	
	//**** CSV READER METHODS ****
	private ArrayList<String[]> readMetadataTable(String folder, String file){
		ArrayList<File> files = this.readCsvFromDirectory(folder);
		ArrayList<String[]> data = null;
		try{		
		for (int i = 0; i < files.size(); i++) {
			// get tablename from file directory
			if(file.equals(files.get(i).getName())){
				data = ch.readTable(files.get(i).toString(),",");
				return data;
			}
		}
		}
		catch(Exception e){
			LogFile.logger.log(Level.SEVERE, "Exception during reading file from metadata folder. \n"
					+e.getMessage()+" Cause "+e.getCause());
		}
		return data;
	}
	
	public Metadata readMetadataFolder(String folder){
		Metadata meta = new Metadata();
		ArrayList<File> files = this.readCsvFromDirectory(folder);
		try{
		for (int i = 0; i < files.size(); i++) {
			// parse file
			ArrayList<String[]> data = ch.readTable(files.get(i).toString(),",");
					
			// get tablename from file directory
			String filename = files.get(i).getName();
			String tablename = filename.substring(0, (filename.length()-4));
					
			// add data to database
			if(fillDB)
				df.insertSimpleRows(data,tablename);		
			
			// fill metadata container
			if(tablename.equals("parameter")){
				Hashtable<String,String> hash = new Hashtable<String,String>();
				for(int j=0; j<data.size(); j++){
					hash.put(data.get(j)[0].trim(),data.get(j)[4].trim());
				}
		//		meta.set_parameter_distribution_hashtable(hash);
			}
			else if(tablename.equals("country")){
				Hashtable<String,String> hash = new Hashtable<String,String>();
				for(int j=0; j<data.size(); j++){
					hash.put(data.get(j)[0].trim(),data.get(j)[1].trim());
				}
				meta.set_country_hashtable(hash);
			}
			else if(tablename.equals("subgroup")){
				ArrayList<PopulationSubgroup> subgrouplist = new ArrayList<PopulationSubgroup>();
				Hashtable<String,String> subgrouphash = new Hashtable<String,String>();
				for(int j=0; j<data.size(); j++){
					PopulationSubgroup group = new PopulationSubgroup(data.get(j)[0].trim(), 
							Integer.parseInt(data.get(j)[1].trim()));
					group.set_gender(data.get(j)[2].trim());
					group.set_employment_status(data.get(j)[3].trim());
					group.set_min_age(Integer.parseInt(data.get(j)[4].trim()));
					group.set_max_age(Integer.parseInt(data.get(j)[5].trim()));
					subgrouplist.add(group);
					subgrouphash.put(data.get(j)[0], data.get(j)[1]);
				}
				meta.set_subgroup_list(subgrouplist);
				meta.set_subgroup_hashtable(subgrouphash);
			}	
			else if(tablename.equals("scenario")){
				meta.set_scenario_list(data);
			}
			else if(tablename.equals("microenvironment")){
				Hashtable<String,String> me_hash = new Hashtable<String,String>();
				for(int j=0; j<data.size(); j++){
					me_hash.put(data.get(j)[0], j+"");
				}
				meta.set_microenvironment_hashtable(me_hash);
			}
		}
		}
		catch(Exception e){
			LogFile.logger.log(Level.SEVERE, "Exception during reading from metadata directory. \n"
					+e.getMessage()+" Cause "+e.getCause());
			return meta;
		}
		return meta;
	}
	
	public Metadata getMetadataDB(){
		Metadata meta = new Metadata();
			// get countries, pollutants and scenarios if data is read from database	
			ArrayList<String[]> country = da.select_metadata("country",2);
			//ArrayList<String[]> pol = da.select_metadata("pollutant");
			ArrayList<String[]> scen = da.select_metadata("scenario",2);
			ArrayList<String[]> me = da.select_metadata("microenvironment",2);
			ArrayList<String[]> subgroups = da.select_metadata("subgroup",6);
			ArrayList<String[]> paras = da.select_metadata("parameter",5);
			
			// get parameters
			Hashtable<String,String> paraHash = new Hashtable<String,String>();
			for(int j=0; j<paras.size(); j++){
				paraHash.put(paras.get(j)[0].trim(),paras.get(j)[4].trim());
			}
		//	meta.set_parameter_distribution_hashtable(paraHash);
			
			// get subgroups
			ArrayList<PopulationSubgroup> subgrouplist = new ArrayList<PopulationSubgroup>();
			Hashtable<String,String> subgrouphash = new Hashtable<String,String>();
			for(int j=0; j<subgroups.size(); j++){
				PopulationSubgroup group = new PopulationSubgroup(subgroups.get(j)[0].trim(), 
						Integer.parseInt(subgroups.get(j)[1].trim()));
				group.set_gender(subgroups.get(j)[2].trim());
				group.set_employment_status(subgroups.get(j)[3].trim());
				group.set_min_age(Integer.parseInt(subgroups.get(j)[4].trim()));
				group.set_max_age(Integer.parseInt(subgroups.get(j)[5].trim()));
				subgrouplist.add(group);
				subgrouphash.put(subgroups.get(j)[0], subgroups.get(j)[1]);
			}
			meta.set_subgroup_list(subgrouplist);
			meta.set_subgroup_hashtable(subgrouphash);
			
			// get microenvironments
			Hashtable<String,String> me_hash = new Hashtable<String,String>();
			for(int j=0; j<me.size(); j++){
				me_hash.put(me.get(j)[0], j+"");
			}
			meta.set_microenvironment_hashtable(me_hash);
			
			// get countries
			// sort countries by name
			System.out.println(country.size()-1+" countries found in database.");

			Hashtable<String,String> c_hash = new Hashtable<String,String>();
			for(int j=0; j<country.size(); j++){
				c_hash.put(country.get(j)[0].trim(),country.get(j)[1].trim());
			}
			meta.set_country_hashtable(c_hash);
			
			int j = 0;

			// get scenarios
			meta.set_scenario_list(scen);
		
		return meta;
	}
	
	public MTUScountry readSingleMTUSfile(String filename, ArrayList<PopulationSubgroup> subgroups, Hashtable<String,String> microenvironments){
		MTUScountry mtus = ch.readMTUSfile(filename, country_id, ",", subgroups, microenvironments);
	//	mtus.set_subgroups(ch.subgroupHash());
		return mtus;
	}
	
	public IndoorParameters[] readSingleParameterFile(String parameterfile, String distfile, 
			String fracfile, Hashtable<String,String> subgroups){
		IndoorParameters[] par = ch.readParameterFile(parameterfile, distfile, fracfile,
				pollutant, scenario, country_id, subgroups);
		return par;
	}
	
	//**** READ EMEP ****
	public boolean readEMEPfile(boolean fromLocalSource, String emepfile){
		ArrayList<String[]> emepdata = null;
		try{
		EMEPdownload ed = new EMEPdownload();	
		if(fromLocalSource)
			emepdata = ed.parseEMEPfile(emepfile);
		else{
			emepdata = ed.downloadAndParseEMEPfile(emepfile);//
			System.out.println("EMEP data entries: "+emepdata.size());
		}
		System.out.println("Adding now EMEP data to table emep50_grid");
		df.insertSimpleRows(emepdata,"emep50_grid");
		}
		catch(Exception e){
			LogFile.logger.log(Level.SEVERE, "Exception during reading EMEP grid data. \n"
					+e.getMessage()+" Cause "+e.getCause());
			return false;
		}
		return true;
	}

	// **** READ POPULATION ****
	//TODO: check if function works correctly
	public EMEPgridCountry readPopulationFile (String file, int year, Hashtable<String,String> subgroups){	
		EMEPgridCountry emep = new EMEPgridCountry(country_id);
		try{
		ArrayList<String[]> data = ch.readTable(file, ",");
		Float[] groupdata = new Float[8];
		
		// structures for database filling
		// csv columns: Emep50_i_j,Subgroup,CountryID,Year,Rate,Pop_Value
		//Emep50_i_j	Subgroup	CountryID	Year	Rate	Pop_Value
		String columns = "emep50_id, group_id, country_id, year, growthrate, value";
		ArrayList<String[]> array = new ArrayList<String[]>();
		int groupCount = 0;
		String co = "";
		//System.out.println(data.size());
		for(int j=0; j<data.size(); j++){
			String row[] = data.get(j);
			// convert population groups to group_id
			//if(subgroups
			//row[1] = ch.groupConverter(data.get(j)[1]);
			row[1] = data.get(j)[1];
			array.add(row);		
			// get current country id
			if(!co.equals(row[2])){
			//	if(fillDB)
			//		console("New population data for "+row[2]);
				co = row[2];
			}
			//console(j+": "+row.length);
			// fill list with population per subgroup, only for medium growthrate and scenario year
			if(row[2].equals(country_id)&&row[4].equals("m")&&row[3].equals(year+"")){
				groupdata[Integer.parseInt(subgroups.get(row[1].trim()))] = Float.parseFloat(row[5]);
				groupCount++;
				
				// if last subgroup has been added, add to arraylist and restart String[]
				if(groupCount==8){ 
					emep.set_cellID(row[0]);
					emep.set_populationGroup(groupdata);
					groupdata = new Float[8];
					groupCount=0;
				}
			}
		}
		
		if(fillDB)
			df.insertRows(array, columns, "population");
		}
		catch(Exception e){
			LogFile.logger.log(Level.SEVERE, "Exception during reading population data \n"
					+e.getMessage()+" Cause "+e.getCause());
			return null;
		}
		return emep;
	}
	
	// **** READ CONCENTRATION ****
	public EMEPgridCountry readConcentrationFile(String concentrationFile, String urbanincrementFile, 
			EMEPgridCountry emep){
		try{
		// collect cell ids and concentration value to fill later emep object
		Hashtable<String,Float> cellIDs = new Hashtable<String,Float>(); 		
		// read urban increment file
		Hashtable<String, String[]> urbdata = this.readUrbanIncrement(urbanincrementFile);
		if(urbdata==null){ // stop method if no urban increment is read
			return null;
		}
		// parse file
		// *** FOR ECOSENSE FILES COMMENT THE NEXT LINE ***
		ArrayList<String[]> data = ch.readTable(concentrationFile, ",");
		// *** FOR ECOSENSE FILES UNCOMMENT THE NEXT LINE ***
		//ArrayList<String[]> datas = ch.readEcoSense(concentrationFile, pollutant, scenario, ";");
			
		// structures for database filling
		String concolumns = "pollutant_id, emep50_id, scenario_id, value";
		ArrayList<String[]> conArray = new ArrayList<String[]>();
				
		// loop through grid cells
		for(int j=0; j<data.size(); j++){
			// csv columns: EmepID;Emep_i;Emep_j;Pollutant;Pol_Value
			// new columns: EmepID,Emep_i,Emep_j, scenario_id, Pollutant,Pol_Value
			// new columns: Pollutant,Emep_i_j,scenario_id,Pol_Value
			// *** FOR ECOSENSE FILES COMMENT THE NEXT LINE ***
			String[] row = data.get(j);
//			String conData[] = {pollutant,(row[1]+"_"+row[2]),
//					row[3], row[5]};
			String conData[] = {row[0],	row[1], row[2], row[3]};
			// EcoSense columns: i50_j50; i50; j50; Pollutant_1; Pollutant_2; Pollutant_3
			// *** FOR ECOSENSE FILES UNCOMMENT THE NEXT LINE ***
			//String conData[] = data.get(j);
			
			// add data to array for database filling
			conArray.add(conData);
			// add data to hashtable with all cells for current scenario and pollutant
			if(scenario.equals(conData[2]))
				cellIDs.put(conData[1], Float.parseFloat(conData[3]));		
			}	
			System.out.println("Concentration cell length: "+cellIDs.size());
			
			if(fillDB)
				df.insertRows(conArray, concolumns, "concentration");
		
		// fill concentration array in EMEP object for the correct Ids 
		// Ids were taken from the population data
			ArrayList<String> emepIDs = emep.get_cellIDArray();
			ArrayList<Float[]> popIDs = emep.get_populationArray();
			// new ArrayLists where population cells without concentration data are removed
			ArrayList<Float[]> pop = new ArrayList<Float[]>();
			ArrayList<String> ids = new ArrayList<String>();
			
			for(int j=0; j<emepIDs.size(); j++){
				String emepID = emepIDs.get(j);
				
				// find concentration data for respective cell
				if(cellIDs.get(emepID)!=null) {
					emep.set_concentrationValue(cellIDs.get(emepID));
					pop.add(popIDs.get(j));
					ids.add(emepID);
				}
				else{
					console("No concentration data for cell "+emepIDs.get(j));
				}
				
				// get urban increment for the current cell
				String[] cellUI = urbdata.get(emepID);
				if(cellUI!=null){
					//UI array structure: pollutant_id, emep50_id, scenario_id, urban_pop, value
					emep.set_urban_incrementValue(Float.parseFloat(cellUI[4]));
					emep.set_urban_populationPercentage(Float.parseFloat(cellUI[3]));
				}
				else{ // default if no data has been entered
					emep.set_urban_incrementValue(0F);
					emep.set_urban_populationPercentage(0F);
				}
			}	
			// set cleaned lists to emep object
			emep.set_cellIDArray(ids);
			emep.set_populationArray(pop);
		}
		catch(Exception e){
			LogFile.logger.log(Level.SEVERE, "Exception during reading concentration data. \n"
					+e.getMessage()+" Cause "+e.getCause());
			return null;
		}
			return emep;
	}	
	
	 // @deprecated 
//	public EMEPgridCountry readFromDirectoryConcentration(String folder, String urbanincrementFile, 
//			EMEPgridCountry emep){
//		ArrayList<File> files = this.readCsvFromDirectory(folder);
//		Hashtable<String,Float> cellIDs = new Hashtable<String,Float>(); // to fill later emep object
//		
//		// read urban increment file
//		Hashtable<String, String[]> urbdata = this.readUrbanIncrement(urbanincrementFile);
//		
//		// loop through  concentration files
//		for (int i = 0; i < files.size(); i++) {
//				
//			// parse file
//			ArrayList<String[]> data = ch.readTable(files.get(i).toString(), ";");
//			
//			// get country from file directory
//			String country_id = files.get(i).getName().substring(0,2);
//			
//			// structures for database filling
//			String concolumns = "pollutant_id, emep50_id, scenario_id, value";
//			String gridcolumns = "emep50_id, country_id";
//			ArrayList<String[]> conArray = new ArrayList<String[]>();
//			ArrayList<String[]> gridArray = new ArrayList<String[]>();
//				
//			// loop through grid cells
//			for(int j=0; j<data.size(); j++){
//				String conData[] = {pollutant,(data.get(j)[0]+"_"+data.get(j)[1]),
//						scenario, data.get(j)[2]};
//				
//				// only add grid cells which are new
//				if(cellIDs.get(conData[1])==null){
//					conArray.add(conData);
//					// add data to temp array
//					cellIDs.put(conData[1], Float.parseFloat(conData[3]));
//				}
//				//else
//				//	System.out.println("EMEPid "+conData[1]+" already exists.");
//						
//				String gridData[] = {conData[1], country_id};
//				gridArray.add(gridData);						
//			}	
//			
//			// fill both tables
//			if(fillDB){
//				df.insertRows(conArray, concolumns, "concentration");
//				df.insertRows(gridArray, gridcolumns, "emep50_grid_country");
//			}	
//		}
//		
//		// fill concentration array for the correct IDS 
//		// Ids were taken from the population data
//			ArrayList<String> emepIDs = emep.get_cellIDArray();
//			ArrayList<Float[]> popIDs = emep.get_populationArray();
//			// new ArrayLists
//			ArrayList<Float[]> pop = new ArrayList<Float[]>();
//			ArrayList<String> ids = new ArrayList<String>();
//			
//			for(int j=0; j<emepIDs.size(); j++){
//				String emepID = emepIDs.get(j);
//				
//				// find concentration data for respective cell
//				if(cellIDs.get(emepID)!=null) {
//					emep.set_concentrationValue(cellIDs.get(emepID));
//					pop.add(popIDs.get(j));
//					ids.add(emepID);
//				}
//				else{
//					console("No concentration data for cell "+emepIDs.get(j));
//				}
//				
//				if(urbdata.get(emepID)!=null){
//					emep.set_urban_incrementValue(Float.parseFloat(urbdata.get(emepID)[3]));
//					emep.set_urban_populationPercentage(Float.parseFloat(urbdata.get(emepID)[2]));
//				}
//				else{ // default if no data has been entered
//					emep.set_urban_incrementValue(0F);
//					emep.set_urban_populationPercentage(0F);
//				}
//			}	
//			// set cleaned lists to emep object
//			emep.set_cellIDArray(ids);
//			emep.set_populationArray(pop);
//			return emep;
//	}	

	private Hashtable<String, String[]> readUrbanIncrement(String file){
		Hashtable<String,String[]> urbinc = new Hashtable<String,String[]>();
		try{
		ArrayList<String[]> data = ch.readTable(file, ",");
		
		// structures for database filling
		String columns = "pollutant_id, emep50_id, scenario_id, urban_pop, value";
		ArrayList<String[]> array = new ArrayList<String[]>();
		
		for(int j=0; j<data.size(); j++){
			// csv columns: i50_j50, scenario_id, PM25_UrbIn, urban_pop, rural_pop, urban_perc, rural_perc
			// new columns: parameter,i50_j50,scenario_id,urban_perc,PM25_UrbIn
			String row[] = {data.get(j)[0], data.get(j)[1],data.get(j)[2],
					data.get(j)[3],data.get(j)[4]};
			array.add(row);
			
			// add data to hashtable with all cells for current scenario and pollutant
			if(scenario.equals(row[2]))
				urbinc.put(row[1], row);
		}
		
		if(fillDB)
			df.insertRows(array, columns, "urban_increment");
		}
		catch(Exception e){
			LogFile.logger.log(Level.SEVERE, "Exception during reading urban increment data. \n"
					+e.getMessage()+" Cause "+e.getCause());
			return null;
		}
		return urbinc;
	}
	
	// **** READ TIME USE DATA ****
	public MTUScountry readMTUSfile (String file, String[] me, Metadata md){	
		Enumeration<String>  id =md.get_country_hashtable().keys();
		MTUScountry mtus = new MTUScountry(country_id);
		
		try{
		// parse all countries
		while (id.hasMoreElements()) {	
			String countryid =  id.nextElement();
			if(countryid.equals("EU"))
				countryid = id.nextElement();
			System.out.println("Time use country: "+countryid);
			MTUScountry mtusC = ch.readMTUSfile(file, countryid, ",", md.get_subgroup_list(), md.get_microenvironment_hashtable());
			if(countryid.equals(country_id)){
				System.out.println(countryid + " added to model.");
				mtus = mtusC;
			}
			int count = 0;
			
			// structures for database filling
			ArrayList<String[]> mtusdata =new ArrayList<String[]>();
			ArrayList<String[]> mtusTimeData = new ArrayList<String[]>();
			String diaryTimeColumns = "diary_id, microenvironment_id, value";
			
			// add actual MTUS data to database
			for(int i=0; i<mtusC.get_size(); i++){
				mtusdata.add(mtusC.get_diaryData(i));				
				
				// loop through microenvironments
				for(int j=0; j<me.length; j++){
				String[] mtusTime = {mtusC.get_diaryData(i)[0], 
						 me[ j], (mtusC.get_time(i, me[j])+"")};
					mtusTimeData.add(mtusTime);
				}	
				count++;	
			}					
				
			// fill data for this country into database
			df.insertSimpleRows(mtusdata, "diary");
			df.insertRows(mtusTimeData, diaryTimeColumns, "diary_time");		
			System.out.println("Added "+count+" diaries for "+countryid+".");
		}
		}
		catch(Exception e){
			LogFile.logger.log(Level.SEVERE, "Exception during reading time use data. \n"
					+e.getMessage()+" Cause "+e.getCause());
			return null;
		}
		return mtus;
	}
	
	// **** READ PARAMETERS ****
	public IndoorParameters[] readParameterFiles (String parameterfile, String distfile, String fracfile, Metadata md){	
		IndoorParameters[] par = new IndoorParameters[2];
		try{
		Enumeration<String>  id = md.get_country_hashtable().keys();		
		// database structures
		String paraDistColumns = "parameter_id, pollutant_id, country_id, microenvironment_id, " +
				"scenario_id, reference, distribution_parameter, value";
		String paraFracColumns = "parameter_id, country_id, group_id, " +
				"scenario_id, reference, value";
		
		// parse all countries
		while (id.hasMoreElements()) {		 
			String countryid =  id.nextElement();
			if(countryid.equals("EU")) // exclude EU
				countryid = id.nextElement();
			ArrayList<String[]> paraDistArray = new ArrayList<String[]>(), paraFracArray = new ArrayList<String[]>();
			
			// get parameters for home and work environment for the current country
			IndoorParameters[] paras = ch.readParameterFile(parameterfile, distfile, fracfile, 
					pollutant, scenario, countryid, md.get_subgroup_hashtable());
			if(countryid.equals(country_id))
				par = paras;
			int count = 0;
			System.out.println("Parameters parsed for "+countryid);
			
			if(fillDB){
			// go through both parameter files, home and work
			for(int c = 0; c<paras.length; c++)
			{
				// get parameter, description and distribution Hashtables to loop through
				Hashtable<String, float[]> pHash = paras[c].get_parameterHash();
				Hashtable<String, String> p_distHash = paras[c].get_parameter_distributionHash();
				Hashtable<String, String[]> p_descrHash = paras[c].get_parameter_metadataHash();
				
				Enumeration<String> p_id = pHash.keys();
			//	System.out.println(" Parameters: "+p.size()+"\n Distributions: "+p_dist.size()+"\n Descriptions: "+p_descr.size());
				
				// loop to add parameter data to database
				while(p_id.hasMoreElements()){					
					String para_id =  p_id.nextElement();
					
					// add distribution data
					if(p_distHash.get(para_id)!=null){
						float[] vals = pHash.get(para_id); // get parameter values
						for(int i=0; i<vals.length; i++){
							String[] distData = {para_id,pollutant,countryid,paras[c].get_me_id(),
									scenario, "",p_descrHash.get(para_id)[i],(pHash.get(para_id)[i]+"")};
							if(i==0&&para_id.equals("p_work"))
								distData[6] = "min";
							else if(i==1&&para_id.equals("p_work"))
								distData[6] = "max";
							else if(i==0&&!para_id.equals("p_work"))
								distData[6] = "mean";
							else
								distData[6] = "sd";
							paraDistArray.add(distData);
						}
					}
					// add fraction data
					else{
						// get values
						float[] g = pHash.get(para_id);
						
						// add all subgroups
						for(int i=0; i<g.length;i++){
							String[] fracData = {para_id, countryid,
									"", scenario, p_descrHash.get(para_id)[i], (g[i]+"")};;
							if(g.length==1)
								fracData[3] = paras[c].get_subgroup(8);
							else
								fracData[3] = paras[c].get_subgroup(i);	
							paraFracArray.add(fracData);
						}
					}
					count++;
				}
				
			}
			df.insertRows(paraDistArray, paraDistColumns, "parameter_distribution");
			df.insertRows(paraFracArray, paraFracColumns, "parameter_fraction");
			System.out.println("Added "+count+" parameters for "+countryid+".");
			}
		}
		}
		catch(Exception e){
			LogFile.logger.log(Level.SEVERE, "Exception during reading parameters. :\n"
					+e.getMessage()+" Cause "+e.getCause());
			return null;
		}
		return par;
	}
	
	//**** DATABASE READER METHODS ****	
	public void get_HashtablesFromDB(){
		
	}
	
	public MTUScountry get_MTUSFromDB(){
		MTUScountry mtus = new MTUScountry(country_id);
		
		try{
		// select diaries, group ids and time spent per ME
		ArrayList<String[]> ids = da.select_diary(country_id);
		ArrayList<String[]> time = da.select_diary_time(ids);
 		console(ids.size()+" diaries with "+time.size()+" time entries selected.");
 		int t = 0;
		// loop through diaries
		for(int i=0; i<ids.size(); i++){
			mtus.set_group(ids.get(i)[1]);
			// loop through microenvironments
			for(int j=0; j<5; j++){				
				mtus.set_time(Integer.parseInt(time.get(t)[1]), time.get(t)[0]);
				t++;
			}
			mtus.close_diary();
		}	
		}
		catch(Exception e){
			LogFile.logger.log(Level.SEVERE, "Exception during selecting time use data from database \n"
					+e.getMessage()+" Cause "+e.getCause());
			return null;
		}
		return mtus;
	}
	
	public EMEPgridCountry get_EMEPFromDB(String year, Hashtable<String,String> subgroupHash){
		EMEPgridCountry emep = new EMEPgridCountry(country_id);
		
		try{
		// select cellID, population, concentration, urban_increment, urban_pop
		// select emep50_id, group_id, population
		ArrayList<String[]> pop = da.select_population(country_id, year);
		console(pop.size()+" cells with population data selected.");
		// loop through grid cells for this country
		for(int i=0; i<pop.size(); i++){
			emep.set_cellID(pop.get(i)[0]);
			// get concentration for this grid cell
			float con = da.select_concentration(pop.get(i)[0], pollutant, scenario);
			emep.set_concentrationValue(con);
			
			// get urban increment and population for this grid cell
			float[] urb = da.select_urban_increment(pop.get(i)[0], pollutant,scenario);
			emep.set_urban_incrementValue(urb[0]);
			emep.set_urban_populationPercentage(urb[1]);
			
			Float[] popData = new Float[8];
			// fill population data for the 8 subgroups
			for(int j=0; j<8; j++){
				// translate subgroup into groupindex
				int groupindex = Integer.parseInt(subgroupHash.get(pop.get(i)[1]));
				popData[groupindex] = Float.parseFloat(pop.get(i)[2]);
				if(j<7)
					i++;
			}
		//	console(i+"");
			emep.set_populationGroup(popData);
		}
		}
		catch(Exception e){
			LogFile.logger.log(Level.SEVERE, "Exception during selecting EMEP data from database \n"
					+e.getMessage()+" Cause "+e.getCause());
			return null;
		}
		
		return emep;
	}
	
	public IndoorParameters[] get_ParametersFromDB(){
		IndoorParameters[] par = new IndoorParameters[2];
		
		try{
		// select parameter id and distribution type
		ArrayList<String[]> ids = da.select_parameter();
		
		// select parameter id, microenvironment, distribution parameter and value
		ArrayList<String[]> dist = da.select_parameter_distribution_array(country_id, pollutant, scenario);
		
		// select parameter id, group id and value
		ArrayList<String[]> frac = da.select_parameter_fraction_array(country_id, pollutant, scenario);
		console(ids.size()+" parameters selected.");
 		
		IndoorParameters home = new IndoorParameters(country_id, "home");
		IndoorParameters work = new IndoorParameters(country_id, "work");
		
		// loop through parameter ids
		for(int i=0; i<ids.size(); i++){
			String id = ids.get(i)[0];
			
			// get distributions for R
			String distribution = "";
			if(ids.get(i)[1].equals("lognormal"))
				distribution="lnorm";
			else if(ids.get(i)[1].equals("normal"))
				distribution="norm";
			else if(ids.get(i)[1].equals("uniform"))
				distribution="unif";	
			
			// loop through distribution entries for this parameter
			for (int d=0; d<dist.size(); d++){
				String[] distVal = dist.get(d);
				if(distVal[0].equals(id)){
					d++;
					// get both distribution parameters
					float[] vals = {Float.parseFloat(distVal[3]), Float.parseFloat(dist.get(d)[3])};
					if(distVal[1].equals("home")){
						home.set_parameter(id, vals);
						home.set_parameter_distribution(id, distribution);
					}
					else{
						work.set_parameter(id, vals);
						work.set_parameter_distribution(id, distribution);
					}
				}
			}
			
			// loop through fraction entries for this parameter
			for(int f=0; f<frac.size(); f++){
				if(frac.get(f)[0].equals(id)){
					if(id.equals("f_bio")){ // f_bio has only one entry for all subgroups
						float[] fracs = {Float.parseFloat(frac.get(f)[2])};
						home.set_subgroup(frac.get(f)[1], 8);
						home.set_parameter(id, fracs);
					}
					else{
						float[] fracs = new float[8];
						for(int j=0; j<8; j++){
					//		console(frac.get(f)[0]+", "+frac.get(f)[1]+","+frac.get(f)[2]);
							fracs[j] = Float.parseFloat(frac.get(f)[2]);
							home.set_subgroup(frac.get(f)[1], j);
							f++;
						}
						home.set_parameter(id, fracs);
					}
				}
			}
		}	
		par[0] = home; par[1] = work;
		}
		catch(Exception e){
			LogFile.logger.log(Level.SEVERE, "Exception during selecting parameters from database. \n"
					+e.getMessage()+" Cause "+e.getCause());
			return null;
		}
		return par;
	}
	
	//**** DATABASE FILLING METHODS ****
	public void initialiseDatabase(String sqlFile){
		df.initialiseDatabase(sqlFile);
	}
	
	public boolean fillSimulationTable(String simID, int iterations, boolean gridbased){
		try{
		String description = "";
		// add details
//		if(withIndoorSources)
//			description += "simulation includes outdoor and indoor sources, ";
//		else
//			description += "simulation includes only outdoor sources, ";
		if(gridbased)
			description += "results are stored based on EMEP grid ";
		else
			description += "results are stored based on individuals ";

		String grid = "0";
		if(gridbased)
			grid = "1";
		String[] data = {simID, pollutant, country_id, scenario, grid, (iterations+""), description};
		df.insertSimpleRow(data, "simulation");
		}
		catch(Exception e){
			LogFile.logger.log(Level.SEVERE, "Exception during filling simulation table. \n"
					+e.getMessage()+" Cause "+e.getCause());
			return false;
		}
		console("Inserted successfully 1 entry into table simulation.");
		return true;
	}

	//**** UTILITIES ****
	private ArrayList<File> readCsvFromDirectory(String folder){
		File directory = new File(folder);
		ArrayList<File> fileArray = new ArrayList<File>();
		if (!directory.exists()) {
			System.out.println("Directory does not exist!");
			LogFile.logger.log(Level.WARNING, "Directory "+folder+" does not exist!");
		} 
		else{
			// list files in this folder
			File[] files = directory.listFiles();
			for (int i = 0; i < files.length; i++) {
				// only parse csv files
				if (files[i].toString().contains("csv")) {
					fileArray.add(files[i]);
				} 
			}
		}
		return fileArray;
	}
	
	public void closeDBFeederConnection(){
		df.close();
	}
	
	public void closeDBAccessConnection(){
		da.close();
	}
	
	private void console(String data){
		System.out.println(data);
	}
}

