package de.ifgi.exposure.io;

import java.io.*;
import java.util.Hashtable;
import java.util.zip.*;
import java.util.logging.*;
import java.util.ArrayList;
import de.ifgi.exposure.control.LogFile;

public class EMEPdownload {

	/**
	 * Class for downloading EMEP textfiles
	 * @param args
	 */
	private String emepurl = "http://www.emep.int/grid/";
	private Hashtable<Integer,String> countryHash;
	
	public EMEPdownload(){
	}
	
	private URLconnection downloadFile(String filename) {
		String url = emepurl + filename;
		// build and test URLconnection
		URLconnection connection = new URLconnection(url);
		try {
			connection.testurl();
		} catch (IOException e) {	
			System.out.println("No download from http://www.emep.int/ possible.");
			LogFile.logger.log(Level.INFO, "No download from http://www.emep.int/ possible.");
		}
	
		System.out.println("EMEP Download successful!");
		LogFile.logger.log(Level.INFO, "Download successful!");
		return connection;
	}
	
	public void mapCountries (String countryfile){
		// get URLconnection
		URLconnection connection = downloadFile(countryfile);
		
		countryHash = new Hashtable<Integer,String>();
		try{
			BufferedReader in = connection.inputStreamReader();
			
			// get header
			String line = in.readLine();
			
			while((line = in.readLine())!=null){	
				System.out.println(line);
				String[] st = line.split("\t");
				//System.out.println(st.length);
				
				// use another delimiter if it changed
				if(st.length<2)
					st = line.split("    ");

				countryHash.put(Integer.parseInt(st[1].trim()), st[0].trim());
			}
		}
		catch(IOException e){		
			System.out.println("IOException during reading EMEP_countries file: " + e.getMessage());
			LogFile.logger.log(Level.SEVERE, "IOException during reading EMEP_countries file: " + e.getMessage());
		}
		
	}
	
	public ArrayList<String[]> parseEMEPfraction(String filename){
		// get URLconnection
		URLconnection connection = downloadFile(filename);
		
		ArrayList<String[]> data = new ArrayList<String[]>();
		try{
			BufferedReader in = connection.inputStreamReader();
			
			// read header
			String line = in.readLine();
			
			while((line = in.readLine())!=null){
			//	System.out.println(line);
				String[] st = line.split(" ");
				
				// new String for columns
				String[] clean = new String[4];
				String[] cols = new String[3];
				int j = 0;
				for(int i=0; i<st.length; i++){
					if(!st[i].equals("")||!st[i].equals("")){
						clean[j] = st[i];
						j++;
					}
				}
				cols[0] = clean[1].trim()+"_"+clean[2].trim(); // EMEP_id
				String c = countryHash.get(Integer.parseInt(clean[0].trim())); // country_id
				if(c != null)
					cols[1] = c;
				else
					cols[1] = clean[0];
				cols[2] = clean[3].trim();
				//System.out.println(clean[0].trim());

				data.add(cols);
			}
		}
		catch(IOException e){		
			System.out.println("Input error during reading EMEP_fraction file.");
			LogFile.logger.log(Level.SEVERE, "Input error during reading EMEP_fraction file."+ e.getMessage());
			
		}
		return data;
	}
	
	public ArrayList<String[]> downloadAndParseEMEPfile(String filename) {
		// get URLconnection
		URLconnection connection = downloadFile(filename);
		
		ArrayList<String[]> data = new ArrayList<String[]>();
		try {
			ZipInputStream zip = new ZipInputStream(connection.getInputStream());
			zip.getNextEntry();

			// define inputstream
			BufferedReader in = new BufferedReader(new InputStreamReader(zip,
					"ISO-8859-1"));
			data = readEMEPfile(in);
			// close the stream
			zip.close();		
		}

		catch (IOException e) {
			System.out.println("Input error during reading ZIP");
			LogFile.logger.log(Level.SEVERE, "Input error during reading ZIP");
		}
		return data;
	}
	
	public ArrayList<String[]> parseEMEPfile(String filename){
		ArrayList<String[]> data = new ArrayList<String[]>();
		try{
			BufferedReader in = new BufferedReader(new FileReader(new File(
					filename)));	
			data = readEMEPfile(in);
		}
		catch(IOException e){
			System.out.println("Input error during reading EMEPfile");
			LogFile.logger.log(Level.SEVERE,"Input error during reading EMEPfile");
		}
		return data;
	}
	
	private ArrayList<String[]> readEMEPfile(BufferedReader in) throws IOException{
		ArrayList<String[]> data = new ArrayList<String[]>();
		String line = "";			
		String[] cols = new String[5];
		while((line = in.readLine())!=null){
			String st[] = line.trim().split("    ");			
			//'emep50_id', emep50_i, emep50_j, center_lat, center_lon
			if(st[0].trim().startsWith("EMEP")){
				// emep50_i
				cols[1] = st[1].trim();
				// emep50_j
				cols[2] = st[2].trim();
				//emep50_i_j
				cols[0] = cols[1] + "_" + cols[2];
			}
			else if(st[0].contains("CENTER"))
			{		
				//center_lon
				cols[4] = st[1].trim();
				//center_lat
				cols[3] = st[2].trim();					
				data.add(cols);
				cols = new String[5];
			}
		}
		// close the streams
		in.close();
		return data;
	}
	
}
