package de.ifgi.exposure.statistics;

import java.util.Random;


public class BinomialDistribution extends ProbabilityDistribution{
	/**
	 * long r_array = re.rniPutDoubleArray(probabilities);
		re.rniAssign("p", r_array, 0);
		REXP r_temp = re.eval("rbinom("+probabilities.length+",1,p)");
		rbinom(n, size, prob)
		size=size, n=n_obs , prob=prob
	 */
	
	
private double prob;
private int size;
	
	public BinomialDistribution(double probability, int size){
		this.size=size;
		this.prob=probability;
	}
	
	public double drawSample() {
		double result = 0; // number of successes
		
		// get random number to evaluate success
		Random r = new Random();
		for(int i=0; i<size; i++){
			double p_success = r.nextDouble();
			if(p_success<prob)
				result++;
		}
		
		return result;
	}
	
	// Getters for distribution parameters
	public double probability(){
		return prob;
	}
	
	public int size(){
		return size;
	}
	
	@Override
	public double mean(){
		return (prob*size);
	}
	
	@Override
	public double standardDeviation(){
		return Math.sqrt(prob*size*(1-prob));
	}
	
	@Override
	public double variance(){
		return (prob*size*(1-prob));
	}


	// methods for size=1
	public static double drawSampleDouble(double probability){
		double result;
		
		// get random number to evaluate success
		Random r = new Random();
		double p_success = r.nextDouble();
		if(p_success<probability) // the greater probability is, the higher is the chance of success
			result = 1;
		else
			result = 0;
		
		return result;
	}
	
	public static float drawSampleFloat(float probability){
		float result;
		
		// get random number to evaluate success
		Random r = new Random();
		float p_success = r.nextFloat();
		if(p_success<probability) // the greater probability is, the higher is the chance of success
			result = 1;
		else
			result = 0;
		
		return result;
	}
	
	// methods for size>1
	public static double[] drawSamplesDouble(double[] probabilities){
		double[] result = new double[probabilities.length];

		// get random number to evaluate success
		Random r = new Random();
		
		for(int i=0; i<probabilities.length;i++){
		double p_success = r.nextDouble();
			if(p_success<probabilities[i]) // the greater probability is, the higher is the chance of success
				result[i] = 1;
			else
				result[i] = 0;
		}
		return result;
	}
	
	public static float[] drawSamplesFloat(float[] probabilities){
		float[] result = new float[probabilities.length];
		
		// get random number to evaluate success
		Random r = new Random();
		
		for(int i=0; i<probabilities.length;i++){
		float p_success = r.nextFloat();
			if(p_success<probabilities[i]) // the greater probability is, the higher is the chance of success
				result[i] = 1;
			else
				result[i] = 0;
		}
		
		return null;
	}
}
