package de.ifgi.exposure.statistics;

import java.util.Random;

public class LognormalDistribution extends NormalDistribution{

	private double mu, sigma;
	
	public LognormalDistribution(double lnMean, double lnSd){
		mu = lnMean;
		sigma = lnSd;
	}
	
	public double normalMean(){
		return lnMean_to_nMean(mu,sigma);
	}
	
	public double normalSd(){
		return lnSd_to_nSd(mu,sigma);
	}
	
	public double drawSample(){
		double result; double standardnormal;
		Random r = new Random();
		
		// get sample from standard normal distribution
		standardnormal = r.nextGaussian();
		
		// convert this sample to a sample from the desired distribution
		double fx = sndProbability(standardnormal);
		
		// normal distribution to standard normal distribution
		result = sndProbabilityToSample(mu, sigma, fx);
		
		// results will all be above the mean as the distribution is symmetrical
		// move some of the values below the mean
		if(standardnormal<0)
			result = 2*mu-result;
		
		// convert back to true value from lognormal result
		double convertedResult = Math.exp(result);

		return convertedResult;
	}
	
	public double[] drawSamples(){
		return null;
	}
	
	public static double drawSampleDouble(double normalMean, double normalSd){
		double result; double standardnormal;
		Random r = new Random();
		
		// transform mean and sd into lognormal parameters
		double lnMean = nMean_to_lnMean(normalMean, normalSd);
		double lnSd = nSd_to_lnSd(normalMean, normalSd);
		
		// get sample from standard normal distribution
		standardnormal = r.nextGaussian();
		
		// convert this sample to a sample from the desired distribution
		double fx = sndProbability(standardnormal);
		
		// normal distribution to standard normal distribution
		result = sndProbabilityToSample(lnMean, lnSd, fx);
		
		// results will all be above the mean as the distribution is symmetrical
		// move some of the values below the mean
		if(standardnormal<0)
			result = 2*lnMean-result;
		
		// convert back to true value from lognormal result
		double convertedResult = Math.exp(result);
		
		System.out.println(convertedResult);
		return convertedResult;
	}
	
	public static float drawSampleFloat(float normalMean, float normalSd){
		float result; float standardnormal;
		Random r = new Random();
		
		// transform mean and sd into lognormal parameters
		float lnMean = (float)nMean_to_lnMean(normalMean, normalSd);
		float lnSd = (float)nSd_to_lnSd(normalMean, normalSd);
		
		// get sample from standard normal distribution
		standardnormal = (float)r.nextGaussian();
		
		// convert this sample to a sample from the desired distribution
		float fx = (float)sndProbability(standardnormal);
		
		// normal distribution to standard normal distribution
		result = (float)sndProbabilityToSample(lnMean, lnSd, fx);
		
		// results will all be above the mean as the distribution is symmetrical
		// move some of the values below the mean
		if(standardnormal<0)
			result = 2*lnMean-result;
		
		// convert back to true value from lognormal result
		float convertedResult = (float)Math.exp(result);
		
		return convertedResult;
	}
	
	public static double lnMean_to_nMean(double lnMean, double lnSd){	
		//mu = exp(m.log+0.5*s.log^2)
		double nMean = Math.exp(lnMean+0.5*Math.pow(lnSd, 2));
		return nMean;
	}
	
	public static double lnSd_to_nSd(double lnMean, double lnSd){
		//sigma = exp(m.log+0.5*s.log^2)*sqrt(exp(s.log^2)-1)
		double nSd = Math.exp(lnMean+0.5*Math.pow(lnSd, 2))*Math.sqrt(Math.exp(Math.pow(lnSd, 2))-1);
		return nSd;
	}
	
	public static double nMean_to_lnMean(double nMean, double nSd){
		//mu = log(m^2*sqrt(1/(s^2+m^2)))
		double lnMean = Math.log(Math.pow(nMean,2)*Math.sqrt(1/(Math.pow(nSd, 2)+Math.pow(nMean, 2))));
		return lnMean;
	}
	
	public static double nSd_to_lnSd(double nMean, double nSd){
		//sigma = sqrt(log(s^2/m^2+1))
		double lnSd = Math.sqrt(Math.log(Math.pow(nSd, 2)/Math.pow(nMean, 2)+1));
		return lnSd;
	}
	
}
