package de.ifgi.exposure.statistics;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.logging.Level;

import de.ifgi.exposure.control.LogFile;

public class Summariser {

	private ArrayList<String> countryList;
	private ArrayList<String> scenarioList;
	private ArrayList<String> subgroupList;
	
	public Summariser(){
		countryList = new ArrayList<String>();
		countryList.add("AT");
		countryList.add("BE");
		countryList.add("BG");
		countryList.add("CY");
		countryList.add("CZ");
		countryList.add("DK");
		countryList.add("EE");
		countryList.add("FI");
		countryList.add("FR");
		countryList.add("DE");
		countryList.add("GR");
		countryList.add("HU");
		countryList.add("IS");
		countryList.add("IE");
		countryList.add("IT");
		countryList.add("LV");
		countryList.add("LT");
		countryList.add("LU");
		countryList.add("MT");
		countryList.add("NL");
		countryList.add("NO");
		countryList.add("PL");
		countryList.add("PT");
		countryList.add("RO");
		countryList.add("SK");
		countryList.add("SI");
		countryList.add("ES");
		countryList.add("SE");
		countryList.add("CH");
		countryList.add("GB");
	
		scenarioList = new ArrayList<String>();
		scenarioList.add("2010BAU");
		scenarioList.add("2020BAU");
		scenarioList.add("2020Policy");
		scenarioList.add("2030BAU");
		scenarioList.add("2030Policy");
		scenarioList.add("2050BAU");
		scenarioList.add("2050Policy");
		scenarioList.add("2050Biomass");
		scenarioList.add("2050Insulation");
		scenarioList.add("2050Renovation");
		
		subgroupList = new ArrayList<String>();
		subgroupList.add("F15");
		subgroupList.add("M15");
		subgroupList.add("Fwork");
		subgroupList.add("Mwork");
		subgroupList.add("Fnonwork");
		subgroupList.add("Mnonwork");
		subgroupList.add("F65");
		subgroupList.add("M65");
	}
	
	private ArrayList<float[]> readSimpleFile(String filename){
		ArrayList<float[]> res = new ArrayList<float[]>();
		try{			
			BufferedReader input = new BufferedReader(new FileReader(new File(
					filename)));
			
			// read header
			String row = input.readLine();
			
			// read data and average each row
			while ((row = input.readLine()) != null) {			
				String[] s = row.split(",");
				float[] cols = new float[s.length-1];
				// loop through columns
				for(int i=1; i<s.length; i++){
					cols[i-1] = Float.parseFloat(s[i].trim());
				}
				float av = this.average(cols);
				float sd = this.standard_deviation(av, cols);
	
				float[] row_res = {av, sd};
				res.add(row_res);
			}
			//System.out.println(asciiCells.size());
			input.close();
			return(res);
		}
		catch(Exception e){
			System.out.println("Exception during reading file: "+ filename+"\n"+e.getMessage());
			LogFile.logger.log(Level.SEVERE,"Exception during reading file: "+ filename+"\n"
					+e.getMessage()+ " Cause "+ e.getCause());
			return null;
		}
	}
	
	public ArrayList<float[]> summariseExposure(float[][] exposure, int length){
		ArrayList<float[]> res = new ArrayList<float[]>();		
			
		// average each row
		for(int i=0; i<length;i++) {			
			float av = this.average(exposure[i]);
			float sd = this.standard_deviation(av, exposure[i]);
	
			float[] row_res = {av, sd};
			res.add(row_res);
		}
		return(res);
	}
	
	// stats utilities
	private float average(float[] data){
		float sum = 0F;
		for(int i=1; i<data.length; i++){
			sum += data[i];
		}
		return (sum/data.length);
	}
	
	private float standard_deviation(float mean, float[] data){
		float diff_sum = 0F;
		for(int i=1; i<data.length; i++){
			diff_sum += Math.pow((data[i]-mean), 2);
		}
		return (float)Math.sqrt(diff_sum/(data.length-1));
	}
	
	// Method to read countrybased subgroup files for one scenario
//	public Hashtable<String[],ArrayList<float[]>> readSubgroups(String add){
//			// read country files for the specified scenarios
//			// e.g. C:/LAMA/output/2010BAU/country/AT_2010BAU_PM2.5_i10000_subgroups.csv
//			Hashtable<String,ArrayList<float[]>> hash = new Hashtable<String,ArrayList<float[]>>();
//			
//			// loop through scenarios
//			for(int s=0; s<scenarioList.size(); s++){
//				String folderName = "C:/LAMA/output/"+scenarioList.get(s)+"/country/";
//				System.out.println("Parsing now "+scenarioList.get(s));
//				// loop through countries to read all results
//				for(int c=0; c<countryList.size(); c++){
//					String fileName = countryList.get(c)+"_"+scenarioList.get(s)+"_PM2.5_i10000_"+add+"subgroups.csv";
//					//System.out.println(folderName+fileName);
//					ArrayList<float[]> countryRes = this.readSimpleFile(folderName+fileName);
//					String hashID = scenarioList.get(s)+"_"+add+countryList.get(c);
//					hash.put(hashID, countryRes);
//				}
//			}
//			return hash;
//	}
	
	// Method to read countrybased subgroup files for one scenario
	private Hashtable<String,ArrayList<float[]>> readScenarioFilesSubgroups(String add){
			// read country files for the specified scenarios
			// e.g. C:/LAMA/output/2010BAU/country/AT_2010BAU_PM2.5_i10000_subgroups.csv
			Hashtable<String,ArrayList<float[]>> hash = new Hashtable<String,ArrayList<float[]>>();
			
			// loop through scenarios
			for(int s=0; s<scenarioList.size(); s++){
				String folderName = "C:/LAMA/output/"+scenarioList.get(s)+"/country/";
				System.out.println("Parsing now "+scenarioList.get(s));
				// loop through countries to read all results
				for(int c=0; c<countryList.size(); c++){
					String fileName = countryList.get(c)+"_"+scenarioList.get(s)+"_PM2.5_i10000_"+add+"subgroups.csv";
					//System.out.println(folderName+fileName);
					ArrayList<float[]> countryRes = this.readSimpleFile(folderName+fileName);
					String hashID = scenarioList.get(s)+"_"+add+countryList.get(c);
					hash.put(hashID, countryRes);
				}
			}
			return hash;
	}
	
	public void writeSummary(Hashtable<String,ArrayList<float[]>> hash, String filename, String add, int parID){
		try{
			
			BufferedWriter out = new BufferedWriter(new OutputStreamWriter(
					new FileOutputStream(filename)));
			
			// write header
			out.write("policy,subgroup,");
			for(int i=0; i<countryList.size(); i++){
				if(i<(countryList.size()-1))
					out.write(countryList.get(i)+",");
				else
					out.write(countryList.get(i));
			}
			out.newLine();
			
			// write data
			for(int s=0; s<scenarioList.size(); s++){
				
				for(int i=0; i<subgroupList.size(); i++){
					out.write(scenarioList.get(s)+","+subgroupList.get(i)+",");
					
					for(int c=0; c<countryList.size(); c++){
						String id = scenarioList.get(s)+"_"+add+countryList.get(c);
						ArrayList<float[]> subgroupRes = hash.get(id);
						if(c<(countryList.size()-1))
							out.write(subgroupRes.get(i)[parID]+",");
						else{
							out.write(subgroupRes.get(i)[parID]+"");	
						}
					}
					out.newLine();
				}
			}
			
			// close stream
			out.close();
			System.out.println("Successfully written file "+filename);
		}
		catch(Exception e){
			System.out.println("Exception during writing file: "+ filename+"\n"+e.getMessage());
			LogFile.logger.log(Level.SEVERE,"Exception during writing file: "+ filename+"\n"
					+e.getMessage()+ " Cause "+ e.getCause());
		}
	}
	
//	public static void main(String[] args) {
//		Summariser sumRes = new Summariser();
//		// possible outdoor_, withoutETS_, withoutBiomass, withoutBackground_
//		String add = "withoutETS_";
//		Hashtable<String,ArrayList<float[]>> hashres = sumRes.readScenarioFilesSubgroups(add);
//		String outMeanFile = "C:/LAMA/output/All_countries_"+add+"mean.csv";
//		sumRes.writeSummary(hashres, outMeanFile, add, 0);
//		String outSdFile = "C:/LAMA/output/All_countries_"+add+"sd.csv";
//		sumRes.writeSummary(hashres, outSdFile, add, 1);
//	}
	
}
