package de.ifgi.exposure.statistics;

import java.util.Random;

public class UniformDistribution extends ProbabilityDistribution {

	double min, max;

	public UniformDistribution(double min, double max) {
		this.min = min;
		this.max = max;
	}

	public double drawSample() {
		double result;
		Random r = new Random();
		result = r.nextDouble();

		// repeat until number within limits is found
		while (result > max || result < min) {
			result = r.nextFloat();
		}

		return result;
	}

	public double[] drawSamples(int n) {
		double result[] = new double[n];
		Random r = new Random();
		result[0] = r.nextDouble();

		// repeat until number within limits is found
		for (int i = 0; i < n; i++) {
			while (result[i] > max || result[i] < min) {
				result[i] = r.nextFloat();
			}
		}

		return result;
	}

	// Getters for distribution parameters
	public double min(){
		return min;
	}
	
	public double max(){
		return max;
	}
	
	@Override
	public double mean(){
		return 0;
	}
	
	@Override
	public double standardDeviation(){
		return 0;
	}
	
	@Override
	public double variance(){
		return 0;
	}

	// static class methods for drawing samples
	public static double drawSampleDouble(double min, double max) {
		double result;
		Random r = new Random();
		result = r.nextDouble();

		// repeat until number within limits is found
		while (result > max || result < min) {
			result = r.nextDouble();
		}

		return result;
	}

	public static float drawSampleFloat(float min, float max) {
		float result;
		Random r = new Random();
		result = r.nextFloat();
	
		// repeat until number within limits is found
		while (result > max || result < min) {
			result = r.nextFloat();
		}

		return result;
	}

}
