package de.ifgi.exposure.view;

import java.io.*;
import java.util.UUID;
import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.Border;
import de.ifgi.exposure.control.*;
import de.ifgi.exposure.datamodel.SimulationParameters;
import de.ifgi.exposure.datamodel.VisualisationParameters;
import de.ifgi.exposure.io.DataHandler;

/*
 * LAMA GUI class
 */

public class GUI extends JFrame implements ActionListener {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private boolean dbVersion;
	// GUI components
	private JFrame frame;
	private JButton openButton, okButton, cancelButton;
	private JTextArea log, logFinal;
	// private JCheckBox resultBox;
	private JFileChooser fc;
	private JRadioButton csvButton, dbButton, fillDBButton, writeCsvButton,
			writeDBButton, mysqlButton, postgresqlButton;
	private JTextField urlField, userField, iterationsField;
	private JPasswordField passwordField;
	private JComboBox countryBox, pollutantBox, scenarioBox;
	static private final String newline = "\n";

	// simulation parameters
	private boolean readDB, fillDB, writeDB, folderCorrect,
			folderChecked = false;
	private String LAMAfolder, errorMessage;
	private SimulationParameters sim;
	private String sim_id, country_id, pollutant_id, scenario_id;
	private int iterations;
	private boolean gridbasedResults, individualResults, subgroupResults,
			exETS, exBMS, exBGR, storeRealisations; // withIndoorSources,
													// withETS,
	private String jdbcDriver, url, user, password;
	private String[] countryList, countryIDList, scenarioList, scenarioIDList,
			pollutantList;
	private VisualisationParameters vis;
	private boolean startAguila, writeQuantiles, visualisation, countryWise;
	private float[] quantiles;
	private float etsRatio;

	// sources used options
	private JCheckBox etsBox, bgrBox, bmsBox;

	// results options
	private JRadioButton gridbasedButton, countrybasedButton,
			individualsButton, subgroupsButton, totalpopButton,
			averagesdButton, realisationsButton;
	private JTextField etsRatioField;

	// visualisation GUI options
	private JRadioButton quantilesButton, statisticsButton;
	private JCheckBox aguilaBox;
	private JTextField quantilesField;

	public GUI(boolean db) {
		// get default values
		setupdefault();
		dbVersion = db;
	}

	public void showGUI() {
		if(dbVersion){
			setupCsvPanel("");
		}
		// Start first GUI
		setupInitialPanel();
	}

	private void setupInitialPanel() {
		// initiate frame
		frame = new JFrame("LAMA configuration");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		// set layout of this main container
		JPanel initPane = new JPanel();
		initPane.setLayout(new GridBagLayout());

		// create buttons for first panel
		csvButton = new JRadioButton("read from csv files");
		csvButton.setActionCommand("readCsv");
		csvButton.setSelected(true);

		dbButton = new JRadioButton("read from database");
		dbButton.setActionCommand("readDB");
		// TODO: set true for database use
		// dbButton.setEnabled(false);

		fillDBButton = new JRadioButton("read from csv and fill database");
		fillDBButton.setActionCommand("fillDB");
		// TODO: set true for database use
		// fillDBButton.setEnabled(false);

		// create buttons for second panel
		writeCsvButton = new JRadioButton("write to csv files");
		writeCsvButton.setActionCommand("writeCsv");
		writeCsvButton.setSelected(true);

		writeDBButton = new JRadioButton(
				"write to database (if db is initiated)");
		writeDBButton.setActionCommand("writeDB");
		// TODO: set true for database use
		writeDBButton.setEnabled(false);

		// Group the radio buttons.
		ButtonGroup group1 = new ButtonGroup();
		group1.add(csvButton);
		group1.add(dbButton);
		group1.add(fillDBButton);
		ButtonGroup group2 = new ButtonGroup();
		group2.add(writeCsvButton);
		group2.add(writeDBButton);

		// Register a listener for the radio buttons.
		addDataButtonListener(csvButton);
		addDataButtonListener(dbButton);
		addDataButtonListener(fillDBButton);
		addDataButtonListener(writeCsvButton);
		addDataButtonListener(writeDBButton);

		// create Label for GUI
		JLabel title = new JLabel("Specify your input and output data types.");

		// Put the radio buttons in a column in a panel.
		JPanel inputPanel = new JPanel(new GridLayout(0, 1));
		inputPanel.setBorder(BorderFactory.createTitledBorder("Input"));
		inputPanel.add(csvButton);
		inputPanel.add(dbButton);
		inputPanel.add(fillDBButton);
		JPanel outputPanel = new JPanel(new GridLayout(0, 1));
		outputPanel.setBorder(BorderFactory.createTitledBorder("Output"));
		outputPanel.add(writeCsvButton);
		outputPanel.add(writeDBButton);

		// create OK and Cancel Button
		okButton = new JButton("Next");
		okButton.setMnemonic(KeyEvent.VK_ENTER);
		cancelButton = new JButton("Cancel");
		okButton.setActionCommand("1");
		cancelButton.setActionCommand("1");
		addOKButtonListener(okButton);
		addCancelButtonListener(cancelButton);

		// put buttons in a row in a panel
		JPanel buttonPanel = new JPanel(new GridLayout(1, 0));
		buttonPanel.add(okButton);
		buttonPanel.add(cancelButton);

		// add log
		log = new JTextArea(3, 20);
		log.setMargin(new Insets(5, 5, 5, 5));
		log.setEditable(false);

		// TODO: delete for database use
		log
				.append("Database support is partially disabled in the current version."
						+ newline);

		// add components to GUI
		// add(title, BorderLayout.PAGE_START);
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.anchor = GridBagConstraints.PAGE_START;
		c.gridwidth = 2;
		c.gridx = 0;
		c.gridy = 0;
		c.ipady = 20;
		initPane.add(title, c);
		c.gridwidth = 1;
		c.gridy = 1;
		c.ipady = 40;
		initPane.add(inputPanel, c);
		c.gridx = 1;
		initPane.add(outputPanel, c);
		c.gridwidth = 2;
		c.gridx = 0;
		c.gridy = 2;
		c.ipady = 0;
		initPane.add(buttonPanel, c);
		c.anchor = GridBagConstraints.CENTER;
		c.gridy = 4;
		c.ipady = 20;
		initPane.add(log, c);

		// define border
		Border etchedBdr = BorderFactory.createEtchedBorder();
		Border titledBdr = BorderFactory.createTitledBorder(etchedBdr,
				"Data source configuration");
		Border emptyBdr = BorderFactory.createEmptyBorder(10, 10, 10, 10);
		Border compoundBdr = BorderFactory.createCompoundBorder(titledBdr,
				emptyBdr);
		initPane.setBorder(compoundBdr);

		// Add content to the window.
		frame.setContentPane(initPane);

		// Display the window.
		frame.pack();
		frame.setLocationRelativeTo(null);
		frame.setVisible(true);
	}

	private void setupCsvPanel(String id) {
		frame = new JFrame("LAMA configuration");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		JPanel csvPane = new JPanel(new GridBagLayout());

		// Create a file chooser
		fc = new JFileChooser();
		fc.setCurrentDirectory(new java.io.File("."));
		fc.setDialogTitle("Choose LAMA Folder");
		fc.setAcceptAllFileFilterUsed(false);
		fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);

		// title
		JLabel title = new JLabel(
				"Please select the location of your LAMA folder: ");

		openButton = new JButton("Folder Selection");
		// add Listener to Button
		addOpenButtonListener(openButton);
		JPanel buttonPanel = new JPanel(); // use FlowLayout
		buttonPanel.add(openButton);

		// create OK and Cancel Button
		okButton = new JButton("Next");
		okButton.setMnemonic(KeyEvent.VK_ENTER);
		cancelButton = new JButton("Cancel");
		okButton.setActionCommand("2a");
		cancelButton.setActionCommand("2a");
		addOKButtonListener(okButton);
		addCancelButtonListener(cancelButton);

		// put buttons in a row in a panel
		JPanel buttonPanel2 = new JPanel(new GridLayout(1, 0));
		buttonPanel2.add(okButton);
		buttonPanel2.add(cancelButton);

		// add log
		log = new JTextArea(3, 20);
		log.setMargin(new Insets(5, 5, 5, 5));
		log.setEditable(false);

		// add components to csv panel
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.anchor = GridBagConstraints.PAGE_START;
		c.gridwidth = 1;
		c.gridx = 0;
		c.gridy = 0;
		c.ipady = 20;
		csvPane.add(title, c);
		// c.gridwidth = 1;
		// c.gridx = 0;
		c.gridy = 1;
		// c.ipady = 20;
		csvPane.add(buttonPanel, c);
		// c.gridwidth = 1;
		// c.gridx = 0;
		c.gridy = 2;
		c.ipady = 0;
		csvPane.add(buttonPanel2, c);
		// c.gridwidth = 1;
		// c.gridx = 0;
		c.gridy = 3;
		c.ipady = 40;
		csvPane.add(log, c);

		// define border
		Border etchedBdr = BorderFactory.createEtchedBorder();
		Border titledBdr = BorderFactory.createTitledBorder(etchedBdr,
				"Csv source configuration");
		Border emptyBdr = BorderFactory.createEmptyBorder(10, 10, 10, 10);
		Border compoundBdr = BorderFactory.createCompoundBorder(titledBdr,
				emptyBdr);

		csvPane.setBorder(compoundBdr);
		// csvPanel.setBorder(BorderFactory.createLineBorder(color,
		// 2);//createEmptyBorder(10,10,10,10));

		// Add content to the window.
		frame.setContentPane(csvPane);

		// Display the window.
		frame.pack();
		frame.setLocationRelativeTo(null);
		frame.setVisible(true);
	}

	private void setupDatabasePanel(String id) {
		frame = new JFrame("LAMA configuration");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		JPanel dbPane = new JPanel(new GridBagLayout());

		// title
		JLabel title = new JLabel("select your database driver: ");

		// radiobuttons for db driver
		mysqlButton = new JRadioButton("MySQL");
		mysqlButton.setMnemonic(KeyEvent.VK_M);
		mysqlButton.setActionCommand("MySQL");

		postgresqlButton = new JRadioButton("PostgreSQL");
		postgresqlButton.setMnemonic(KeyEvent.VK_M);
		postgresqlButton.setActionCommand("PostgreSQL");

		// group both radiobuttons
		ButtonGroup group = new ButtonGroup();
		group.add(mysqlButton);
		group.add(postgresqlButton);

		// Register a listener for the radio buttons.
		addDBButtonListener(mysqlButton);
		addDBButtonListener(postgresqlButton);

		// panel for radiobuttons
		JPanel radioPanel = new JPanel(new GridLayout(0, 1));
		radioPanel.add(mysqlButton);
		radioPanel.add(postgresqlButton);

		// textfields
		urlField = new JTextField(20);
		urlField.setActionCommand("url");
		addTextFieldListener(urlField);

		userField = new JTextField(20);
		userField.setActionCommand("user");
		addTextFieldListener(userField);

		passwordField = new JPasswordField(20);
		passwordField.setActionCommand("password");
		passwordField.setEchoChar('*');
		addTextFieldListener(passwordField);

		JLabel urlTitle = new JLabel("URL with your database name: ");
		JLabel userTitle = new JLabel("Database user: ");
		JLabel pwTitle = new JLabel("Database password: ");

		// panel for textfields
		JPanel textPanel = new JPanel(new GridLayout(3, 2));
		textPanel.add(urlTitle);
		textPanel.add(urlField);
		textPanel.add(userTitle);
		textPanel.add(userField);
		textPanel.add(pwTitle);
		textPanel.add(passwordField);

		// create OK and Cancel Button
		okButton = new JButton("Next");
		okButton.setMnemonic(KeyEvent.VK_ENTER);
		cancelButton = new JButton("Cancel");
		okButton.setActionCommand("2b");
		cancelButton.setActionCommand("2b");
		addOKButtonListener(okButton);
		addCancelButtonListener(cancelButton);

		// put buttons in a row in a panel
		JPanel buttonPanel = new JPanel(new GridLayout(1, 0));
		buttonPanel.add(okButton);
		buttonPanel.add(cancelButton);

		// set log
		log = new JTextArea(3, 20);
		log.setMargin(new Insets(5, 5, 5, 5));
		log.setEditable(false);

		// add components to db panel
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.anchor = GridBagConstraints.CENTER;
		c.gridwidth = 1;
		c.gridx = 0;
		c.gridy = 0;
		c.ipady = 20;
		dbPane.add(title, c);
		// c.gridwidth = 1;
		// c.gridx = 0;
		c.gridy = 1;
		c.ipady = 40;
		dbPane.add(radioPanel, c);
		// c.gridwidth = 1;
		// c.gridx = 0;
		c.gridy = 2;
		// c.ipady = 40;
		dbPane.add(textPanel, c);
		// c.gridwidth = 1;
		// c.gridx = 0;
		c.gridy = 3;
		c.ipady = 0;
		dbPane.add(buttonPanel, c);
		// c.gridwidth = 1;
		// c.gridx = 0;
		c.gridy = 4;
		dbPane.add(log, c);

		// define border
		Border etchedBdr = BorderFactory.createEtchedBorder();
		Border titledBdr = BorderFactory.createTitledBorder(etchedBdr,
				"Database source configuration");
		Border emptyBdr = BorderFactory.createEmptyBorder(10, 10, 10, 10);
		Border compoundBdr = BorderFactory.createCompoundBorder(titledBdr,
				emptyBdr);

		dbPane.setBorder(compoundBdr);

		// Add content to the window.
		frame.setContentPane(dbPane);

		// Display the window.
		frame.pack();
		frame.setLocationRelativeTo(null);
		frame.setVisible(true);
	}

	private void setupFinalPanel() {
		frame = new JFrame("LAMA configuration");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		// set layout of this main container
		JTabbedPane simPanel = new JTabbedPane();
		JPanel finalPane = new JPanel();
		// JPanel finalPane = new JPanel(new GridLayout(2, 0));
		finalPane.setOpaque(true); // content panes must be opaque
		finalPane.setLayout(new GridBagLayout());

		// *** SIMULATION PANEL ***
		JLabel countryTitle = new JLabel("Country: ");
		JLabel scenarioTitle = new JLabel("Scenario: ");
		JLabel pollutantTitle = new JLabel("Pollutant: ");
		JLabel iterationTitle = new JLabel("Number of iterations: ");

		// combo boxes: countryBox, pollutantBox, scenarioBox;
		countryBox = new JComboBox(countryList);
		countryBox.setSelectedIndex(0);
		countryBox.setActionCommand("c");
		addComboBoxListener(countryBox);

		scenarioBox = new JComboBox(scenarioList);
		scenarioBox.setSelectedIndex(0);
		scenarioBox.setActionCommand("s");
		addComboBoxListener(scenarioBox);

		pollutantBox = new JComboBox(pollutantList);
		pollutantBox.setSelectedIndex(0);
		pollutantBox.setActionCommand("p");
		addComboBoxListener(pollutantBox);

		// iterationsField
		iterationsField = new JTextField(20);
		iterationsField.setActionCommand("i");
		iterationsField.setText("10000");
		addTextFieldListener(iterationsField);

		// add comboboxes and text box with titles to panel
		JPanel cbPanel = new JPanel(new GridLayout(4, 2));
		// cbPanel.setBorder(BorderFactory.createTitledBorder("Simulation parameters"));

		// cbPanel.setLayout(new GridBagLayout());
		cbPanel.add(countryTitle);
		cbPanel.add(countryBox);
		cbPanel.add(pollutantTitle);
		cbPanel.add(pollutantBox);
		cbPanel.add(scenarioTitle);
		cbPanel.add(scenarioBox);
		cbPanel.add(iterationTitle);
		cbPanel.add(iterationsField);

		// *** RESULTS PANEL ***
		// gridbasedButton, countrybasedButton
		gridbasedButton = new JRadioButton("grid based");
		countrybasedButton = new JRadioButton("country based");
		countrybasedButton.setSelected(true);

		// individualsButton, subgroupsButton, totalpopButton,
		individualsButton = new JRadioButton("diaries");
		subgroupsButton = new JRadioButton("subgroups");
		totalpopButton = new JRadioButton("total population");
		subgroupsButton.setSelected(true);

		// averagesdButton, realisationsButton;
		averagesdButton = new JRadioButton("mean and standard deviation");
		realisationsButton = new JRadioButton("realisations");
		realisationsButton.setSelected(true);

		// group radiobuttons into groups
		ButtonGroup spatialGroup = new ButtonGroup();
		spatialGroup.add(gridbasedButton);
		spatialGroup.add(countrybasedButton);
		ButtonGroup populationGroup = new ButtonGroup();
		populationGroup.add(individualsButton);
		populationGroup.add(subgroupsButton);
		populationGroup.add(totalpopButton);
		ButtonGroup storageGroup = new ButtonGroup();
		storageGroup.add(averagesdButton);
		storageGroup.add(realisationsButton);

		// Register a listener for the radio buttons.
		addResultButtonListener(gridbasedButton);
		addResultButtonListener(countrybasedButton);
		addResultButtonListener(individualsButton);
		addResultButtonListener(subgroupsButton);
		addResultButtonListener(totalpopButton);
		addResultButtonListener(averagesdButton);
		addResultButtonListener(realisationsButton);

		// panels for radiobuttons
		JPanel spatialPanel = new JPanel(new GridLayout(0, 1));
		spatialPanel.setBorder(BorderFactory.createTitledBorder("Spatial"));
		spatialPanel.add(gridbasedButton);
		spatialPanel.add(countrybasedButton);
		JPanel populationPanel = new JPanel(new GridLayout(0, 1));
		populationPanel.setBorder(BorderFactory
				.createTitledBorder("Population"));
		populationPanel.add(individualsButton);
		populationPanel.add(subgroupsButton);
		populationPanel.add(totalpopButton);
		JPanel storagePanel = new JPanel(new GridLayout(0, 1));
		storagePanel.setBorder(BorderFactory
				.createTitledBorder("Uncertainties"));
		storagePanel.add(averagesdButton);
		storagePanel.add(realisationsButton);

		JPanel resultsPanel = new JPanel(new GridLayout(0, 3));
		// resultsPanel.setBorder(BorderFactory.createTitledBorder("Results"));
		resultsPanel.add(spatialPanel);
		resultsPanel.add(populationPanel);
		resultsPanel.add(storagePanel);

		// *** SOURCES PANEL ***
		// checkboxes for different indoor sources to include
		etsBox = new JCheckBox("exclude ETS sources");
		addCheckBoxListener(etsBox);
		bmsBox = new JCheckBox("exclude biomass sources");
		addCheckBoxListener(bmsBox);
		bgrBox = new JCheckBox("exclude background sources");
		addCheckBoxListener(bgrBox);

		
		etsRatioField = new JTextField(5);
		etsRatioField.setText(etsRatio+"");
		addTextFieldListener(etsRatioField);
		JLabel etsTitle = new JLabel("Choose factor for ETS source");

		
		JPanel sourcesPanel = new JPanel(new GridLayout(0, 1));
		// sourcesPanel.setBorder(BorderFactory.createTitledBorder("Calculation"));
		sourcesPanel.add(etsBox);
		sourcesPanel.add(bmsBox);
		sourcesPanel.add(bgrBox);
		sourcesPanel.add(etsTitle);
		sourcesPanel.add(etsRatioField);

		// *** VIS PANEL ***
		// statistics buttons
		quantilesButton = new JRadioButton("quantiles");
		addVisButtonListener(quantilesButton);

		statisticsButton = new JRadioButton("mean and standard deviation");
		statisticsButton.setSelected(true);
		addVisButtonListener(statisticsButton);

		// group radiobuttons into one group
		ButtonGroup group3 = new ButtonGroup();
		group3.add(quantilesButton);
		group3.add(statisticsButton);

		// textfield for quantiles
		quantilesField = new JTextField(20);
		quantilesField.setActionCommand("q");
		String text = "";
		for (int i = 0; i < quantiles.length; i++) {
			if (i == (quantiles.length - 1))
				text += quantiles[i];
			else
				text += quantiles[i] + "\n";
		}
		quantilesField.setText(text);
		quantilesField.setEnabled(false);
		addTextFieldListener(quantilesField);

		// checkbox for Aguila visualisation
		aguilaBox = new JCheckBox("start Aguila");
		addCheckBoxListener(aguilaBox);

		// panel for visualisation
		JPanel visPanel = new JPanel(new GridLayout(0, 1));
		// visPanel.setBorder(BorderFactory.createTitledBorder("Map visualisation"));
		visPanel.add(statisticsButton);
		visPanel.add(quantilesButton);
		visPanel.add(quantilesField);
		visPanel.add(aguilaBox);

		// *** OK/CANCEL PANEL ***
		// create OK and Cancel Button
		okButton = new JButton("Run Simulation");
		okButton.setMnemonic(KeyEvent.VK_ENTER);
		cancelButton = new JButton("Cancel");
		okButton.setActionCommand("3");
		addOKButtonListener(okButton);
		addCancelButtonListener(cancelButton);

		// put buttons in a row in a panel
		JPanel buttonPanel = new JPanel(new GridLayout(1, 0));
		buttonPanel.add(okButton);
		buttonPanel.add(cancelButton);

		// *** LOG PANEL ***
		// add log
		logFinal = new JTextArea(3, 20);
		logFinal.setMargin(new Insets(5, 5, 5, 5));
		logFinal.setEditable(false);
		logFinal
				.append("If you choose 'all' LAMA will be executed for all countries "
						+ newline + "and/or scenarios in succession." + newline);
		JScrollPane logScrollPane = new JScrollPane(logFinal);

		// *** CONTROL PANEL ***
		// panel for all three option panels
		JPanel controlPanel = new JPanel(new GridLayout(2, 0));
		controlPanel.add(buttonPanel);
		controlPanel.add(logScrollPane);

		// add all to tabbed panel
		simPanel.addTab("simulation", cbPanel);
		simPanel.addTab("results", resultsPanel);
		simPanel.addTab("sources", sourcesPanel);
		simPanel.addTab("visualisation", visPanel);

		// fill final panel
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.anchor = GridBagConstraints.CENTER;
		c.gridwidth = 2;
		c.gridheight = 5;
		c.ipady = 40;
		c.gridx = 0;
		c.gridy = 0;
		finalPane.add(simPanel, c);
		// c.gridwidth = 2;
		c.ipady = 0;
		c.gridheight = 1;
		// c.gridx = 0;
		c.gridy = 5;
		finalPane.add(buttonPanel, c);
		c.ipady = 20;
		c.gridy = 6;
		finalPane.add(logScrollPane, c);

		// Add content to the window.
		frame.setContentPane(finalPane);

		// Display the window.
		frame.pack();
		frame.setLocationRelativeTo(null);
		frame.setVisible(true);

		// checkbox for resulttable
		// resultBox = new JCheckBox("Clear result tables");
		// addCheckBoxListener(resultBox);
		// resultBox.setSelected(false);
		// // only allow selection if results should be written to database
		// if (writeDB)
		// resultBox.setEnabled(true);
		// else
		// resultBox.setEnabled(false);
		// JLabel resultLabel = new JLabel("Database output");
		// JPanel resultBoxPanel = new JPanel(new GridLayout(0, 1));
		// resultBoxPanel.add(resultLabel);
		// resultBoxPanel.add(resultBox);

	}

	// **** UTILITIES ****
	private void setupdefault() {
		readDB = false;
		fillDB = false;
		writeDB = false;
		LAMAfolder = "C:\\LAMA";
		country_id = "DE";
		pollutant_id = "PM2.5";
		scenario_id = "0";
		iterations = 1;

		// sources
		exETS = false;
		exBMS = false;
		exBGR = false;
		etsRatio = 1.0F;
		
		// results
		gridbasedResults = false;
		individualResults = false;
		subgroupResults = true;
		storeRealisations = true;

		jdbcDriver = "org.postgresql.Driver";
		url = "";
		user = "";
		password = "";
		// vis parameters
		visualisation = true;
		countryWise = true;
		startAguila = false;
		writeQuantiles = false;
		quantiles = new float[9];
		quantiles[0] = 0.01F;
		quantiles[1] = 0.05F;
		quantiles[2] = 0.1F;
		quantiles[3] = 0.25F;
		quantiles[4] = 0.5F;
		quantiles[5] = 0.75F;
		quantiles[6] = 0.9F;
		quantiles[7] = 0.95F;
		quantiles[8] = 0.99F;

		if (checkLAMAFolder()) {
			folderCorrect = true;
			folderChecked = true;
		}
	}

	// if no action was performed
	private void getInputDataSources() {
		// Input from Step 1
		// get boolean values for simulation
		if (csvButton.isSelected()) {
			readDB = false;
			fillDB = false;
		} else if (dbButton.isSelected()) {
			readDB = true;
			fillDB = false;
		} else if (fillDBButton.isSelected()) {
			readDB = false;
			fillDB = true;
		}

		if (writeCsvButton.isSelected()) {
			writeDB = false;
		} else if (writeDBButton.isSelected()) {
			writeDB = true;
		}
	}

	// if no action was performed
	private boolean getInputDB() {
		// Input from Step 2b
		// get input from textfields
		String urlName = urlField.getText();
		if (urlName.endsWith("/"))
			urlName = urlName.substring(0, urlName.length() - 1);
		System.out.println(urlName);
		if (mysqlButton.isSelected())
			url = "jdbc:mysql://" + urlName;
		else if (postgresqlButton.isSelected())
			url = "jdbc:postgresql://" + urlName;
		else
			// if no database type is selected, return false
			return false;
		user = userField.getText();
		char[] passChar = passwordField.getPassword();
		password = "";
		for (int i = 0; i < passChar.length; i++) {
			password += passChar[i];
		}
		// System.out.println(password);
		return true;
	}

	// if no action was performed
	private void getInputFinal() {
		// Input from Step 3
		// get input from comboboxes if no selection took place
		country_id = countryIDList[countryBox.getSelectedIndex()];
		if (scenarioBox.getSelectedIndex() == scenarioIDList.length)
			scenario_id = "all";
		else
			scenario_id = scenarioIDList[scenarioBox.getSelectedIndex()];
		pollutant_id = (String) pollutantBox.getSelectedItem();
		// get input from textfields
		iterations = Integer.parseInt(iterationsField.getText());
		
		// get input from ETS text field
		etsRatio = Float.parseFloat(etsRatioField.getText());
		// if(quantilesButton.isSelected()){
		// String[] qText = quantilesField.getText().split(",");
		// float[] newQuantiles = new float[qText.length];
		// try{
		// for(int i=0; i<qText.length; i++){
		//				
		// newQuantiles[i] = Integer.parseInt(qText[i]);
		// }
		// }
		// catch(Exception ex){
		// logFinal.append("Invalid quantile list!");
		// }
		// quantiles = newQuantiles;
		// }
	}

	private boolean checkLAMAFolder() {
		DataHandler dh = new DataHandler();
		boolean success = dh.checkLAMAFolder(LAMAfolder);
		countryList = dh.get_Countries().get(0);
		countryIDList = dh.get_Countries().get(1);
		pollutantList = dh.get_Pollutants();
		String[] scenList = dh.get_Scenarios().get(0);
		scenarioList = new String[scenList.length + 1];
		for (int i = 0; i < scenList.length; i++) {
			scenarioList[i] = scenList[i];
		}
		scenarioList[scenList.length] = "all";
		scenarioIDList = dh.get_Scenarios().get(1);
		// System.out.println("countries. "+countryList.length+", pollutants: "
		// +pollutantList.length+", scenarios: "+scenarioList.length);
		if (!success)
			errorMessage = dh.returnErrorMessage();
		return success;
	}

	private boolean checkDBconnection() {
		DataHandler dh = new DataHandler();
		boolean success = dh.checkDBconnection(jdbcDriver, url, user, password,
				readDB);
		// if data is read from DB get it here
		if (readDB) {
			countryList = dh.get_Countries().get(0);
			countryIDList = dh.get_Countries().get(1);
			pollutantList = dh.get_Pollutants();
			scenarioList = dh.get_Scenarios().get(0);
			scenarioIDList = dh.get_Scenarios().get(1);
			System.out.println(scenarioIDList.length
					+ " scenarios found in database.");
			System.out.println(pollutantList.length
					+ " pollutant found in database.");
		}
		return success;
	}

	// **** MODEL EXECUTION METHOD
	private void initiateModel() {
		System.out.println("Initiate model");
		UUID id = UUID.randomUUID();
		sim_id = id.toString();
		// intiate simulation parameters
		sim = new SimulationParameters(sim_id, country_id, pollutant_id,
				scenario_id, iterations, gridbasedResults, individualResults,
				subgroupResults, storeRealisations);
		if (exETS)
			sim.set_exETS(exETS);
		if (exBGR)
			sim.set_exBackground(exBGR);
		if (exBMS)
			sim.set_exBiomass(exBMS);

		sim.set_ratioETS(etsRatio);
		if (fillDB || readDB) {
			sim.set_JDBC_Driver(jdbcDriver);
			sim.set_dbURL(url);
			sim.set_dbUser(user);
			sim.set_dbPassword(password);
		}

		ModelController mc = new ModelController(sim, vis);
		// delete result tables if selected
		// if (resultBox.isSelected()) {
		// mc.resetResultTables();
		// }

		if (mc.loadInputData(readDB, fillDB, writeDB)) {
			mc.runModel(writeDB);
			// log.append("Expected calculation time: "+sec+" seconds."+newline);
			logFinal.append("Finished model run successfully for " + iterations
					+ " iterations for " + country_id + ". " + newline);
		}
	}

	// **** ACTION LISTENER METHODS ***
	/** General ActionListener. */
	public void actionPerformed(ActionEvent e) {
		// String url = urlField.getText();
		System.out.println(e.getActionCommand().toString());
	}

	/** Listens to the result table check box */
	private void addCheckBoxListener(JCheckBox j) {
		j.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// String cmd = e.getActionCommand();
				if (aguilaBox.isSelected())
					startAguila = true;

				// options for calculation
				if (etsBox.isSelected())
					exETS = true;
				else
					exETS = false;
				if (bmsBox.isSelected())
					exBMS = true;
				else
					exBMS = false;
				if (bgrBox.isSelected())
					exBGR = true;
				else
					exBGR = false;
				// if (resultBox.isSelected())
				// log
				// .append("INFO: This will delete all existing results in the database!");
			}
		});
	}

	/** Listens to the country, pollutant and scenario combo boxes */
	private void addComboBoxListener(JComboBox j) {
		j.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String cmd = e.getActionCommand();
				// System.out.println(cmd);
				if (cmd.equals("c"))
					country_id = countryIDList[countryBox.getSelectedIndex()];
				else if (cmd.equals("s")) {
					if (scenarioBox.getSelectedIndex() == scenarioIDList.length)
						scenario_id = "all";
					else
						scenario_id = scenarioIDList[scenarioBox
								.getSelectedIndex()];
				} else if (cmd.equals("p"))
					pollutant_id = (String) pollutantBox.getSelectedItem();
				// System.out.println(scenario_id);
			}
		});
	}

	/** Listens to textfields. */
	private void addTextFieldListener(JTextField t) {
		t.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String cmd = e.getActionCommand();
				// System.out.println(cmd);
				if (cmd.equals("url") && mysqlButton.isSelected())
					url = "jdbc:mysql://" + urlField.getText();
				else if (cmd.equals("url") && postgresqlButton.isSelected())
					url = "jdbc:postgresql://" + urlField.getText();
				else if (cmd.equals("user"))
					user = userField.getText();
				else if (cmd.equals("password")) {
					char[] passChar = passwordField.getPassword();
					password = "";
					for (int i = 0; i < passChar.length; i++) {
						password += passChar[i];
					}
				} else if (cmd.equals("i"))
					iterations = Integer.parseInt(iterationsField.getText());

				else if (cmd.equals("q") && quantilesButton.isSelected()) {
					String[] qText = quantilesField.getText().split(",");
					float[] newQuantiles = new float[qText.length];
					try {
						for (int i = 0; i < qText.length; i++) {

							newQuantiles[i] = Integer.parseInt(qText[i]);
						}
					} catch (Exception ex) {
						logFinal.append("Invalid quantile list!");
					}
					quantiles = newQuantiles;
				}
			}
		});
	}

	/** Listens to the database radio buttons. */
	private void addDBButtonListener(JRadioButton r) {
		r.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// String cmd = e.getActionCommand();
				// System.out.println(cmd);
				if (mysqlButton.isSelected()) {
					jdbcDriver = "com.mysql.jdbc.Driver";
					urlField.setText("localhost:3306/");
				} else {
					jdbcDriver = "org.postgresql.Driver";
					urlField.setText("localhost:5432/");
				}
				System.out.println("selected driver: " + jdbcDriver);
			}
		});
	}

	/** Listens to the result and visualisation radio buttons. */
	public void addResultButtonListener(JRadioButton r) {
		r.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// String change = e.getActionCommand();
				// System.out.println(change);

				// spatial format
				if (gridbasedButton.isSelected()) {
					gridbasedResults = true;

					individualsButton.setEnabled(false);
					quantilesButton.setEnabled(true);
					statisticsButton.setEnabled(true);
					aguilaBox.setEnabled(true);
				} else {
					gridbasedResults = false;

					individualsButton.setEnabled(true);
					quantilesButton.setEnabled(false);
					statisticsButton.setEnabled(false);
					aguilaBox.setEnabled(false);
				}

				// population options
				if (individualsButton.isSelected()) {
					individualResults = true;
					subgroupResults = false;

					gridbasedButton.setEnabled(false);
					iterationsField.setText("100");
				} else if (subgroupsButton.isSelected()) {
					individualResults = false;
					subgroupResults = true;

					gridbasedButton.setEnabled(true);
					iterationsField.setText("10000");
				} else {
					individualResults = false;
					subgroupResults = false;

					gridbasedButton.setEnabled(true);
					iterationsField.setText("10000");
				}

				// result storage
				if (averagesdButton.isSelected())
					storeRealisations = false;
				else
					storeRealisations = true;
			}
		});
	}

	public void addVisButtonListener(JRadioButton r) {
		r.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// String change = e.getActionCommand();
				// System.out.println(change);

				// visualisation of results
				if (quantilesButton.isSelected()) {
					writeQuantiles = true;
					quantilesField.setEnabled(true);
				} else {
					writeQuantiles = false;
					quantilesField.setEnabled(false);
				}
			}

		});
	}

	/** Listens to the data source radio buttons. */
	public void addDataButtonListener(JRadioButton r) {
		r.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String cmd = e.getActionCommand();
				if (cmd.equals("fillDB"))
					log.append("INFO: This will delete all existing (result) "
							+ "tables in the database!" + newline);

				// System.out.println(change);

				// get boolean values for simulation
				if (csvButton.isSelected()) {
					readDB = false;
					fillDB = false;
				} else if (dbButton.isSelected()) {
					readDB = true;
					fillDB = false;
				} else if (fillDBButton.isSelected()) {
					readDB = false;
					fillDB = true;
				}

				if (writeCsvButton.isSelected()) {
					writeDB = false;
				} else if (writeDBButton.isSelected()) {
					writeDB = true;
				}
			}
		});
	}

	/** Listens to the csv filechooser. */
	private void addOpenButtonListener(JButton b) {
		b.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// Handle open button action.
				if (e.getSource() == openButton) {
					int returnVal = fc.showOpenDialog(frame);

					if (returnVal == JFileChooser.APPROVE_OPTION) {
						File file = fc.getSelectedFile();
						LAMAfolder = file.getPath();
						// selFolder.setText(LAMAfolder);
						log.append("Selected LAMA path: " + LAMAfolder
								+ newline);

						// check if folder is correct and get countries,
						// pollutants and scenarios
						log.append("Checking LAMA folder..." + newline);
						if (checkLAMAFolder()) {
							log.append("LAMA folder is correct.");
							folderCorrect = true;
							folderChecked = true;
						} else {
							log
									.append("LAMA folder is not correct."
											+ newline
											+ "Please check your inputs and try again.");
							folderChecked = true;
							folderCorrect = false;
						}
					}
				}
			}
		});
	}

	/** Listens to the ok button. */
	private void addOKButtonListener(JButton b) {
		b.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String cmd = e.getActionCommand();
				// for startPanel
				if (cmd.equals("1")) {
					getInputDataSources(); // get input from radio buttons

					// check if options are correct
					if (writeDB && !fillDB && !readDB) {
						log
								.append("Writing into database is only possible if database has been initialised. "
										+ newline
										+ "Please change your options.");
					} else {
						// next frame
						frame.dispose();
						// if LAMA folder is C:\LAMA skip step 2
						if (folderCorrect && folderChecked) {
							if (fillDB || readDB || writeDB)
								setupDatabasePanel("2");
							else {
								setupFinalPanel();
								log.append("Skipped step 2" + newline
										+ "LAMA path is: " + LAMAfolder
										+ newline);
							}

						} else {
							if (fillDB)
								setupCsvPanel("2a");
							else if (readDB || writeDB)
								setupDatabasePanel("2");
							else
								setupCsvPanel("2");
						}
					}
				}
				// for csvPanel
				else if (cmd.equals("2a")) {
					// if folder was checked and correct proceed
					if (folderCorrect && folderChecked) {
						// next frame
						frame.dispose();
						if (fillDB)
							setupDatabasePanel("2b");
						else if (!readDB && !writeDB)
							setupFinalPanel();
					}
					// if folder was checked and is not correct wait
					else if (!folderCorrect && folderChecked) {
						log
								.append("LAMA folder is not correctly set or missing input files."
										+ newline
										+ "Please check your input files and try again."
										+ newline);
						log.append(errorMessage);
					}
					// if folder was not checked, do so now
					else if (folderChecked == false) { // if no folder was
						// selected
						log.append("Using default folder C:\\LAMA" + newline);
						log.append("Checking LAMA folder..." + newline);
						if (checkLAMAFolder()) {
							log.append("LAMA folder is correct." + newline
									+ "Click Next to enter the next step.");
							folderCorrect = true;
							folderChecked = true;
						} else {
							log
									.append("LAMA folder is not correct or missing input files."
											+ newline
											+ "Please check your input files and try again.");
							log.append(errorMessage);
						}
					}
				}
				// for dbPanel
				else if (cmd.equals("2b")) {
					if (getInputDB()) // get input from text fields before close
					{
						// check database connection and get countries,
						// pollutants
						// and scenarios
						log.append("Checking database connection..." + newline);
						if (checkDBconnection()) { // only proceed if connection
							// works
							log.append("Database connection works.");
							sleep(1000);
							// next frame
							frame.dispose();
							setupFinalPanel();
						} else
							log
									.append("Database connection could not be established."
											+ newline
											+ "Please check your inputs and try again.");
					} else {
						log
								.append("Please select a database driver!"
										+ newline);
					}
				}
				// For final panel: start simulation
				else if (cmd.equals("3")) {
					getInputFinal();
					// log.append("Initiating model parameters for "+country_id+"."+newline);

					// for whole EU start loop
					if (country_id.equals("EU")) {
						// set visualisation parameter to show whole EU map
						countryWise = false;
						// initiate visualisation parameters
						vis = new VisualisationParameters(visualisation,
								startAguila, writeQuantiles);
						if (writeQuantiles)
							vis.set_quantile_levels(quantiles);
						vis.set_countryWise_visualisation(countryWise);

						// loop through scenarios if necessary
						if (scenario_id.equals("all")) {
							for (int j = 0; j < scenarioIDList.length; j++) {
								scenario_id = scenarioIDList[j];
								for (int i = 0; i < countryIDList.length; i++) {
									if (!countryIDList[i].equals("EU")) {
										country_id = countryIDList[i];
										logFinal.append("Start model run for "
												+ country_id + newline);
										initiateModel();
									}
								}
							}
						} else {
							for (int i = 0; i < countryIDList.length; i++) {
								if (!countryIDList[i].equals("EU")) {
									country_id = countryIDList[i];
									logFinal.append("Start model run for "
											+ country_id + newline);
									initiateModel();
								}
							}
						}

					} else {
						// loop through scenarios if necessary
						if (scenario_id.equals("all")) {
							for (int j = 0; j < scenarioIDList.length; j++) {
								scenario_id = scenarioIDList[j];
								vis = new VisualisationParameters(
										visualisation, startAguila,
										writeQuantiles);
								if (writeQuantiles)
									vis.set_quantile_levels(quantiles);
								vis.set_countryWise_visualisation(countryWise);

								initiateModel();
							}
						} else {
							vis = new VisualisationParameters(visualisation,
									startAguila, writeQuantiles);
							if (writeQuantiles)
								vis.set_quantile_levels(quantiles);
							vis.set_countryWise_visualisation(countryWise);

							initiateModel();
						}
					}
				}
			}
		});
	}

	private void addCancelButtonListener(JButton b) {
		b.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				System.out.println("Operation cancelled.");
				frame.dispose();
			}
		});
	}

	// public void add_log(String logmsg) {
	// log.append(logmsg);
	// }

	private void sleep(int ms) {
		try {
			Thread.currentThread();
			Thread.sleep(ms);// sleep for x ms
		} catch (InterruptedException ie) {
		}
	}

	/*
	 * public static void main(String[] args) {
	 * 
	 * GUI gui = new GUI(); gui.showGUI();
	 * 
	 * }
	 */
}
